Package: UComp
Type: Package
Title: Automatic Univariate Time Series Modelling of many Kinds
Version: 5.3.1
Date: 2026-05-06
Author: Diego J. Pedregal [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4958-0969>)
Authors@R: c(person("Diego J. Pedregal", role = c("aut", "cre"),
                     email = "diego.pedregal@uclm.es",
                     comment = c(ORCID = "0000-0003-4958-0969")))
Maintainer: Diego J. Pedregal <diego.pedregal@uclm.es>
Description: Comprehensive analysis and forecasting 
             of univariate time series using automatic 
             time series models of many kinds.
             Harvey AC (1989) <doi:10.1017/CBO9781107049994>.
             Pedregal DJ and Young PC (2002) <doi:10.1002/9780470996430>.
             Durbin J and Koopman SJ (2012) <doi:10.1093/acprof:oso/9780199641178.001.0001>.
             Hyndman RJ, Koehler AB, Ord JK, and Snyder RD (2008) <doi:10.1007/978-3-540-71918-2>.
             Gómez V, Maravall A (2000) <doi:10.1002/9781118032978>.
             Pedregal DJ, Trapero JR and Holgado E (2024) <doi:10.1016/j.ijforecast.2023.09.004>.
License: GPL-3
Encoding: UTF-8
Imports: ggplot2, gridExtra, tsibble, tsoutliers, stats, ggforce,
        utils, parallel
LinkingTo: Rcpp, RcppArmadillo
Depends: Rcpp (>= 1.0.3), R (>= 3.5.0)
LazyData: true
Suggests: knitr, rmarkdown
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-05-04 07:55:04 UTC; diego.pedregal
Repository: CRAN
Date/Publication: 2026-05-04 08:20:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-04 23:09:25 UTC; unix
Archs: UComp.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(AIC,UComp)
S3method(BIC,UComp)
S3method(coef,UComp)
S3method(fitted,ETS)
S3method(fitted,TETS)
S3method(fitted,UComp)
S3method(logLik,UComp)
S3method(plot,ARIMA)
S3method(plot,ETS)
S3method(plot,TETS)
S3method(plot,UComp)
S3method(predict,UComp)
S3method(print,ARIMA)
S3method(print,ETS)
S3method(print,TETS)
S3method(print,UComp)
S3method(residuals,ARIMA)
S3method(residuals,ETS)
S3method(residuals,TETS)
S3method(residuals,UComp)
S3method(summary,ARIMA)
S3method(summary,ETS)
S3method(summary,TETS)
S3method(summary,UComp)
S3method(tsdiag,UComp)
export(ARIMA)
export(ARIMAestim)
export(ARIMAforecast)
export(ARIMAsetup)
export(ARIMAvalidate)
export(Accuracy)
export(DAYforecast)
export(ETS)
export(ETScomponents)
export(ETSestim)
export(ETSforecast)
export(ETSsetup)
export(ETSvalidate)
export(TETS)
export(TETScomponents)
export(TETSestim)
export(TETSforecast)
export(TETSsetup)
export(TETSvalidate)
export(UC)
export(UCcommand)
export(UCcomponents)
export(UCdisturb)
export(UCestim)
export(UCfilter)
export(UCforecast)
export(UChp)
export(UCsetup)
export(UCsmooth)
export(UCvalidate)
export(WEEKforecast)
export(acft)
export(arma2tsi)
export(armaFilter)
export(auxInvBoxCox)
export(box.cox)
export(colMedians)
export(conv)
export(cusum)
export(dif)
export(extract)
export(gaussTest)
export(getp0)
export(ident)
export(inv.box.cox)
export(invBoxCox)
export(plotAcfPacf)
export(plotBar)
export(plotSlide)
export(plus_one)
export(removeNaNs)
export(roots)
export(rowMedians)
export(size)
export(slide)
export(slideAux)
export(sumStats)
export(tests)
export(tsDisplay)
export(varTest)
export(zplane)
useDynLib(UComp, .registration=TRUE)
importFrom(Rcpp, evalCpp)
import(stats)
import(utils)
import(tsoutliers)
import(ggplot2)
import(gridExtra)
import(tsibble)
import(parallel)
import(ggforce)

#' UComp
#' 
#' 
#' Package for time series modelling and forecasting of 
#' times series models inspired on different sources:
#' 
#' \itemize{
#' \item Unobserved Components models due to A.C. Harvey (Basic Structural Model: BSM), enhanced with automatic identification 
#' tools by Diego J. Pedregal. 
#' \item ExponenTial Smoothing by R.J. Hyndman and colaborators.
#' \item ARIMA models by V. Gómez and A. Maravall
#' \item Tobit ETS models by Pedregal, Trapero and Holgado
#' }
#' 
#' The package is designed for automatic identification 
#' among a wide range of possible models. The models may include exogenous variables. ARMA irregular components 
#' and automatic detection of outliers in some instances.
#' 
#' @section References:
#' Harvey AC (1989). Forecasting, Structural Time Series Models and the Kalman Filter. Cam- bridge University Press.
#' 
#' de Jong, P & Penzer, J (1998). Diagnosing Shocks in Time Series, Journal of the American Statistical Association, 93, 442, 796-806.
#' 
#' Pedregal, DJ, & Young, PC (2002). Statistical approaches to modelling and forecasting time series. In M. Clements,
#' & D. Hendry (Eds.), Companion to economic forecasting (pp. 69–104). Oxford: Blackwell Publishers.
#' 
#' Durbin J, Koopman SJ (2012). Time Series Analysis by State Space Methods. 38. Oxford University Press.
#' 
#' Proietti T and Luati A (2013). Maximum likelihood estimation of time series models: the Kalman filter and beyond, in Handbook of research methods and applications in empirical macroeconomics, ed. Nigar Hashimzade and Michael Thornton, E. Elgar, UK.
#'
#' Hyndman RJ, Koehler AB, Ord JK and Snyder RD (2008), Forecasting with exponential smoothing, The State Sapce approach,
#' Berlin, Springer-Verlag.
#' 
#' Gómez V and Maravall, A (2000), Automatic methods for univariate series. In Peña, D., Tiao, G.C. and Tsay R.S., A course in time series analyis. Wiley.
#' 
#' Trapero JR, Holgado E, Pedregal DJ (2024), Demand forecasting under lost sales stock policies, International Journal of Forecasting, 40, 1055-1068.
#' 
#' @name UComp
#' @aliases UComp-package
#' @author Diego J. Pedregal
#' 
#' @section Maintainer: Diego J. Pedregal
NULL

#' @title size
#' @description Size of vector, matrix or array
#'
#' @param y a vector, matrix or array
#'
#' @return A vector with all the dimensions
#'
#' @author Diego J. Pedregal
#'
#' @seealso \code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
#'          \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
#'          \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
#'          \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
#'          \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
#'          \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}}
#'
#' @examples
#' s <- size(matrix(4, 3, 2))
#' s <- size(rep(4, 3))
#' s <- size(array(4, c(3, 2, 2)))
#' @rdname size
#' @export
size = function(y){
  # Size of any vector or matrix
  out = dim(y)
  if (is.null(out))
    out = length(y)
  return(out)
}
#' @title plus_one
#' @description Returns date of next to end time series y
#'
#' @param y a ts object
#'
#' @return Next time stamp
#'
#' @author Diego J. Pedregal
#'
#' @rdname plus_one
#' @export
plus_one = function(y){
  # Returns date of next to end time series y
  # y is a ts object
  return(end(ts(c(1, 2), start=end(y), frequency=frequency(y))))
}
#' @title extract
#' @description Reorder data frame returning column col reordered
#' according to the values in column accordingTo
#'
#' @param x a data frame
#' @param col column to be ordered
#' @param accordingTo column to take as the pattern
#'
#' @return Data frame reordered accoring to a given column data
#'
#' @author Diego J. Pedregal
#'
#' @rdname extract
#' @export
extract = function(x, col, accordingTo = 1){
  # Reorder data frame returning column col reordered according to
  # the values in column accordingTo
  y = as.data.frame(x)
  values = unique(y[, accordingTo])
  n = nrow(y)
  out = matrix(y[, col], n / length(values), length(values))
  colnames(out) = values
  return(out)
}
#' @title colMedians
#' @description Medians of matrix by columns
#'
#' @param x a matrix
#' @param na.rm boolean indicating whether to remove nans
#' @param ... rest of inputs
#'
#' @return A vector with all the medians in columns
#'
#' @author Diego J. Pedregal
#'
#' @seealso \code{\link{rowMedians}}, \code{\link{tests}},
#'          \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
#'          \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
#'          \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
#'          \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
#'          \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
#'          \code{\link{size}}
#'
#' @examples
#' s <- colMedians(matrix(4, 3, 2))
#' @rdname colMedians
#' @export
colMedians = function(x, na.rm = TRUE, ...){
  # Median of matrix columns
  colx = dim(x)[2]
  out = rep(NA, colx)
  for (i in 1 : colx){
    out[i] = median(x[, i], na.rm = na.rm, ...)
  }
  return(out)
}
#' @title rowMedians
#' @description Medians of matrix by rows
#'
#' @param x a matrix
#' @param na.rm boolean indicating whether to remove nans
#' @param ... rest of inputs
#'
#' @return A vector with all the medians in rows
#'
#' @author Diego J. Pedregal
#'
#' @seealso \code{\link{colMedians}}, \code{\link{tests}},
#'          \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
#'          \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
#'          \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
#'          \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
#'          \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
#'          \code{\link{size}}
#'
#' @examples
#' s <- rowMedians(matrix(4, 3, 2))
#' @rdname rowMedians
#' @export
rowMedians = function(x, na.rm = TRUE, ...){
  # Median of matrix rows
  return(colMedians(t(x), na.rm = na.rm, ...))
}
#' @title removeNaNs
#' @description Remove nans at beginning or end of vector
#'
#' @param x a vector or a ts object
#'
#' @return vector with nans removed (only those at beginning or end)
#'
#' @author Diego J. Pedregal
#'
#' @rdname removeNaNs
#' @export
removeNaNs = function(x){
    x = as.ts(x)
    t = time(x)
    ind = which(!is.na(x))
    ind1 = min(ind)
    ind2 = max(ind)
    return(ts(x[ind1 : ind2], start = t[ind1], frequency = frequency(x)))
}
#' @title tests
#' @description Tests on a time series
#'
#' @details Multiple tests on a time series, including summary statistics,
#' autocorrelation, Gaussianity and heteroskedasticity,
#'
#' @param y a vector, ts or tsibble object
#' @param parts proportion of sample to include in ratio of variances test
#' @param nCoef number of autocorrelation coefficients to estimate
#' @param nPar number of parameters in a model if y is a residual
#' @param s seasonal period, number of observations per year
#' @param avoid number of observations to avoid at beginning of sample to
#' eliminate initial effects
#'
#' @return Table with all test results
#' 
#' @author Diego J. Pedregal
#'
#' @seealso \code{\link{colMedians}}, \code{\link{rowMedians}},
#'          \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
#'          \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
#'          \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
#'          \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
#'          \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
#'          \code{\link{size}}
#'
#' @examples
#' tests(AirPassengers)
#' @rdname tests
#' @export
tests = function(y,
                 parts = 1 / 3,
                 nCoef = min(25, length(x) / 4),
                 nPar = 0,
                 s = frequency(y), avoid = 16){
  # Statistical tests for time series
  if (is.list(y) && !is_tsibble(y)){
      x = tail(as.ts(residuals(y)), -avoid)
      s = frequency(x)
  } else {
      x = as.ts(y)
  }
  #x = removeNaNs(x)
  sumS = capture.output(sumStats(x))
  nCoef = max(floor(nCoef), 1)
  LB = ident(x, nCoef, nPar, TRUE)
  LBs = capture.output(LB)
  pLB = plotAcfPacf(LB$SACF, LB$SPACF, s, length(x), TRUE)
  pGAUSS = gaussTest(x, TRUE)
  pGs = capture.output(pGAUSS[[3]])
  pCUSUM = cusum(x, TRUE)
  pVar = varTest(x, parts)
  pVars = capture.output(pVar)
  out = c("Summary statistics:\n",
              "==================\n",
              sumS,
              "\n",
              "Autocorrelation tests:\n",
              "=====================\n",
              LBs,
              "\n",
              "Gaussianity tests:\n",
              "=================\n",
              pGs,
              "\n",
              "Ratio of variance tests:\n",
              "=======================\n",
              pVars)
  for (i in 1 : length(out)){
      if (substring(out[i], nchar(out[i])) != "\n"){
          out[i] = paste0(out[i], "\n")
      }
  }
  # cat("Summary statistics:\n")
  # cat("==================\n")
  # print(sumStats(x))
  # cat("Autocorrelation tests:\n")
  # cat("=====================\n")
  # nCoef = max(floor(nCoef), 1)
  # LB = ident(x, nCoef, nPar, TRUE)
  # print(LB)
  # pLB = plotAcfPacf(LB$SACF, LB$SPACF, s, length(x), TRUE)
  # cat("Gaussianity tests:\n")
  # cat("=================\n")
  # pGAUSS = gaussTest(x, TRUE)
  # cat("Ratio of variance tests:\n")
  # cat("=======================\n")
  # print(varTest(x, parts))
  # pCUSUM = cusum(x, TRUE)
  # Plotting
  p1 = autoplot(x)
  grid.arrange(
    p1, pLB[[1]], pLB[[2]], pGAUSS[[1]],
    pGAUSS[[2]], pCUSUM[[1]], pCUSUM[[2]],
    widths = c(1, 1),
    layout_matrix = rbind(c(1, 1),
                          c(1, 1),
                          c(2, 3),
                          c(4, 5),
                          c(6, 7))
  )
  return(out)
}
#' @title tsDisplay
#' @description Displays time series plot with autocorrelation functions
#'
#' @param y a vector, ts or tsibble object
#' @param nCoef number of autocorrelation coefficients to estimate
#' @param nPar number of parameters in a model if y is a residual
#' @param s seasonal period, number of observations per year
#'
#' @return No return value, called for side effects
#' 
#' @author Diego J. Pedregal
#'
#' @seealso \code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
#'          \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
#'          \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
#'          \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
#'          \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
#'          \code{\link{plotSlide}}, \code{\link{Accuracy}},
#'          \code{\link{size}}
#'
#' @examples
#' tsDisplay(AirPassengers)
#' @rdname tsDisplay
#' @export
tsDisplay = function(y, nCoef = 25, nPar = 0, s = NA){
  # Plots time series and ACF and PACF
  if (is.list(y) && !is_tsibble(y)){
      x = as.ts(residuals(y))
      s = frequency(x)
  } else {
      x = as.ts(y)
  }
  LB = ident(x, nCoef, nPar, TRUE)
  pLB = plotAcfPacf(LB$SACF, LB$SPACF, frequency(x), length(x), TRUE)
  p1 = autoplot(x)
  grid.arrange(
    p1, pLB[[1]], pLB[[2]],
    widths = c(1, 1),
    layout_matrix = rbind(c(1, 1),
                          c(2, 3))
  )
}
#' @title sumStats
#' @description Summary statistics of a matrix of variables
#'
#' @details Position, dispersion, skewness, kurtosis, etc.
#'
#' @param y a vector, matrix of time series
#' @param decimals number of decimals for table
#'
#' @return Table of values in string matrix
#'
#' @author Diego J. Pedregal
#'
#' @seealso \code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
#'          \code{\link{gaussTest}}, \code{\link{ident}},
#'          \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
#'          \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
#'          \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
#'          \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
#'          \code{\link{size}}
#'
#' @examples
#' s <- sumStats(AirPassengers)
#' @rdname sumStats
#' @export
sumStats = function(y, decimals = 5){
  # Summary statistics of variables
  if (is.list(y) && !is_tsibble(y)){
      x = as.ts(residuals(y))
  } else {
      x = as.ts(y)
  }
  x = removeNaNs(x)
  q25 = quantile(x, 0.25, na.rm = TRUE)
  q75 = quantile(x, 0.75, na.rm = TRUE)
  maxx = max(x, na.rm = TRUE)
  minx = min(x, na.rm = TRUE)
  meanx = mean(x, na.rm = TRUE)
  sdx = sd(x, na.rm = TRUE)
  ind = !is.na(x)
  n = sum(ind)
  m1 = x[ind] - meanx
  m2 = sum(m1 ^ 2)
  skewness = (sum(m1 ^ 3) / n) / ((m2 / n) ^ (3 / 2))
  kurtosis = (sum(m1 ^ 4) / n) / ((m2 / n) ^ 2) - 3
  aux = c(length(y),
          sum(is.na(y)),
          minx,
          q25,
          meanx,
          2 * pt(-abs(meanx / (sdx / sqrt(n))), n - 1),
          median(x, na.rm = TRUE),
          q75,
          maxx,
          q75 - q25,
          maxx - minx,
          sdx,
          sdx * sdx,
          skewness,
          kurtosis)
  out = round(matrix(aux, 15, 1), decimals)
  colnames(out) = "Serie 1"
  rownames(out) = c("Data points: ",
                    "Missing: ",
                    "Minimum: ",
                    "1st quartile: ",
                    "Mean: ",
                    "P(Mean = 0): ",
                    "Median: ",
                    "3rd quartile:",
                    "Maximum: ",
                    "Interquartile range: ",
                    "Range: ",
                    "Satandard deviation: ",
                    "Variance: ",
                    "Skewness: ",
                    "Kurtosis: ")
  return(out)
}
#' @title gaussTest
#' @description Gaussianity tests
#'
#' @param y a vector, ts or tsibble object
#' @param runFromTests internal check
#'
#' @return No return value, called for side effects
#' 
#' @author Diego J. Pedregal
#'
#' @seealso \code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
#'          \code{\link{sumStats}}, \code{\link{ident}},
#'          \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
#'          \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
#'          \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
#'          \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
#'          \code{\link{size}}
#'
#' @examples
#' gaussTest(AirPassengers)
#' @rdname gaussTest
#' @export
gaussTest = function(y, runFromTests = FALSE){
  # Gaussianity test for independent time series
  if (is.list(y) && !is_tsibble(y)){
      x = as.ts(residuals(y))
  } else {
      x = as.ts(y)
  }
  x = x[!is.na(x)]
  n = length(x)
  df = data.frame(x)
  p1 = ggplot(df, aes(x = x)) +
          geom_histogram(aes(y = after_stat(density)),
                         bins = floor(sqrt(n)), color = "#000000", fill = "#0099F8") +
          geom_density(lwd = 1, color = "#000000", fill = "#F85700", alpha = 0.6) +
          stat_function(fun = dnorm, args = list(mean = mean(df$x), sd = sd(df$x)))
  # QQplot
  p2 = ggplot(df, aes(sample = x)) + stat_qq() + stat_qq_line(color = "darkred", linewidth = 1)
  shap = shapiro.test(x)
  if (runFromTests){
    return(list(p1, p2, shap))
  } else {
    grid.arrange(p1, p2, nrow = 2)
  }
}
#' @title ident
#' @description Autocorrelation functions of a time series
#'
#' @param y a vector, ts or tsibble object
#' @param nCoef number of autocorrelation coefficients to estimate
#' @param nPar number of parameters in a model if y is a residual
#' @param runFromTests internal check
#'
#' @return A vector with output table including ACF, etc.
#'
#' @author Diego J. Pedregal
#'
#' @seealso \code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
#'          \code{\link{sumStats}}, \code{\link{gaussTest}},
#'          \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
#'          \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
#'          \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
#'          \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
#'          \code{\link{size}}
#'
#' @examples
#' ident(AirPassengers)
#' @rdname ident
#' @export
ident = function(y,
                 nCoef = min(37, floor(length(y) / 4)),
                 nPar = 0,
                 runFromTests = FALSE){
  # ACF and PACF with Ljung Box tests and plots
  if (is.list(y) && !is_tsibble(y)){
      x = as.ts(residuals(y))
      s = frequency(x)
  } else {
      x = as.ts(y)
  }
  nCoef = max(nCoef, frequency(x))
  x = removeNaNs(x)
  x = x - mean(x, na.rm = TRUE)
  aux = matrix(0, nCoef + 1, 1)
  n = length(x)
  for (i in 0 : nCoef){
      prod = x[1 : (n - i)] * x[(i + 1) : n]
      ind = which(!is.na(prod))
      aux[i + 1, 1] = sum(prod[ind])
  }
  aux = aux / aux[1]
  ACF = as.numeric(tail(aux, nCoef))
  # PACF from ACF
  PACF <- ACF
  fi <- numeric(nCoef)
  fi[1] <- ACF[1]
  for (i in 1:(nCoef - 1)) {
      fi[i + 1] <- (ACF[i + 1] - sum(fi[1:i] * rev(ACF[1:i]))) / (1 - sum(fi * ACF[1:length(fi)]))
      PACF[i + 1] <- fi[i + 1]
      fi[1:i] <- fi[1:i] - fi[i + 1] * rev(fi[1:i])
  }
  #PACF = as.numeric(pacf(x, nCoef, plot = FALSE)$acf)
  BAND = rep(2 / sqrt(length(x)), nCoef)
  SIGa = rep(".", nCoef)
  SIGa[ACF > BAND] = "+"
  SIGa[ACF < -BAND] = "-"
  SIGp = matrix(".", nCoef)
  SIGp[PACF > BAND] = "+"
  SIGp[PACF < -BAND] = "-"
  BOX = rep(NA, nCoef)
  pval = BOX
  for (i in 1 : nCoef){
    ci = Box.test(x, i, "Ljung-Box", nPar)
    BOX[i] = ci$statistic
    pval[i] = ci$p.value
  }
  out = data.frame(SACF = round(ACF, 3), sa = SIGa,
                   LB = round(BOX, 3), p.val = round(pval, 3),
                   SPACF = round(PACF, 3), sp = SIGp)
  if (runFromTests){
    return(out)
  } else {
    plotAcfPacf(ACF, PACF, frequency(x), length(x))
  }
  return(out)
}
#' @title plotBar
#' @description Plot variable in bars
#'
#' @param ACF variable to plot
#' @param s seasonal period
#' @param n number of coefficients
#' @param label label for plot
#'
#' @return Handle of plot
#'
#' @author Diego J. Pedregal
#'
#' @rdname plotBar
#' @export
plotBar = function(ACF, s = 1, n = NA, label = "ACF"){
  # Plot of ACF & PACF
  ncoef = length(ACF)
  band = 2 / sqrt(n)
  ACFs = rep(0, ncoef)
  ACFs[seq(s, ncoef, s)] = ACF[seq(s, ncoef, s)]
  df = data.frame(lag = seq(1, length(ACF)), ACF, ACFs)
  p = ggplot(df, aes(x = lag, y = ACF)) +
    geom_hline(aes(yintercept = 0)) +
    geom_segment(aes(xend = lag, yend = 0)) +
    labs(y = label) + theme(legend.position = "none")
  if (!is.na(n)){
    p = p +
      geom_hline(aes(yintercept = band), linetype = 2, color = 'red') +
      geom_hline(aes(yintercept = -band), linetype = 2, color = 'red')
  }
  if (s > 1){
    p = p + geom_segment(data = df,
                         aes(y = ACFs, xend = lag, yend = 0, color = "red"))
  }
  return(p)
}
#' @title plotAcfPacf
#' @description Plot of ACF and PACF
#'
#' @param ACF variable to plot
#' @param PACF second variable to plot
#' @param s seasonal period
#' @param n number of coefficients
#' @param runFromTest internal check variable
#'
#' @return No return value, called for side effects
#' 
#' @author Diego J. Pedregal
#'
#' @rdname plotAcfPacf
#' @export
plotAcfPacf = function(ACF, PACF, s = 1, n = NA, runFromTest = FALSE){
  # Plot of ACF & PACF
  p1 = plotBar(ACF, s, n)
  p2 = plotBar(PACF, s, n, "PACF")
  if (runFromTest){
    return(list(p1, p2))
  } else {
      grid.arrange(p1, p2, nrow = 2)
  }
}
#' @title cusum
#' @description Cusum and cusumsq tests
#'
#' @param y a vector, ts or tsibble object
#' @param runFromTest internal check variable
#'
#' @return No return value, called for side effects
#' 
#' @author Diego J. Pedregal
#'
#' @seealso \code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
#'          \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
#'          \code{\link{varTest}}, \code{\link{conv}},
#'          \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
#'          \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
#'          \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
#'          \code{\link{size}}
#'
#' @examples
#' cusum(AirPassengers)
#' @rdname cusum
#' @export
cusum = function(y, runFromTest = FALSE){
  # CUSUM and CUSUMsq tests
  if (is.list(y) && !is_tsibble(y)){
      x = as.ts(residuals(y))
  } else {
      x = as.ts(y)
  }
  x = removeNaNs(x)
  n = length(x)
  mz = mean(x, na.rm = TRUE)
  stdz = sd(x, na.rm = TRUE)
  NMiss = !is.na(x)
  Miss = is.na(x)
  standz = matrix(NA, n, 1)
  cusumLine = standz
  cusumsq = standz
  NMissi = which(NMiss)
  zmi = x[NMissi] - mz
  standz[NMissi] = zmi / stdz
  cusumLine[NMissi] = cumsum(standz[NMissi])
  cusumsq[NMissi] = cumsum(zmi ^ 2 / (sum(zmi ^ 2)))
  # Calculating bands
  t = 1 : n
  bands= matrix(NA, n, 2)
  bands[, 1] = c(0.948 * sqrt(n) + 2 * 0.948 * t / sqrt(n))
  bands[, 2]= -bands[, 1]
  # Plotting CUSUM
  miny = min(min(bands, na.rm = TRUE), min(cusumLine, na.rm = TRUE))
  maxy = max(max(bands, na.rm = TRUE), max(cusumLine, na.rm = TRUE))
  p1 = autoplot(as.ts(cusumLine)) +
    autolayer(as.ts(bands[, 1]), color = "red", linetype = 2) +
    autolayer(as.ts(bands[, 2]), color = "red", linetype = 2) +
    autolayer(as.ts(rep(0, length(cusumLine))), color = "red") +
    labs(y = "CUSUM")
  # Plotting CUSUMsq
  wu = .948 * sqrt(n) + 2 * .948 * t / sqrt(n)
  bands = cbind(-.32894 + t / n, .32894 + t / n)
  miny = min(min(bands, na.rm = TRUE), min(cusumsq, na.rm = TRUE))
  maxy = max(max(bands, na.rm = TRUE), max(cusumsq, na.rm = TRUE))
  p2 = autoplot(as.ts(cusumsq)) +
    autolayer(as.ts(bands[, 1]), color = "red", linetype = 2) +
    autolayer(as.ts(bands[, 2]), color = "red", linetype = 2) +
    autolayer(as.ts(t / length(cusumLine)), color = "red") +
    labs(y = "CUSUMsq")
  if (runFromTest){
    return(list(p1, p2))
  } else {
    grid.arrange(p1, p2, nrow = 2)
  }
}
#' @title varTest
#' @description Ratio of variances test
#'
#' @param y a vector, ts or tsibble object
#' @param parts portion of sample to estimate variances
#'
#' @return Table with test results
#'
#' @author Diego J. Pedregal
#'
#' @seealso \code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
#'          \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
#'          \code{\link{cusum}}, \code{\link{conv}},
#'          \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
#'          \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
#'          \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
#'          \code{\link{size}}
#'
#' @examples
#' varTest(AirPassengers)
#' @rdname varTest
#' @export
varTest = function(y, parts = 1 / 3){
  # Ratio of varainces tests
  if (is.list(y) && !is_tsibble(y)){
      x = as.ts(residuals(y))
  } else {
      x = as.ts(y)
  }
  x = removeNaNs(x)
  n = length(x)
  n1 = floor(n * parts)
  aux = var.test(head(x, n1), tail(x, n1))
  out = data.frame(Portion_of_data = paste(round(parts, 5)),
                   F_statistic = round(aux$statistic, 4),
                   p.value = round(aux$p.value, 4))
  rownames(out) = ""
  return(out)
}
#' @title conv
#' @description 1D convolution: filtering or polynomial multiplication
#'
#' @param ... list of vectors to convolute
#'
#' @return Convolution of all input vectors
#'
#' @author Diego J. Pedregal
#'
#' @seealso \code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
#'          \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
#'          \code{\link{cusum}}, \code{\link{varTest}},
#'          \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
#'          \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
#'          \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
#'          \code{\link{size}}
#'
#' @examples
#' conv(c(1, -1), c(1, -2, 1))
#' conv(c(1, -1), c(1, 0.8))
#' @rdname conv
#' @export
conv = function(...) {
  # 1D convolution: filtering or polynomial multiplication
  args = list(...)
  n = ...length()
  if (n > 2){
    return(conv(args[[1]], do.call(conv, args[2 : n])))
  } else {
    # Convolution of n vectors
    #This function convolves two vectors x and y
    #x and y must be numeric vectors, length at least 1
    #z=x*y is returned
    #L. Schweitzer A4, 1
    lx = length(args[[1]])
    ly = length(args[[2]])
    lz = lx + ly - 1
    z = 0 * numeric(lz)
    for (i in 1 : lx) {
      # begin convolving x and y
      for (j in 1 : ly) {
        z[(i - 1) + (j - 1) + 1] = z[(i - 1) + (j - 1) + 1] + args[[1]][i] * args[[2]][j]
      }
    } # end convolving x and y
    return(z)
  }
}

#' @title armaFilter
#' @description Filter of time series
#'
#' @param MA numerator polynomial
#' @param AR denominator polynomial
#' @param y a vector, ts or tsibble object
#'
#' @return Filtered time series
#'
#' @author Diego J. Pedregal
#'
#' @seealso \code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
#'          \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
#'          \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
#'          \code{\link{dif}}, \code{\link{roots}},
#'          \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
#'          \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
#'          \code{\link{size}}
#'
#' @examples
#' y <- armaFilter(1, c(1 , -0.8), rnorm(200))
#' @rdname armaFilter
#' @export
armaFilter = function(MA = 1, AR = 1, y){
  # Filtering a time series with linear filter
  xs = as.ts(y)
  q = length(MA)
  p = length(AR)
  x = conv(xs, MA)
  if (p > 1){
    x = stats::filter(x, -AR[2 : length(AR)], "recursive")
    x = ts(x[1 : length(xs)],
           frequency = frequency(xs),
           start = time(xs)[1])
  } else {
    x = ts(x[length(MA) : length(xs)],
           frequency = frequency(xs),
           start = time(xs)[length(MA)])
  }
  if (is_tsibble(x)){
    x = as_tsibble(x)
  }
  return(x)
}
#' @title dif
#' @description Discrete differencing of time series
#'
#' @param y a vector, ts or tsibble object
#' @param difs vector with differencing orders
#' @param seas vector of seasonal periods
#'
#' @return Differenced time series
#'
#' @author Diego J. Pedregal
#'
#' @seealso \code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
#'          \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
#'          \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
#'          \code{\link{armaFilter}}, \code{\link{roots}},
#'          \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
#'          \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
#'          \code{\link{size}}
#'
#' @examples
#' dif(AirPassengers)
#' dif(AirPassengers, 2)
#' dif(AirPassengers, c(1, 1), c(1, 12))
#' @rdname dif
#' @export
dif = function(y, difs = 1, seas = 1){
  # Mixture of differences operating on x
  x = as.ts(y)
  n = length(difs)
  pol = 1
  for (i in 1 : n){
    poli = c(1, rep(0, seas[i] - 1), -1)
    if (difs[i] > 0){
      for (j in 1 : difs[i]){
        pol = conv(pol, poli)
      }
    }
  }
  dx = armaFilter(pol, 1, x)
  if (is_tsibble(x)){
    dx = as_tsibble(dx)
  }
  return(head(dx, (length(x) - length(pol) + 1)))
}
#' @title roots
#' @description Roots of polynomial
#'
#' @param x coefficients of polynomial in descending order
#'
#' @return Roots of polynomial
#'
#' @author Diego J. Pedregal
#'
#' @seealso \code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
#'          \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
#'          \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
#'          \code{\link{armaFilter}}, \code{\link{dif}},
#'          \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
#'          \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
#'          \code{\link{size}}
#'
#' @examples
#' roots(c(1, -2 ,1))
#' roots(conv(c(1, -1), c(1, 0.8)))
#' @rdname roots
#' @export
roots = function(x){
  # Roots of a polynomial
  if (length(x) > 1){
    return(polyroot(rev(x)))
  } else {
    return(NULL)
  }
}
#' @title zplane
#' @description Real-imaginary plane to show roots of digital filters (ARMA)
#'
#' @details Shows the real-imaginary plane to show zeros (roots of numerator or
#' MA polynomial) and poles (roots of denominator of AR polynomial). Unit roots
#' and real vs imaginary roots can be seen by eye
#'
#' @param MApoly coefficients of numerator polynomial in descending order
#' @param ARpoly coefficients of denominator polynomial in descending order
#'
#' @return No return value, called for side effects
#' 
#' @author Diego J. Pedregal
#'
#' @seealso \code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
#'          \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
#'          \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
#'          \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
#'          \code{\link{acft}}, \code{\link{slide}},
#'          \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
#'          \code{\link{size}}
#' @examples
#' zplane(c(1, -2, 1), c(1, -0.8))
#' @rdname zplane
#' @export
zplane = function(MApoly = 1, ARpoly = 1){
  # zplane plot
  # if (is.list(MApoly)){   # MApoly is an ARIMA fable model
  #     aux = glance(MApoly)
  #     arRoots = aux$ar_roots[[1]]
  #     maRoots = aux$ma_roots[[1]]
  #     modN = abs(arRoots) ^ 2
  #     arRoots = complex(real = Re(arRoots) / modN, imaginary = -Im(arRoots) / modN)
  #     modN = abs(maRoots) ^ 2
  #     maRoots = complex(real = Re(maRoots) / modN, imaginary = -Im(maRoots) / modN)
  # } else {
      arRoots = roots(ARpoly)
      maRoots = roots(MApoly)
  # }
  reaux = NA
  imaux = NA
  x = seq(-1, 1, 0.005)
  y = sqrt(1 - x ^ 2)
  # Dibujando círculo
  p = ggplot() +
    geom_circle(aes(x0 = 0, y0 = 0, r = 1)) +
    geom_hline(yintercept=0, linetype="dashed") +
    geom_vline(xintercept=0, linetype="dashed") +
    coord_fixed() +
    labs(x = "Real part", y = "Imaginary part") +
    ggtitle("Poles (x - AR) and zeros (o - MA)")
  # Añadiendo raices
  if (length(arRoots) > 0){
    modAR = abs(arRoots)
    # Stationary
    aux = arRoots[modAR < 0.99]
    aux = data.frame(reaux = Re(aux), imaux = Im(aux))
    if (size(aux)[1] > 0){
      p = p + geom_point(data = aux, aes(x = reaux, y = imaux),
                         shape = "x", size = 4)
    }
    # Non-stationary
    aux = arRoots[modAR >= 0.99]
    aux = data.frame(reaux = Re(aux), imaux = Im(aux))
    if (size(aux)[1] > 0){
      p = p + geom_point(data = aux, aes(x = reaux, y = imaux),
                         shape = "x", size = 4, color = "red")
    }
  }
  if (length(maRoots) > 0){
    # Inverible
    modMA = abs(maRoots)
    aux = maRoots[modMA < 0.99]
    aux = data.frame(reaux = Re(aux), imaux = Im(aux))
    if (size(aux)[1] > 0){
      p = p + geom_point(data = aux, aes(x = reaux, y = imaux),
                         shape = "o", size = 4)
    }
    # Non-invertible
    aux = maRoots[modMA >= 0.99]
    aux = data.frame(reaux = Re(aux), imaux = Im(aux))
    if (size(aux)[1] > 0){
      p = p + geom_point(data = aux, aes(x = reaux, y = imaux),
                         shape = "o", size = 4, color = "red")
    }
  }
  print(p)
}
#' @title arma2tsi
#' @description AR polynomial coefficients of ARMA model
#'
#' @param MApoly coefficients of numerator polynomial in descending order
#' @param ARpoly coefficients of denominator polynomial in descending order
#' @param n number of coefficients
#'
#' @return Tsi (MA form) coefficients of equivalent ARMA model
#' 
#' @author Diego J. Pedregal
#'
#' @rdname arma2tsi
#' @export
arma2tsi = function(MApoly, ARpoly, n = 100){
  # Converts ARMA to pure AR
  return(armaFilter(MApoly, ARpoly, c(1, rep(0, n - 1))))
}
#' @title acft
#' @description Theoretical autocorrelation functions of ARMA models
#'
#' @param MApoly coefficients of numerator polynomial in descending order
#' @param ARpoly coefficients of denominator polynomial in descending order
#' @param ncoef number of coefficients
#' @param s seasonal period, number of observations per year
#'
#' @return Theoretical autocorrelation functions
#'
#' @author Diego J. Pedregal
#'
#' @seealso \code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
#'          \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
#'          \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
#'          \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
#'          \code{\link{zplane}}, \code{\link{slide}},
#'          \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
#'          \code{\link{size}}
#'
#' @examples
#' acft(c(1, -0.8), c(1, 0.8))
#' @rdname acft
#' @export
acft = function(MApoly = 1, ARpoly = 1, ncoef = 38, s = 1){
  # Theoretical ACF and PACF from ARMA model
  correct = TRUE
  if (length(ARpoly) > 1 && any(abs(roots(ARpoly)) >= 1)){
    warning('Non-stationary model')
    correct = FALSE
  }
  if (length(MApoly) > 1 && any(abs(roots(MApoly)) >= 1)){
    warning('Non-invertible model')
    correct = FALSE
  }
  if (!correct){
    stop('SACF and SPACF do not exist for this model!!!')
    return()
  }
  p = size(ARpoly) - 1
  q = size(MApoly) - 1
  out = matrix(1, ncoef, 3)
  out[, 1] = 1 : ncoef
  # ACF
  aux = 250
  MAPi = rep(1, aux)
  MAP = MApoly
  if (p > 0){
    while (MAPi[aux] > 1e-4){
      aux = aux + 100
      MAPi = arma2tsi(MApoly, ARpoly, aux)
    }
    MAP = MAPi
  }
  if (size(MAP) < aux){
    MAP = c(MAP, rep(0, aux - size(MAP)))
  }
  gam0= sum(MAP ^ 2)
  for (i in 1 : ncoef){
    out[i, 2] = sum(MAP[(i + 1) : aux] * MAP[1 : (aux - i)]) / gam0
  }
  # PACF
  fi = rep(0, ncoef)
  fi[1] = out[1, 2]
  out[1, 3] = fi[1]
  for (p in 1 : (ncoef - 1)){
    fi[p + 1] = (out[p + 1, 2] - sum(fi[1 : p] * rev(out[1 : p, 2]))) / (1 - sum(fi * out[, 2]))
    out[p + 1, 3] = fi[p + 1]
    fi[1 : p] = fi[1 : p] - fi[p + 1] * rev(fi[1 : p])
  }
  plotAcfPacf(out[, 2], out[, 3], s)
  colnames(out) = c("lag", "SACF", "SPACF")
  return(out)
}
#' @title slide
#' @description Rolling forecasting of a matrix of time series
#'
#' @details Takes time series and run forecasting methods implemented in function
#' forecFun h steps ahead along the time series y, starting at forecasting
#' origin orig, and moving step observations ahead. Forecasts may be run in parallel
#' by setting parallel to TRUE. A fixed window width may be
#' specified with input window. The output is of dimensions (h, nOrigs, nModels, nSeries)
#'
#' @param y a vector, a matrix or a list of time series
#' @param orig starting forecasting origin
#' @param forecFun user function that implements forecasting methods
#' @param ... rest of inputs to forecFun function
#' @param h forecasting horizon
#' @param step observations ahead to move the forecasting origin
#' @param output output TRUE/FALSE
#' @param window fixed window width in number of observations (NA for non fixed)
#' @param parallel run forecasts in parallel
#'
#' @return An array of forecasts of dimensions (horizon x nOrigs x nModels x nSeries)
#'
#' @author Diego J. Pedregal
#'
#' @seealso \code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
#'          \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
#'          \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
#'          \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
#'          \code{\link{zplane}}, \code{\link{acft}},
#'          \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
#'          \code{\link{size}}
#'
#' @examples
#' \dontrun{slide(AirPassengers, 100, forecFun)}
#' @rdname slide
#' @export
slide <- function(y,
                  orig,
                  forecFun,
                  ...,
                  h = 12,
                  step = 1,
                  output = TRUE,
                  window = NA,
                  parallel = FALSE) {
  # suppressWarnings()
  # Preparación de datos
  yisList <- is.list(y)
  yisListList <- if (yisList) is.list(y[[1]]) else FALSE
  if (!yisList) {
    if (!is.ts(y)) y <- as.ts(y)
    if (length(size(y)) == 1) {
      nSeries <- 1
      y <- list(list(y))
    } else {
      nSeries <- ncol(y)
      aux <- frequency(y)
      auxs <- start(y)
      y <- split(y, col(y))
      y <- lapply(y, ts, start = auxs, frequency = aux)
      y <- lapply(y, function(x) list(x))
    }
  } else if (!yisListList) {
    nSeries <- length(y)
    y <- lapply(y, function(x) list(x))
  } else {
    nSeries <- length(y)
  }
  # Detección de tipo de entrada para forecFun
  test1 <- !is.null(tryCatch(forecFun(y[[1]], h, ...), error = function(e) NULL))
  test2 <- !is.null(tryCatch(forecFun(y[[1]][[1]], h, ...), error = function(e) NULL))
  isList <- if (test1 && !test2) TRUE else if (!test1 && test2) FALSE else yisList
  # Ejecución
  if (nSeries == 1) {
    listOut <- lapply(y, slideAux, orig, forecFun, h, step, output, FALSE, window, parallel, isList, ...)
  } else {
    if (parallel) {
      cores <- max(1, parallel::detectCores() - 1)
      cl <- parallel::makeCluster(cores)
      on.exit(parallel::stopCluster(cl))
      parallel::clusterExport(cl, varlist = c("slideAux", "orig", "forecFun", "h", "step", "output", "window", "isList"), envir = environment())
      listOut <- parallel::parLapply(cl, y, function(x) slideAux(x, orig, forecFun, h, step, output, FALSE, window, FALSE, isList, ...))
    } else {
      listOut <- lapply(y, slideAux, orig, forecFun, h, step, output, FALSE, window, FALSE, isList, ...)
    }
  }
  # Construcción del array de salida
  out <- array(NA, c(dim(listOut[[1]]), nSeries))
  for (i in 1:nSeries) {
    out[, , , i] <- listOut[[i]]
  }
  dimnames(out)[[3]] <- dimnames(listOut[[1]])[[3]]
  # options(warn = 0)
  return(out)
}
#' @title slideAux
#' @description Auxiliary function run from slide
#'
#' @param y a vector or matrix of time series
#' @param orig starting forecasting origin
#' @param forecFun user function that implements forecasting methods
#' @param h forecasting horizon
#' @param step observations ahead to move the forecasting origin
#' @param output output TRUE/FALSE
#' @param graph fraphical output TRUE/FALSE
#' @param window fixed window width in number of observations (NA for non fixed)
#' @param parallel run forecasts in parallel
#' @param isList whether the input data y is a list or a matrix
#' @param ... rest of inputs to forecFun function
#'
#' @return Auxiliary output of slide function for just one time series
#' 
#' @author Diego J. Pedregal
#'
#' @rdname slideAux
#' @export
slideAux = function(y,
                    orig,
                    forecFun,
                    h = 12,
                    step = 1,
                    output = TRUE,
                    graph = TRUE,
                    window = NA,
                    parallel = FALSE,
                    isList = FALSE,
                    ...){
  # Rolling for 1 series
  # out = [h, nOrigs, nModels]
  n = length(y[[1]])
  origs = seq(orig, n - h, step)
  nOr = length(origs)
  # # Detecting whether forecFun accepts list as input
  # isList = FALSE
  # x = names(formals(forecFun))[1]
  # code <- deparse(body(forecFun))
  # firstTest = any(grepl(paste0(x, "\\$"), code))
  # secondTest = any(grepl(paste0(x, "\\[\\["), code))
  # if (firstTest || secondTest){
  #     isList = TRUE
  # }
  # Checking kind of input to function forecFun
  # outi = tryCatch({
  #     forecFun(y, h, ...)  # llamada a la función
  # }, error = function(e) {
  #     return(NULL)  # puedes devolver un valor por defecto si lo deseas
  # })
  # isList = TRUE
  # if (is.null(outi)) {
  #     outi = forecFun(y[[1]], h, ...)
  #     isList = FALSE
  # }
  if (isList) {
    outi = forecFun(y, h, ...)
  } else {
    outi = forecFun(y[[1]], h, ...)
  }
  outi = as.matrix(outi)
  nMethods = dim(outi)[2]
  out = array(NA, c(h, length(origs), nMethods))
  out[, 1, ] = outi
  dimnames(out)[[3]] = colnames(outi)
  dataList = vector(mode = "list", length = nOr - 1)
  if (nOr > 1){
    for (j in 1 : (nOr - 1)){
      if (isList) {
        dataList[[j]] = y
        if (is.na(window)){
          dataList[[j]][[1]] = subset(y[[1]], end = orig + j)
        } else {
          dataList[[j]][[1]] = subset(y[[1]], start = orig - window + step, end = orig + j)
        }
      } else {
        if (is.na(window)){
          dataList[[j]] = subset(y[[1]], end = orig + j)
        } else {
          dataList[[j]] = subset(y[[1]], start = orig - window + step, end = orig + j)
        }
      }
    }
    
  }
  # if (parallel){
  #         listOut = mclapply(dataList, forecFun, h, ..., mc.cores = detectCores())
  #         # if (substr(.Platform$OS.type, 1, 1) == "w"){
  #         #     listOut = parallelsugar::mclapply(dataList, forecFun, h, ..., mc.cores = detectCores())
  #         # } else {
  #         #     listOut = parallel::mclapply(dataList, forecFun, h, ..., mc.cores = detectCores())
  #         # }
  #         
  # } else {
  listOut = lapply(dataList, forecFun, h, ...)
  # }
  if (nOr > 1){
    for (i in 2 : nOr){
      out[, i, ] = as.matrix(listOut[[i - 1]])
    }
  }
  return(out)
}
#' @title plotSlide
#' @description Plot summarised results from slide
#'
#' @param py1 output from slide function
#' @param y a vector, matrix or list of time series (the same used in slide call)
#' @param orig starting forecasting origin (the same used in slide call)
#' @param step observations ahead to move the forecasting origin (the same used in slide call)
#' @param errorFun user function to calculate error measures
#' @param collectFun aggregation function (mean, median, etc.)
#'
#' @return An array of forecasting errors of dimensions (horizon x nOrigs x nModels x nSeries)
#'
#' @author Diego J. Pedregal
#'
#' @seealso \code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
#'          \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
#'          \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
#'          \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
#'          \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
#'          \code{\link{Accuracy}}, \code{\link{tsDisplay}},
#'          \code{\link{size}}
#'
#' @examples
#' \dontrun{plotSlide(py1, AirPassengers, 100, 1, errorFun)}
#' @rdname plotSlide
#' @export
plotSlide = function(py1, y, orig, step = 1,
                     errorFun,
                     collectFun = mean){
        # py1 = [h, nOrigs, nModels, nSeries]
        h = dim(py1)[1]
        nOrigs = dim(py1)[2]
        nMethods = dim(py1)[3]
        nSeries = dim(py1)[4]
        if (is.na(nSeries)){
                nSeries = 1
                py = array(NA, c(dim(py1), 1))
                dimnames(py) = dimnames(py1)
                py[, , , 1] = py1
        } else {
                py = py1
        }
        metrics = matrix(NA, h, nMethods)
        colnames(metrics) = dimnames(py)[[3]]
        outj = array(NA, c(h, nOrigs, nMethods, nSeries))
        # Converting list of lists to just a list of time series
        yisList = is.list(y)
        yisListList = FALSE
        if (yisList)
                yisListList = is.list(y[[1]])
        if (yisListList) {
                y = lapply(y, function(x) x[[1]])
        }
        # End of converting list of lists
        for (i in 1 : nOrigs){
                if (yisList) {
                        actuali = lapply(y, function(serie) {
                                subset(serie, end = orig + (i - 1) * step + h)
                        })
                } else {
                        actuali = subset(y, end = orig + (i - 1) * step + h)
                }
                aux = array(NA, c(h, nMethods, nSeries))
                aux[, , ] = py[, i, , ]
                colnames(aux) = dimnames(py)[[3]]
                if (nSeries == 1){
                        for (j in 1 : nMethods){
                                outj[, i, j, 1] = errorFun(aux[, j, 1], actuali)
                        }
                } else {
                        for (k in 1 : nSeries){
                                for (j in 1 : nMethods){
                                        if (yisList) {
                                                auxx = as.vector(actuali[[k]])
                                        } else {
                                                auxx = as.vector(actuali[, k])
                                        }
                                        outj[, i, j, k] = errorFun(aux[, j, k], auxx)
                                }
                        }
                }
                # outj[, i, , ] = AccuracyAll(aux, actuali, collectFun = collectFun, one = FALSE, )[, , criteriaColumn]
        }
        for (m in 1 : nMethods){
                for (j in 1 : h){
                        metrics[j, m] = collectFun(outj[j, , m, ], na.rm = TRUE)
                }
        }
        plotH = autoplot(ts(metrics, frequency = 1, start = 1), ylab = "")
        print(plotH)
        return(outj)
}
#' @title Accuracy
#' @description Accuracy for 1 time series y and several forecasting
#'  methods py and h steps ahead py is h x nMethods x nSeries
#'
#' @param py matrix of forecasts (h x nMethods x nForecasts)
#' @param y a matrix of actual values (n x nForecasts)
#' @param s seasonal period, number of observations per year
#' @param collectFun aggregation function (mean, median, etc.)
#'
#' @return Table of accuracy results
#'
#' @author Diego J. Pedregal
#'
#' @seealso \code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
#'          \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
#'          \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
#'          \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
#'          \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
#'          \code{\link{plotSlide}}, \code{\link{tsDisplay}},
#'          \code{\link{size}}
#'
#' @examples
#' \dontrun{Accuracy(py, y, 12)}
#' @rdname Accuracy
#' @export
Accuracy = function(py, y, s = frequency(y), collectFun = mean){
    # Accuracy for 1 time series y and several forecasting methods py and h steps ahead
    # py is h x nMethods x nSeries
    # y  is n x nSeries
    spy = length(size(py))
    if (is.ts(py) && is.ts(y)){
        tpy = time(py)
        ty = time(y)
        # if (min(tpy) > max(ty)){
        if (min(tpy) - max(ty) > 1e-5){
            stop("No errors to estimate, check dates!!!")
        }
        # if (max(ty) < max(tpy)){
        if (max(tpy) - max(ty) > 1e-5){
            # Cut forecasts
            # py = head(py, which(max(ty) == tpy))
            py = head(py, which.min(abs(max(ty) - tpy)))
        }
        # if (max(tpy) < max(ty)){
        if (max(ty) - max(tpy) > 1e-5){
            # Cut time series
            # y = head(y, which(max(tpy) == ty))
            y = head(y, which.min(abs(max(tpy) - ty)))
        }
    }
    if (spy == 1){
        stop("Input \'py\' should be: h x number of methods x number of series!!!")
    } else if (spy == 2){
        aux = array(NA, c(dim(py), 1))
        aux[, , ] = as.matrix(py)
        dimnames(aux)[[2]] = colnames(py)
        py = aux
        y = as.ts(y)
    }
    h = dim(py)[1]
    ny = dim(py)[2]
    nSeries = dim(py)[3]
    n = size(y)[1]
    if (nSeries == 1){
        y = ts(matrix(y, length(y), 1), start = start(y), frequency = frequency(y))
    }
    insample = FALSE
    if (n > h){
        insample = TRUE
    } else if (n < h){
        stop("Insample data should be longer than forecasting horizon!!!")
    }
    out = matrix(NA, ny, 7 + 3 * insample)
    rownames(out) = c(1 : ny)
    namesc = c("ME", "RMSE", "MAE", "MPE", "PRMSE", "MAPE", "sMAPE")
    if (insample){
        namesc = c(namesc, "MASE", "RelMAE", "Theil\'s U")
    }
    colnames(out) = namesc
    rownames(out) = colnames(py)
    for (i in 1 : ny){
        e = array(py[, i, ], c(h, nSeries)) - tail(y, h)
        p = 100 * e / tail(y, h)
        aux = cbind(colMeans(e, na.rm = TRUE),
                    sqrt(colMeans(e * e, na.rm = TRUE)),
                    colMeans(abs(e), na.rm = TRUE),
                    colMeans(p, na.rm = TRUE),
                    sqrt(colMeans(p * p, na.rm = TRUE)),
                    colMeans(abs(p), na.rm = TRUE),
                    colMeans(200 * abs(e) / (array(py[, i, ], c(h, nSeries)) + tail(y, h)), na.rm = TRUE))
        if (insample){
            theil = 100 * (tail(y, h) - y[(n - h) : (n - 1), ]) / tail(y, h)
            fRW = y[(s + 1) : (n - h), ] - y[1 : (n - h - s), ]
            if (nSeries == 1){
                fRW = matrix(fRW, length(fRW), 1)
            }
            aux = cbind(aux,
                        colMeans(abs(e), na.rm = TRUE) / colMeans(abs(fRW), na.rm = TRUE),
                        colSums(abs(e), na.rm = TRUE) / colSums(abs(fRW), na.rm = TRUE),
                        sqrt(colSums(p * p, na.rm = TRUE) / colSums(theil * theil, na.rm = TRUE)))
        }
        out[i, ] = apply(aux, MARGIN = 2, FUN = collectFun, na.rm = TRUE)
    }
    return(out)
}
#' @title box.cox
#' @description Runs Box-Cox transform of a time series
#'
#' @param x Time series object.
#' @param lambda Lambda parameter for Box-Cox transform.
#' 
#' @return Box-Cox transformed time series
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{inv.box.cox}}, \code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, \code{\link{UCsmooth}}, 
#'          \code{\link{UCdisturb}}, \code{\link{UCcomponents}}
#'          
#' @examples
#' \donttest{
#' y <- box.cox(AirPassengers, 0.5)
#' plot(y)
#' }
#' @rdname box.cox
#' @export
box.cox <- function(x, lambda){
        if (abs(lambda) < 1e-4){
                return(log(x))
        } else if (lambda > 0.99 && lambda < 1.01) {
                return(x)
        } else {
                return((x ^ lambda - 1) / lambda)
        }
}
#' @title inv.box.cox
#' @description Runs inverse of Box-Cox transform of a time series
#'
#' @param x Transformed time series object.
#' @param lambda Lambda parameter used for Box-Cox transform.
#' 
#' @return Inverse Box-Cox transformed time series
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{box.cox}}, \code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, \code{\link{UCsmooth}}, 
#'          \code{\link{UCdisturb}}, \code{\link{UCcomponents}}
#'          
#' @examples
#' \donttest{
#' y <- inv.box.cox(box.cox(AirPassengers, 0.5), 0.5)
#' plot(y)
#' }
#' @rdname inv.box.cox
#' @export
inv.box.cox <- function(x, lambda){
        if (abs(lambda) < 1e-4){
                return(exp(x))
        } else if (lambda > 0.99 && lambda < 1.01) {
                return(x)
        } else {
                return((x * lambda + 1) ^ (1 / lambda))
        }
}

#' @title print.UComp
#' @description Prints an UComp object
#'
#' @details See help of \code{UC}.
#'
#' @param x Object of class \dQuote{UComp}.
#' @param ... Additional inputs to handle the way to print output.
#' 
#' @return No return value, called for side effects
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, \code{\link{UCsmooth}}, 
#'          \code{\link{UCdisturb}}, \code{\link{UCcomponents}}
#'          
#' @examples
#' \donttest{
#' y <- log(AirPassengers)
#' m1 <- UCforecast(y, model = "llt/equal/arma(0,0)")
#' print(m1)
#' }
#' @noRd
#' @export 
print.UComp = function(x, ...){
    if (x$model != "error") {
        x = UCvalidate(x, TRUE)
    } else {
        stop("ERROR: Model is not valid!!")
        return()
    }
}
#' @title summary.UComp
#' @description Prints an UComp object on screen
#'
#' @param object Object of class \dQuote{UComp}.
#' @param ... Additional inputs to function.
#' 
#' @details See help of \code{UC}.
#'
#' @return No return value, called for side effects
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, \code{\link{UCsmooth}}, 
#'          \code{\link{UCdisturb}}, \code{\link{UCcomponents}}
#'          
#' @examples
#' \donttest{
#' y <- log(AirPassengers)
#' m1 <- UCforecast(y, model = "llt/equal/arma(0,0)")
#' summary(m1)
#' }
#' @noRd
#' @export 
summary.UComp = function(object, ...){
    print(object)
}
#' @title plot.UComp
#' @description Plot components of UComp object
#'
#' @details See help of \code{UC}.
#'
#' @param x Object of class \dQuote{UComp}.
#' @param ... Additional inputs to function.
#' 
#' @return No return value, called for side effects
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, \code{\link{UCsmooth}}, 
#'          \code{\link{UCdisturb}}, \code{\link{UCcomponents}}
#'          
#' @examples
#' \donttest{
#' y <- log(AirPassengers)
#' m1 <- UCforecast(y, model = "llt/equal/arma(0,0)")
#' plot(m1)
#' }
#' @noRd
#' @export 
plot.UComp = function(x, ...){
    if (x$model == "error") {
        stop("ERROR: Model is not valid!!")
        return()
    }
    if (length(x$comp) < 2){
        VERBOSE = x$verbose
        x$verbose = FALSE
        x = UCcomponents(x)
        x$verbose = VERBOSE
    }
    if (is.ts(x$comp)){
        plot(x$comp, main = "Time Series Decomposition")
    } else {
        plot(ts(x$comp, frequency = x$periods[1]),
             main = "Time Series Decomposition")
    }
}
#' @title fitted.UComp
#' @description Fitted output values of UComp object
#'
#' @details See help of \code{UC}.
#'
#' @param object Object of class \dQuote{UComp}.
#' @param ... Additional inputs to function.
#' 
#' @return Fitted values of a UC model
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, \code{\link{UCsmooth}}, 
#'          \code{\link{UCdisturb}}, \code{\link{UCcomponents}}
#'          
#' @examples
#' \donttest{
#' y <- log(AirPassengers)
#' m1 <- UCforecast(y, model = "llt/equal/arma(0,0)")
#' fitted(m1)
#' }
#' @noRd
#' @export 
fitted.UComp = function(object, ...){
       if (object$model == "error") {
           stop("ERROR: Model is not valid!!")
           return()
        }
        if (length(object$yFit) < 2){
                VERBOSE = x$verbose
                x$verbose = FALSE
                object = UCsmooth(object)
                x$verbose = VERBOSE
        }
        return(object$yFit)
}
#' @title residuals.UComp
#' @description Standardised innovations values of UComp object
#'
#' @details See help of \code{UC}.
#'
#' @param object Object of class \dQuote{UComp}.
#' @param ... Additional inputs to function.
#' 
#' @return Residuals of a UC model
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, \code{\link{UCsmooth}}, 
#'          \code{\link{UCdisturb}}, \code{\link{UCcomponents}}
#'          
#' @examples
#' \donttest{
#' y <- log(AirPassengers)
#' m1 <- UCforecast(y, model = "llt/equal/arma(0,0)")
#' residuals(m1)
#' }
#' @noRd
#' @export 
residuals.UComp = function(object, ...){
    if (object$model == "error") {
        stop("ERROR: Model is not valid!!")
        return()
    }
    if (length(object$yFit) < 2){
        VERBOSE = object$verbose
        object$verbose = FALSE
        object = UCfilter(object)
        object$verbose = VERBOSE
    }
    return(object$v)
}
#' @title logLik.UComp
#' @description Extract log Likelihood value of UComp object
#'
#' @details See help of \code{UC}.
#'
#' @param object Object of class \dQuote{UComp}.
#' @param ... Additional inputs to function.
#' 
#' @return Log-Likelihood value of a UC model
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, \code{\link{UCsmooth}}, 
#'          \code{\link{UCdisturb}}, \code{\link{UCcomponents}}
#'          
#' @examples
#' \donttest{
#' y <- log(AirPassengers)
#' m1 <- UCforecast(y, model = "llt/equal/arma(0,0)")
#' logLik(m1)
#' }
#' @noRd
#' @export 
logLik.UComp = function(object, ...){
    if (object$model == "error") {
        stop("ERROR: Model is not valid!!")
        return()
    }
    out = object$criteria[1]
    class(out) = "logLik"
    attr(out, "df") = length(object$p) - 1
    attr(out, "nobs") = length(object$y)
    return(out)
}
#' @title AIC.UComp
#' @description Extract AIC value of UComp object
#'
#' @details Selection criteria for models with different number of 
#' parameters, the smaller AIC the better. The formula used here is
#' \eqn{AIC=-2 (ln(L) - k) / n}, where \eqn{ln(L)} is the log-likelihood
#' at the optimum, \eqn{k} is the number of parameters plus
#' non-stationary states and \eqn{n} is the number of observations.
#' Mind that this formulation differs from the usual definition that
#' does not divide by \eqn{n}. This makes that AIC(m) and AIC(logLik(m))
#' give different results, being m an UComp object.
#'
#' @param object Object of class \dQuote{UComp}.
#' @param ... Additional inputs to function.
#' @param k The penalty per parameter to be used.
#' 
#' @return AIC value of a UC model
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, \code{\link{UCsmooth}}, 
#'          \code{\link{UCdisturb}}, \code{\link{UCcomponents}}
#'          
#' @examples
#' \donttest{
#' y <- log(AirPassengers)
#' m1 <- UCforecast(y, model = "llt/equal/arma(0,0)")
#' AIC(m1)
#' }
#' @export 
AIC.UComp = function(object, ..., k = 2){
     if (object$model == "error") {
         stop("ERROR: Model is not valid!!")
        return()
     }
     return(object$criteria[2])
}
#' @title BIC.UComp
#' @description Extract BIC (or SBC) value of UComp object
#'
#' @details Selection criteria for models with different number of 
#' parameters, the smaller BIC the better. The formula used here is
#' \eqn{BIC=(-2 ln(L) + k ln(n)) / n}, where \eqn{ln(L)} is the log-likelihood
#' at the optimum, \eqn{k} is the number of parameters plus
#' non-stationary states and \eqn{n} is the number of observations.
#' Mind that this formulation differs from the usual definition that
#' does not divide by \eqn{n}. This makes that BIC(m) and BIC(logLik(m))
#' give different results, being m an UComp object.
#'
#' @param object Object of class \dQuote{UComp}.
#' @param ... Additional inputs to function.
#' 
#' @return BIC value of a UC model
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, \code{\link{UCsmooth}}, 
#'          \code{\link{UCdisturb}}, \code{\link{UCcomponents}}
#'          
#' @examples
#' \donttest{
#' y <- log(AirPassengers)
#' m1 <- UCforecast(y, model = "llt/equal/arma(0,0)")
#' BIC(m1)
#' }
#' @export 
BIC.UComp = function(object, ...){
    if (object$model == "error") {
        stop("ERROR: Model is not valid!!")
        return()
    }
    return(object$criteria[3])
}
#' @title coef.UComp
#' @description Extracts model coefficients of UComp object
#'
#' @details See help of \code{UC}.
#'
#' @param object Object of class \dQuote{UComp}.
#' @param ... Ignored.
#' 
#' @return Estimated coefficients of a UC model
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, \code{\link{UCsmooth}}, 
#'          \code{\link{UCdisturb}}, \code{\link{UCcomponents}}
#'          
#' @examples
#' \donttest{
#' y <- log(AirPassengers)
#' m1 <- UCforecast(y, model = "llt/equal/arma(0,0)")
#' coef(m1)
#' }
#' @noRd
#' @export 
coef.UComp = function(object, ...){
    if (object$model == "error") {
        stop("ERROR: Model is not valid!!")
        return()
    }
    if (length(object$table) < 2){
        VERBOSE = object$verbose
        object$verbose = FALSE
        object = UCvalidate(object, FALSE)
        object$verbose = VERBOSE
    }
    return(object$p)
}
#' @title predict.UComp
#' @description Forecasting using structural Unobseved Components models with prediction intervals
#'
#' @details See help of \code{UC}.
#'
#' @param object Object of class \dQuote{UComp}.
#' @param newdata New output data to apply \dQuote{UComp} object to.
#' @param n.ahead Number of steps ahead to forecast or new inputs variables 
#' including their predictions.
#' @param level Confidence level for prediction intervals.
#' @param ... Ignored.
#' 
#' @return Forecasts of a UC model
#' 
#' @author Diego J. Pedregal
#' 
#' @return A matrix with the mean forecasts and lower and upper prediction intervals
#' 
#' @seealso \code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, \code{\link{UCsmooth}}, 
#'          \code{\link{UCdisturb}}, \code{\link{UCcomponents}}
#'          
#' @examples
#' \donttest{
#' y <- log(AirPassengers)
#' m1 <- UCforecast(y, model = "llt/eq/arma(0,0)")
#' f1 <- predict(m1)
#' }
#' @export 
predict.UComp = function(object, newdata = NULL, n.ahead = NULL, level = 0.95, ...){
    if (object$model == "error") {
        stop("ERROR: Model is not valid!!")
        return()
    }
    cnst = qt(level + (1 - level) / 2, length(object$y) - length(object$p))
    if (!is.null(newdata)){
        object$y = newdata
    }
   if (!is.null(n.ahead) && length(size(n.ahead)) == 1){
        object$h = n.ahead
    }
    if (!is.null(n.ahead) && length(size(n.ahead)) > 1){
        object$u = n.ahead
    }
    m = UCfilter(object)
    pred = as.numeric(tail(m$yFit, m$h))
    predS = as.numeric(sqrt(tail(m$yFitV, m$h)))
    out = cbind(pred, pred - cnst * predS, pred + cnst * predS)
    if (is.ts(object$y)){
        aux = ts(matrix(0, length(object$y) + 1), start = start(object$y),
                 frequency = frequency(object$y))
        stDate = end(aux)
        freq = frequency(object$y)
        out = ts(out, start = stDate, frequency = freq)
    }
    colnames(out) = c("frcst", "lower", "upper")
    return(out)
}
#' @title tsdiag.UComp
#' @description Diagnostic plots for UComp objects
#'
#' @details See help of \code{UC}.
#'
#' @param object Object of class \dQuote{UComp}.
#' @param gof.lag Maximum number of lags for pormanteau Ljung-Box test
#' @param ... Additional inputs to function.
#' 
#' @return Diagnostics of a UC model
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, \code{\link{UCsmooth}}, 
#'          \code{\link{UCdisturb}}, \code{\link{UCcomponents}}
#'          
#' @examples
#' \donttest{
#' y <- log(AirPassengers)
#' m1 <- UCforecast(y, model = "llt/eq/arma(0,0)")
#' tsdiag(m1)
#' }
#' @noRd
#' @export 
tsdiag.UComp = function(object, gof.lag = NULL, ...){
    if (object$model == "error") {
        stop("ERROR: Model is not valid!!")
        return()
    }
    if (length(object$v) < 2){
        object = UCfilter(object)
    }
    # Number of u's
    nu = dim(object$u)[1]
    if (dim(object$u)[2] == 2){
        nu = 0
    }
    # Number of parameters
    nPar = length(object$p) - 1 + nu
    aux = list(residuals = object$v,
               sigma2 = 1,
               nobs = length(object$y) - nPar,
               coef = object$p,
               x = object$y,
               fitted = object$yFit)
    if (is.null(gof.lag)){
        tsdiag(structure(aux, class = "Arima"))
    } else {
        tsdiag(structure(aux, class = "Arima"), gof.lag)
    }
}
#' @title getp0
#' @description Get initial conditions for parameters of \code{UComp} object
#'
#' @details Provides initial parameters of a given model for the time series.
#' They may be changed arbitrarily by the user to include as an input \code{p0} to
#' \code{UC} or \code{UCforecast} functions (see example below).
#' There is no guarantee that the model will converge and selecting initial conditions
#' should be used with care.
#'
#' @param y a time series to forecast.
#' @param model any valid \code{UComp} model without any ?.
#' @param periods vector of fundamental period and harmonics required.
#' 
#' @author Diego J. Pedregal
#' 
#' @return A set of parameters p0 of an object of class \code{UComp}
#' to use as input to \code{\link{UC}}, \code{\link{UCforecast}}.
#' 
#' @seealso \code{\link{UC}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, \code{\link{UCsmooth}}, 
#'          \code{\link{UCdisturb}}, \code{\link{UCcomponents}},
#'          \code{\link{UChp}}
#'          
#' @examples
#' \dontrun{
#' p0 <- getp0(log(AirPassengers), model = "llt/equal/arma(0,0)")
#' p0[1] <- 0  # p0[1] <- NA
#' m <- UCforecast(log(AirPassengers), model = "llt/equal/arma(0,0)", p0 = p0)
#' }
#' @rdname getp0
#' @export
getp0 = function(y, model = "llt/equal/arma(0,0)", periods = NA){
    if (model == "error") {
        stop("ERROR: Model is not valid!!")
        return()
    }
    if (any(utf8ToInt(model) == utf8ToInt("?"))){
        stop("UComp ERROR: Model should not contain any \'?\'!!!")
    }
    sys = UCsetup(y, model = model, periods = periods, verbose = FALSE)
    sys = UCestim(sys)
    p0 = as.vector(sys$p0)
    p1 = coef(sys)
    names(p0) = names(p1)
    
    return(p0)
}

#' @title print.ARIMA
#' @description Prints an ARIMA object
#'
#' @details See help of \code{ARIMA}.
#'
#' @param x Object of class \dQuote{ARIMA}.
#' @param ... Additional inputs to handle the way to print output.
#' 
#' @return No return value, called for side effects
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{ARIMA}}, \code{\link{ARIMAforecast}}, \code{\link{ARIMAvalidate}},
#'          
#' @examples
#' \donttest{
#' m1 <- ARIMAforecast(log(gdp))
#' print(m1)
#' }
#' @rdname print
#' @noRd
#' @export 
print.ARIMA = function(x, ...){
    if (length(x$table) < 3) {
        x = ARIMAvalidate(x)
    }
    cat(x$table)
}
#' @title summary.ARIMA
#' @description Prints an ARIMA object on screen
#'
#' @param object Object of class \dQuote{ARIMA}.
#' @param ... Additional inputs to function.
#' 
#' @details See help of \code{ARIMA}.
#'
#' @return No return value, called for side effects
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{ARIMA}}, \code{\link{ARIMAforecast}}, \code{\link{ARIMAvalidate}},
#'          
#' @examples
#' \donttest{
#' m1 <- ARIMAforecast(log(gdp))
#' summary(m1)
#' }
#' @rdname summary.ARIMA
#' @noRd
#' @export 
summary.ARIMA = function(object, ...){
    print(object)
}
#' @title residuals.ARIMA
#' @description Residuals values of ARIMA object
#'
#' @details See help of \code{ARIMA}.
#'
#' @param object Object of class \dQuote{ARIMA}.
#' @param ... Additional inputs to function.
#' 
#' @return Error from ARIMA model
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{ARIMA}}, \code{\link{ARIMAforecast}}, \code{\link{ARIMAvalidate}},
#'          
#' @examples
#' \donttest{
#' y <- log(AirPassengers)
#' m1 <- ARIMA(y)
#' residuals(m1)
#' }
#' @noRd
#' @export 
residuals.ARIMA = function(object, ...){
    return(object$error)
}
#' @title plot.ARIMA
#' @description Plot zplane of ARIMA object
#'
#' @details See help of \code{ARIMA}.
#'
#' @param x Object of class \dQuote{ARIMA}.
#' @param ... Additional inputs to function.
#' 
#' @return No return value, called for side effects
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{ARIMA}}, \code{\link{ARIMAforecast}}, \code{\link{ARIMAvalidate}},
#'          
#' @examples
#' \donttest{
#' m1 <- ARIMAforecast(log(gdp))
#' plot(m1)
#' }
#' @rdname plot.ARIMA
#' @noRd
#' @export 
plot.ARIMA = function(x, ...){
   if (is.null(x$p)){
           stop("ERROR: Please, estimate the model first!!!")
   } else {
        ARaux = matrix(1, x$model[1] + 1)
        ARS = matrix(0, x$model[4] * x$s + 1)
        MAaux = matrix(1, x$model[3] + 1)
        MAS = matrix(0, x$model[6] * x$s + 1)
        ARaux[1] = 1
        MAaux[1] = 1
        ARS[1] = 1
        MAS[1] = 1
        i = 1
        if (x$model[1] > 0){
                for (j in 1 : x$model[1]){
                        ARaux[j + 1] = x$p[i]
                        i = i + 1
                }
        }
        if (x$model[4] > 0){
                for (j in 1 : x$model[4]){
                        ARS[j * x$s + 1] = x$p[i]
                        i = i + 1
                }
        }
        if (x$model[3] > 0){
                for (j in 1 : x$model[3]){
                        MAaux[j + 1] = x$p[i]
                        i = i + 1
                }
        }
        if (x$model[6] > 0){
                for (j in 1 : x$model[6]){
                        MAS[j * x$s + 1] = x$p[i]
                        i = i + 1
                }
        }
        AR = conv(ARaux, ARS)
        MA = conv(MAaux, MAS)
        zplane(MA, AR)
   }
}

#' @title TETScomponents
#' @description Estimates components of TOBIT TETS models
#'
#' @param m an object of type \code{TETS} created with \code{TETSforecast}
#' 
#' @return The same input object with the appropriate fields 
#' filled in, in particular:
#' \item{comp}{Estimated components in matrix form}
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{TETS}}, \code{\link{TETSforecast}}, \code{\link{TETSvalidate}}
#'        
#' @examples
#' \donttest{
#' m1 <- TETS(log(gdp))
#' m1 <- TETScomponents(m1)
#' }
#' @rdname TETScomponents
#' @export
TETScomponents= function(m){
    if (is.null(m$u))
        u = m$u
    else {
        if (is.vector(m$u)){
            u = matrix(m$u, 1, length(m$u))
        } else {
            nu = dim(m$u)
            u = as.numeric(m$u);
            u = matrix(u, nu[1], nu[2])
        }
    }
    if (inherits(m, "ETS")){
        output = ETSc("components", as.numeric(m$y), u, m$model, m$s, m$h,
                  m$criterion, m$armaIdent, m$identAll, m$forIntervals,
                  m$bootstrap, m$nSimul, m$verbose, m$lambda,
                  m$alphaL, m$betaL, m$gammaL, m$phiL, m$p0)
    } else {
        output = TETSc("components", as.numeric(m$y), u, m$model, m$s, m$h,
                  m$criterion, m$armaIdent, m$identAll, m$forIntervals,
                  m$bootstrap, m$nSimul, m$verbose, m$lambda,
                  m$alphaL, m$betaL, m$gammaL, m$phiL, m$p0, m$Ymin, m$Ymax)
    }
    if (is.ts(m$y))
        m$comp = ts(output$comp, start = start(m$y), frequency = frequency(m$y))
    else
        m$comp = output$comp
    colnames(m$comp) = strsplit(output$compNames, split = "/")[[1]]
    return(m)
}
    

#' @title ETSsetup
#' @description Sets up ETS general univariate models
#'
#' @details See help of \code{ETSforecast}.
#'
#' @param y a time series to forecast (it may be either a numerical vector or
#' a time series object). This is the only input required. If a vector, the additional
#' input \code{s} should be supplied compulsorily (see below).
#' @param u a matrix of input time series. If 
#' the output wanted to be forecast, matrix \code{u} should contain future values for inputs.
#' @param model the model to estimate. It is a single string indicating the type of 
#' model for each component with one or two letters:
#' \itemize{
#' \item Error: ? / A / M
#' 
#' \item Trend: ? / N / A / Ad / M / Md 
#' 
#' \item Seasonal: ? / N / A / M
#' 
#' }
#' 
#' @param s seasonal period of time series (1 for annual, 4 for quarterly, ...)
#' @param h forecast horizon. If the model includes inputs h is not used, the lenght of u is used instead.
#' @param criterion information criterion for identification ("aic", "bic" or "aicc").
#' @param lambda Box-Cox lambda parameter (NULL: estimate)
#' @param armaIdent check for arma models for error component (TRUE / FALSE).
#' @param identAll run all models to identify the best one (TRUE / FALSE)
#' @param forIntervals estimate forecasting intervals (TRUE / FALSE)
#' @param bootstrap use bootstrap simulation for predictive distributions
#' @param nSimul number of simulation runs for bootstrap simulation of predictive distributions
#' @param verbose intermediate estimation output (TRUE / FALSE)
#' @param alphaL constraints limits for alpha parameter
#' @param betaL constraints limits for beta parameter
#' @param gammaL constraints limits for gamma parameter
#' @param phiL constraints limits for phi parameter
#' @param p0 initial values for parameter search (alpha, beta, phi, gamma) with consraints:
#' \itemize{ 
#' \item 0 < alpha < 1
#' 
#' \item 0 < beta < alpha
#' 
#' \item 0 < phi < 1
#' 
#' \item 0 < gamma < 1 - alpha
#' }
#' 
#' @return An object of class \code{ETS}. It is a list with fields including all the inputs and
#'         the fields listed below as outputs. All the functions in this package fill in
#'         part of the fields of any \code{ETS} object as specified in what follows (function 
#'         \code{ETS} fills in all of them at once):
#' 
#' After running \code{ETSforecast}:
#' \item{p}{Estimated parameters}
#' \item{criteria}{Values for estimation criteria (LogLik, AIC, BIC, AICc)}
#' \item{yFor}{Forecasted values of output}
#' \item{yForV}{Variance of forecasted values of output}
#' \item{ySimul}{Bootstrap simulations for forecasting distribution evaluation}
#' 
#' After running \code{ETSvalidate}:
#' \item{table}{Estimation and validation table}
#' \item{comp}{Estimated components in matrix form}
#' 
#' After running \code{ETScomponents}:
#' \item{comp}{Estimated components in matrix form}
#' 
#' @author Diego J. Pedregal
#' 
#' @return An object of class \code{ETS}. See \code{ETSforecast}.
#' 
#' @seealso \code{\link{ETS}}, \code{\link{ETSforecast}}, \code{\link{ETSvalidate}},
#'          \code{\link{ETScomponents}}
#'          
#' @examples
#' \donttest{
#' y <- log(AirPassengers)
#' m1 <- ETSsetup(y)
#' m1 <- ETSsetup(y,  model = "???")
#' m1 <- ETSsetup(y,  model = "?AA")
#' }
#' @rdname ETSsetup
#' @export
ETSsetup = function(y, u = NULL, model = "???", s = frequency(y), h = 2 * s, criterion = "aicc", 
                    lambda = 1, armaIdent = FALSE, identAll = FALSE, forIntervals = FALSE,
                    bootstrap = FALSE, nSimul = 5000, verbose = FALSE,
                    alphaL = c(1e-8, 1-1e-8), betaL = alphaL, gammaL = alphaL, 
                    phiL = c(0.8, 0.98), p0 = -99999){
    y = as.ts(y)
    if (alphaL[1] >= alphaL[2])
        stop("Wrong alpha limits!!")
    if (betaL[1] >= betaL[2])
        stop("Wrong alpha limits!!")
    if (gammaL[1] >= gammaL[2])
        stop("Wrong alpha limits!!")
    if (phiL[1] >= phiL[2])
        stop("Wrong alpha limits!!")
    if (s > 24)
        stop("Data with period greater than 24 are not allowed!!")
    if (length(y) < s + 2 || length(y) < 8)
        stop("Error: Not enough data to estimate model!!")
    if (is.null(lambda))
        lambda = 9999.9
    out =  list(y = y,
                u = u,
                model = model,
                s = s,
                h = h,
                p0 = p0,
                criterion = criterion,
                lambda = lambda,
                armaIdent = armaIdent,
                identAll = identAll,
                forIntervals = forIntervals,
                bootstrap = bootstrap,
                nSimul = nSimul,
                verbose = verbose,
                alphaL = alphaL,
                betaL = betaL,
                gammaL = gammaL,
                phiL = phiL,
                # outputs
                criteria = NULL,
                yFor = NULL,
                yForV = NULL,
                comp = NULL,
                ySimul = NULL,
                table = "",
                p = NULL,
                truep = NULL,
                v = NULL)
    return(structure(out, class = "ETS"))
}
#' @title ETSforecast
#' @description Estimates and forecasts ETS general univariate models
#'
#' @details \code{ETSforecast} is a function for modelling and forecasting univariate
#' time series with ExponenTial Smoothing (ETS) time series models. 
#' It sets up the model with a number of control variables that
#' govern the way the rest of functions in the package will work. It also estimates 
#' the model parameters by Maximum Likelihood and forecasts the data.
#'
#' @inheritParams ETS
#' 
#' @return An object of class \code{ETS}. It is a list with fields including all the inputs and
#'         the fields listed below as outputs. All the functions in this package fill in
#'         part of the fields of any \code{ETS} object as specified in what follows (function 
#'         \code{ETS} fills in all of them at once):
#' 
#' After running \code{ETSforecast}:
#' \item{p}{Estimated parameters}
#' \item{criteria}{Values for estimation criteria (LogLik, AIC, BIC, AICc)}
#' \item{yFor}{Forecasted values of output}
#' \item{yForV}{Variance of forecasted values of output}
#' \item{ySimul}{Bootstrap simulations for forecasting distribution evaluation}
#' 
#' After running \code{ETSvalidate}:
#' \item{table}{Estimation and validation table}
#' \item{comp}{Estimated components in matrix form}
#' 
#' After running \code{ETScomponents}:
#' \item{comp}{Estimated components in matrix form}
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{ETS}}, \code{\link{ETSvalidate}},
#'          \code{\link{ETScomponents}}
#'          
#' @examples
#' \donttest{
#' y <- log(AirPassengers)
#' m1 <- ETSforecast(y)
#' m1 <- ETSforecast(y, model = "A?A")
#' }
#' @rdname ETSforecast
#' @export
ETSforecast = function(y, u = NULL, model = "???", s = frequency(y), h = max(2 * s, 6), criterion = "aicc", 
                    lambda = 1, armaIdent = FALSE, identAll = FALSE, forIntervals = FALSE,
                    bootstrap = FALSE, nSimul = 5000, verbose = FALSE,
                    alphaL = c(1e-8, 1-1e-8), betaL = alphaL, gammaL = alphaL, 
                    phiL = c(0.8, 0.98), p0 = -99999){
    m1 = ETSsetup(y, u, model, s, h, criterion, lambda, armaIdent, identAll, forIntervals,
                  bootstrap, nSimul, verbose, alphaL, betaL, gammaL, phiL, p0)
    m1 = ETSestim(m1)
    if (verbose)
            cat(m1$table)
    return(m1)
}
#' @title ETS
#' @description Runs all relevant functions for ETS modelling
#'
#' @details See help of \code{ETSforecast}.
#'
#' @param y a time series to forecast (it may be either a numerical vector or
#' a time series object). This is the only input required. If a vector, the additional
#' input \code{s} should be supplied compulsorily (see below).
#' @param u a matrix of input time series. If 
#' the output wanted to be forecast, matrix \code{u} should contain future values for inputs.
#' @param model the model to estimate. It is a single string indicating the type of 
#' model for each component with one or two letters:
#' \itemize{
#' \item Error: ? / A / M
#' 
#' \item Trend: ? / N / A / Ad / M / Md 
#' 
#' \item Seasonal: ? / N / A / M
#' 
#' }
#' 
#' @param s seasonal period of time series (1 for annual, 4 for quarterly, ...)
#' @param h forecast horizon. If the model includes inputs h is not used, the lenght of u is used instead.
#' @param criterion information criterion for identification ("aic", "bic" or "aicc").
#' @param lambda Box-Cox lambda parameter (NULL: estimate)
#' @param armaIdent check for arma models for error component (TRUE / FALSE).
#' @param identAll run all models to identify the best one (TRUE / FALSE)
#' @param forIntervals estimate forecasting intervals (TRUE / FALSE)
#' @param bootstrap use bootstrap simulation for predictive distributions
#' @param nSimul number of simulation runs for bootstrap simulation of predictive distributions
#' @param verbose intermediate estimation output (TRUE / FALSE)
#' @param alphaL constraints limits for alpha parameter
#' @param betaL constraints limits for beta parameter
#' @param gammaL constraints limits for gamma parameter
#' @param phiL constraints limits for phi parameter
#' @param p0 initial values for parameter search (alpha, beta, phi, gamma) with consraints:
#' \itemize{ 
#' \item 0 < alpha < 1
#' 
#' \item 0 < beta < alpha
#' 
#' \item 0 < phi < 1
#' 
#' \item 0 < gamma < 1 - alpha
#' }
#' 
#' @return An object of class \code{ETS}. It is a list with fields including all the inputs and
#'         the fields listed below as outputs. All the functions in this package fill in
#'         part of the fields of any \code{ETS} object as specified in what follows (function 
#'         \code{ETS} fills in all of them at once):
#' 
#' After running \code{ETSforecast}:
#' \item{p}{Estimated parameters}
#' \item{criteria}{Values for estimation criteria (LogLik, AIC, BIC, AICc)}
#' \item{yFor}{Forecasted values of output}
#' \item{yForV}{Variance of forecasted values of output}
#' \item{ySimul}{Bootstrap simulations for forecasting distribution evaluation}
#' 
#' After running \code{ETSvalidate}:
#' \item{table}{Estimation and validation table}
#' \item{comp}{Estimated components in matrix form}
#' 
#' After running \code{ETScomponents}:
#' \item{comp}{Estimated components in matrix form}
#' 
#' @author Diego J. Pedregal
#' 
#' @return An object of class \code{ETS}. See \code{ETSforecast}.
#' 
#' @seealso \code{\link{ETSforecast}}, \code{\link{ETSvalidate}},
#'          \code{\link{ETScomponents}}
#'          
#' @examples
#' \donttest{
#' y <- log(AirPassengers)
#' m1 <- ETS(y)
#' m1 <- ETS(y, model = "MAM")
#' }
#' @rdname ETS
#' @export
ETS = function(y, u = NULL, model = "???", s = frequency(y), h = 2 * s, criterion = "aicc", 
               lambda = 1, armaIdent = FALSE, identAll = FALSE, forIntervals = FALSE,
               bootstrap = FALSE, nSimul = 5000, verbose = FALSE,
               alphaL = c(1e-8, 1-1e-8), betaL = alphaL, gammaL = alphaL, 
               phiL = c(0.8, 0.98), p0 = -99999){
    m1 = ETSsetup(y, u, model, s, h, criterion, lambda, armaIdent, identAll, forIntervals,
                  bootstrap, nSimul, verbose, alphaL, betaL, gammaL, phiL, p0)
    m1 = ETSvalidate(m1)
    m1$v = m1$comp[, 1]
    if (verbose)
            cat(m1$table)
    return(m1)
}

#' @title print.ETS
#' @description Prints an ETS object
#'
#' @details See help of \code{ETS}.
#'
#' @param x Object of class \dQuote{ETS}.
#' @param ... Additional inputs to handle the way to print output.
#' 
#' @return No return value, called for side effects
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{ETS}}, \code{\link{ETSforecast}}, \code{\link{ETSvalidate}},
#'          \code{\link{ETScomponents}}, \code{\link{ETSestim}}
#'          
#' @examples
#' \donttest{
#' m1 <- ETSforecast(log(gdp))
#' print(m1)
#' }
#' @rdname print
#' @noRd
#' @export 
print.ETS = function(x, ...){
    if (length(x$table) < 3){
        x = ETSvalidate(x)
    }
    cat(x$table)
}
#' @title summary.ETS
#' @description Prints an ETS object on screen
#'
#' @param object Object of class \dQuote{ETS}.
#' @param ... Additional inputs to function.
#' 
#' @details See help of \code{ETS}.
#'
#' @return No return value, called for side effects
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{ETS}}, \code{\link{ETSforecast}}, \code{\link{ETSvalidate}},
#'          \code{\link{ETScomponents}}, \code{\link{ETSestim}}
#'          
#' @examples
#' \donttest{
#' m1 <- ETSforecast(log(gdp))
#' summary(m1)
#' }
#' @rdname summary.ETS
#' @noRd
#' @export 
summary.ETS = function(object, ...){
    print(object)
}
#' @title plot.ETS
#' @description Plot components of ETS object
#'
#' @details See help of \code{ETS}.
#'
#' @param x Object of class \dQuote{ETS}.
#' @param ... Additional inputs to function.
#' 
#' @return No return value, called for side effects
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{ETS}}, \code{\link{ETSforecast}}, \code{\link{ETSvalidate}},
#'          \code{\link{ETScomponents}}, \code{\link{ETSestim}}
#'          
#' @examples
#' \donttest{
#' m1 <- ETSforecast(log(gdp))
#' plot(m1)
#' }
#' @rdname plot
#' @noRd
#' @export 
plot.ETS = function(x, ...){
    if (length(x$comp) < 2){
        x = ETScomponents(x)
    }
    if (is.ts(x$comp)){
        plot(x$comp, main = "Time Series Decomposition")
    } else {
        plot(ts(x$comp, frequency = x$s),
             main = "Time Series Decomposition")
    }
}
#' @title fitted.ETS
#' @description Fitted output values of ETS object
#'
#' @details See help of \code{ETS}.
#'
#' @param object Object of class \dQuote{ETS}.
#' @param ... Additional inputs to function.
#' 
#' @return Fitted values from ETS model
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{ETS}}, \code{\link{ETSforecast}}, \code{\link{ETSvalidate}},
#'          \code{\link{ETScomponents}}, \code{\link{ETSestim}}
#'          
#' @examples
#' \donttest{
#' m1 <- ETSforecast(log(gdp))
#' fitted(m1)
#' }
#' @rdname fitted
#' @noRd
#' @export 
fitted.ETS = function(object, ...){
    if (length(object$comp) < 2){
        object = ETScomponents(object)
    }
    return(object$com[, 2])
}
#' @title residuals.ETS
#' @description Residuals of ETS object
#'
#' @details See help of \code{ETS}.
#'
#' @param object Object of class \dQuote{ETS}.
#' @param ... Additional inputs to function.
#' 
#' @return Residuals of ETS model
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{ETS}}, \code{\link{ETSforecast}}, \code{\link{ETSvalidate}},
#'          \code{\link{ETScomponents}}, \code{\link{ETSestim}}
#'          
#' @examples
#' \donttest{
#' m1 <- ETSforecast(log(gdp))
#' residuals(m1)
#' }
#' @rdname residuals
#' @noRd
#' @export 
residuals.ETS = function(object, ...){
    if (length(object$comp) < 2){
        object = ETScomponents(object)
    }
    return(object$com[, 1])
}
#' @title auxInvBoxCox
#' @description Inverse of Box-Cox transformation
#'
#' @param y matrix, array or vector
#' @param lambda lambda parameter of Box-Cox transformation
#' 
#' @return Inverse of Box-Cox heteroskedasticity transformation
#' 
#' @author Diego J. Pedregal
#' 
#' @rdname auxInvBoxCox
#' @export
auxInvBoxCox = function(y, lambda){
 if (abs(lambda) < 0.02){
  out = exp(y)
 } else if (lambda > 0.98) {
  out = y
 } else {
  out = (y * lambda + 1) ^ (1 / lambda)
 }
 return(out)
}
#' @title invBoxCox
#' @description Calculates inverse of Box-Cox transformation 
#' with confidence bands, calculated as const time the standard error
#'
#' @param y matrix, array or vector
#' @param yVar matrix, array or vector of variances of y
#' @param lambda lambda parameter of Box-Cox transformation
#' @param const number of standard error for confidence band
#' 
#' @return Inverse of Box-Cox heteroskedasticity transformation with confidence bands
#' 
#' @author Diego J. Pedregal
#' 
#' @rdname invBoxCox
#' @export
invBoxCox = function(y, yVar, lambda, const = 2){
 if (is.vector(y)){
  out = cbind(auxInvBoxCox(y, lambda), auxInvBoxCox(y - const * sqrt(yVar), lambda), 
              auxInvBoxCox(y + const * sqrt(yVar), lambda))
  colnames(out) = c("y", "conf-", "conf+")
 } else {
  out = list(y = auxInvBoxCox(y, lambda), "conf-" = auxInvBoxCox(y - const * sqrt(yVar), lambda), 
             "conf+" = auxInvBoxCox(y + const * sqrt(yVar), lambda))
 }
 return(out)
}

#' @title ARIMAestim
#' @description Estimates and forecasts ARIMA models
#'
#' @details \code{ARIMAestim} estimates and forecasts a time series using 
#' an ARIMA model
#'
#' @param m an object of type \code{ARIMA} created with \code{ARIMAforecast}
#' 
#' @return The same input object with the appropriate fields 
#' filled in, in particular:
#' \item{p}{Estimated parameters}
#' \item{yFor}{Forecasted values of output}
#' \item{yForV}{Variance of forecasted values of output}
#' \item{ySimul}{Bootstrap simulations for forecasting distribution evaluation}
#' 
#' @export
#' 
ARIMAestim = function(m){
    if (is.null(m$u))
        u = m$u
    else {
        if (is.vector(m$u)){
            u = matrix(m$u, 1, length(m$u))
        } else {
            nu = dim(m$u)
            u = as.numeric(m$u);
            u = matrix(u, nu[1], nu[2])
        }
    }
    output = ARIMAc("estimate", as.numeric(m$y), u, m$model, m$cnst, m$s, m$criterion,
                    m$h, m$verbose, m$lambda, m$maxOrders, m$bootstrap, m$nSimul,
                    m$fast, m$identDiff, m$identMethod)
    if (length(output) == 1){   # ERROR!!
            stop()
    } else {
            m$p = output$p
            m$ySimul = output$ySimul
            m$lambda = output$lambda
            m$model = output$orders
            m$cnst = output$cnst
            m$u = output$u
            m$BIC = output$BIC
            m$AIC = output$AIC
            m$AICc = output$AICc
            m$IC = output$IC
            # m$betaAug = output$betaAug
            # m$betaAugVar = output$betaAugVar
            if (!is.null(dim(m$u))){
                    lu = dim(m$u)[1]
                    if (lu == 1)
                            lu = dim(m$u)[2]
            } else {
                    lu = length(m$u)
            }
            if (lu > 0)
                m$h = lu - length(m$y)
            if (is.ts(m$y) && m$h > 0){
                fake = ts(c(m$y, NA), start = start(m$y), frequency = frequency(m$y))
                m$yFor = ts(output$yFor, start = end(fake), frequency = frequency(m$y))
                m$yForV = ts(output$yForV, start = end(fake), frequency = frequency(m$y))
                if (m$bootstrap)
                    m$ySimul = ts(output$ySimul, start = end(fake), frequency = frequency(m$y))
            } else if (m$h > 0) {
                m$yFor = output$yFor
                m$yForV = output$yForV
                m$ySimul = output$ySimul
            }
            return(m)
    }
}

#' @title ETSestim
#' @description Estimates and forecasts ETS models
#'
#' @details \code{ETSestim} estimates and forecasts a time series using an
#' an ETS model
#'
#' @param m an object of type \code{ETS} created with \code{ETSforecast}
#' 
#' @return The same input object with the appropriate fields 
#' filled in, in particular:
#' \item{p}{Estimated parameters}
#' \item{yFor}{Forecasted values of output}
#' \item{yForV}{Variance of forecasted values of output}
#' \item{ySimul}{Bootstrap simulations for forecasting distribution evaluation}
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{ETS}}, \code{\link{ETSforecast}}, \code{\link{ETSvalidate}},
#'          \code{\link{ETScomponents}}
#'          
#' @examples
#' \donttest{
#' m1 <- ETSsetup(log(gdp))
#' m1 <- ETSestim(m1)
#' }
#' @rdname ETSestim
#' @export
ETSestim = function(m){
    if (is.null(m$u))
        u = m$u
    else {
        if (is.vector(m$u)){
            u = matrix(m$u, 1, length(m$u))
        } else {
            nu = dim(m$u)
            u = as.numeric(m$u);
            u = matrix(u, nu[1], nu[2])
        }
    }
    output = ETSc("estimate", as.numeric(m$y), u, m$model, m$s, m$h,
                  m$criterion, m$armaIdent, m$identAll, m$forIntervals,
                  m$bootstrap, m$nSimul, m$verbose, m$lambda,
                  m$alphaL, m$betaL, m$gammaL, m$phiL, m$p0)
    if (length(output) == 1){   # ERROR!!
            stop()
    } else {
            m$model = output$model
            m$lambda = output$lambda
            m$p = output$p
            m$truep = output$truep
            lu = size(m$u)[1]
            if (lu > 0)
                m$h = lu - length(m$y)
            if (is.ts(m$y) && m$h > 0){
                fake = ts(c(m$y, NA), start = start(m$y), frequency = frequency(m$y))
                m$yFor = ts(output$yFor, start = end(fake), frequency = frequency(m$y))
                m$yForV = ts(output$yForV, start = end(fake), frequency = frequency(m$y))
                if (m$bootstrap)
                    m$ySimul = ts(output$ySimul, start = end(fake), frequency = frequency(m$y))
            } else if (m$h > 0) {
                m$yFor = output$yFor
                m$yForV = output$yForV
                m$ySimul = output$ySimul
            }
            m$criteria = output$criteria
            return(m)
    }
}

#' @title print.TETS
#' @description Prints a TOBIT TETS object
#'
#' @details See help of \code{TETS}.
#'
#' @param x Object of class \dQuote{TETS}.
#' @param ... Additional inputs to handle the way to print output.
#' 
#' @return No return value, called for side effects
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{TETS}}, \code{\link{TETSforecast}}, \code{\link{TETSvalidate}},
#'          \code{\link{TETScomponents}}, \code{\link{TETSestim}}
#'          
#' @examples
#' \donttest{
#' m1 <- TETSforecast(log(gdp))
#' print(m1)
#' }
#' @rdname print
#' @noRd
#' @export 
print.TETS = function(x, ...){
    if (length(x$table) < 3){
        x = TETSvalidate(x)
    }
    cat(x$table)
}
#' @title summary.TETS
#' @description Prints a TOBIT TETS object on screen
#'
#' @param object Object of class \dQuote{TETS}.
#' @param ... Additional inputs to function.
#' 
#' @details See help of \code{TETS}.
#'
#' @return No return value, called for side effects
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{TETS}}, \code{\link{TETSforecast}}, \code{\link{TETSvalidate}},
#'          \code{\link{TETScomponents}}, \code{\link{TETSestim}}
#'          
#' @examples
#' \donttest{
#' m1 <- TETSforecast(log(gdp))
#' summary(m1)
#' }
#' @rdname summary.TETS
#' @noRd
#' @export 
summary.TETS = function(object, ...){
    print(object)
}
#' @title plot.TETS
#' @description Plot components of TETS object
#'
#' @details See help of \code{TETS}.
#'
#' @param x Object of class \dQuote{TETS}.
#' @param ... Additional inputs to function.
#' 
#' @return No return value, called for side effects
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{TETS}}, \code{\link{TETSforecast}}, \code{\link{TETSvalidate}},
#'          \code{\link{TETScomponents}}, \code{\link{TETSestim}}
#'          
#' @examples
#' \donttest{
#' m1 <- TETSforecast(log(gdp))
#' plot(m1)
#' }
#' @rdname plot
#' @noRd
#' @export 
plot.TETS = function(x, ...){
    if (length(x$comp) < 2){
        x = TETScomponents(x)
    }
    if (is.ts(x$comp)){
        plot(x$comp, main = "Time Series Decomposition")
    } else {
        plot(ts(x$comp, frequency = x$s),
             main = "Time Series Decomposition")
    }
}
#' @title fitted.TETS
#' @description Fitted output values of TETS object
#'
#' @details See help of \code{TETS}.
#'
#' @param object Object of class \dQuote{TETS}.
#' @param ... Additional inputs to function.
#' 
#' @return Fitted values of TETS model
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{TETS}}, \code{\link{TETSforecast}}, \code{\link{TETSvalidate}},
#'          \code{\link{TETScomponents}}, \code{\link{TETSestim}}
#'          
#' @examples
#' \donttest{
#' m1 <- TETSforecast(log(gdp))
#' fitted(m1)
#' }
#' @rdname fitted
#' @noRd
#' @export 
fitted.TETS = function(object, ...){
    if (length(object$comp) < 2){
        object = TETScomponents(object)
    }
    return(object$com[, 2])
}
#' @title residuals.TETS
#' @description Residuals of TETS object
#'
#' @details See help of \code{TETS}.
#'
#' @param object Object of class \dQuote{TETS}.
#' @param ... Additional inputs to function.
#' 
#' @return Residuals of TETS model
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{TETS}}, \code{\link{TETSforecast}}, \code{\link{TETSvalidate}},
#'          \code{\link{TETScomponents}}, \code{\link{TETSestim}}
#'          
#' @examples
#' \donttest{
#' m1 <- TETSforecast(log(gdp))
#' residuals(m1)
#' }
#' @rdname residuals
#' @noRd
#' @export 
residuals.TETS = function(object, ...){
    if (length(object$comp) < 2){
        object = TETScomponents(object)
    }
    return(object$com[, 1])
}

#.First.lib <-
.onAttach <- function(libname, pkgname) {
    ver <- packageDescription("UComp")$Version
    txt <- c("\n",
             paste(sQuote("UComp"), "version:", ver),
             paste(sQuote("UComp"),
                   "is a package for time series modelling"),
             "using Unobserved Components, Exponential ",
             "smoothing, ARIMA models and Tobit ETS",
             "\n",
             "Author: Diego J. Pedregal",
             "Diego.Pedregal@uclm.es",
             "\n"
    )
    if (interactive() || getOption("verbose")) {
        msg = paste(strwrap(txt, indent = 4, exdent = 4), collapse = "\n")
        packageStartupMessage(msg)
    }
}

#' @title ARIMAsetup
#' @description Sets up ARIMA general models
#'
#' @details See help of \code{ARIMAforecast}.
#'
#' @param y a time series to forecast (it may be either a numerical vector or
#' a time series object). This is the only input required. If a vector, the additional
#' input \code{s} should be supplied compulsorily (see below).
#' @param u a matrix of input time series. If 
#' the output wanted to be forecast, matrix \code{u} should contain future values for inputs.
#' @param model the model to estimate. A vector c(p,d,q,P,D,Q) containing the model orders
#'               of an ARIMA(p,d,q)x(P,D,Q)_s model. A constant may be estimated with the
#'               cnst input.
#'               Use a NULL to automatically identify the ARIMA model.
#' @param cnst flag to include a constant in the model (TRUE/FALSE/NULL). Use NULL to estimate
#' @param s seasonal period of time series (1 for annual, 4 for quarterly, ...)
#' @param h forecast horizon. If the model includes inputs h is not used, the lenght of u is used instead.
#' @param criterion information criterion for identification stage ("aic", "bic", "aicc")
#' @param verbose intermediate estimation output (TRUE / FALSE)
#' @param lambda Box-Cox lambda parameter (NULL: estimate)
#' @param maxOrders a vector c(p,d,q,P,D,Q) containing the maximum orders of model orders 
#'                  to search for in the automatic identification
#' @param bootstrap use bootstrap simulation for predictive distributions
#' @param nSimul number of simulation runs for bootstrap simulation of predictive distributions
#' @param fast fast identification (avoids post-identification checks)
#' 
#' @author Diego J. Pedregal
#' 
#' @return An object of class \code{ARIMA}. It is a list with fields including all the inputs and
#'         the fields listed below as outputs. All the functions in this package fill in
#'         part of the fields of any \code{ARIMA} object as specified in what follows (function 
#'         \code{ARIMA} fills in all of them at once):
#' 
#' After running \code{ARIMAforecast} or \code{ARIMA}:
#' \item{p}{Estimated parameters}
#' \item{yFor}{Forecasted values of output}
#' \item{yForV}{Variance of forecasted values of output}
#' \item{ySimul}{Bootstrap simulations for forecasting distribution evaluation}
#' 
#' After running \code{ARIMAvalidate}:
#' \item{table}{Estimation and validation table}
#' 
#' @seealso \code{\link{ARIMA}}, \code{\link{ARIMAforecast}}, \code{\link{ARIMAvalidate}},
#'          
#' @examples
#' \donttest{
#' y <- log(AirPassengers)
#' m1 <- ARIMAsetup(y)
#' m1 <- ARIMAsetup(y, lambda = NULL)
#' }
#' @rdname ARIMAsetup
#' @export
ARIMAsetup = function(y, u = NULL, model = NULL, cnst = NULL, s = frequency(y), 
                      criterion = "bic", h = 2 * s, verbose = FALSE, lambda = 1, 
                      maxOrders = c(3, 2, 3, 2, 1, 2), bootstrap = FALSE, nSimul = 5000,
                      fast = FALSE){
        y = as.ts(y)
        if (s > 24)
                stop("Error: Data with period greater than 24 are not allowed!!")
        if (length(y) < s + 2 || length(y) < 8)
                stop("Error: Not enough data to estimate model!!")
        if (is.null(lambda))
                lambda = 9999.9
        if (typeof(cnst) == "logical" && cnst){
                cnst = 1.0
        } else if (typeof(cnst) == "logical" && !cnst){
                cnst = 0.0
        } else if (is.null(cnst)){
                cnst = 9999.9
        }
        out =  list(y = y,
                    u = u,
                    model = model,
                    cnst = cnst,
                    s = s,
                    h = h,
                    lambda = lambda,
                    bootstrap = bootstrap,
                    nSimul = nSimul,
                    verbose = verbose,
                    maxOrders = maxOrders,
                    criterion = criterion,
                    fast = fast,
                    identDiff = TRUE,
                    identMethod = "gm",
                    # outputs
                    error = NULL,
                    yFor = NULL,
                    yForV = NULL,
                    ySimul = NULL,
                    table = "",
                    p = NULL,
                    BIC = NA,
                    AIC = NA,
                    AICc = NA,
                    IC = NA,
                    v = NULL)
        return(structure(out, class = "ARIMA"))
}
#' @title ARIMAforecast
#' @description Estimates and forecasts ARIMA general univariate models
#'
#' @details \code{ARIMAforecast} is a function for modelling and forecasting univariate
#' time series with Autoregressive Integrated Moving Average (ARIMA) time series models. 
#' It sets up the model with a number of control variables that
#' govern the way the rest of functions in the package will work. It also estimates 
#' the model parameters by Maximum Likelihood and forecasts the data.
#'
#' @inheritParams ARIMA
#' 
#' @return An object of class \code{ARIMA}. It is a list with fields including all the inputs and
#'         the fields listed below as outputs. All the functions in this package fill in
#'         part of the fields of any \code{ARIMA} object as specified in what follows (function 
#'         \code{ARIMA} fills in all of them at once):
#' 
#' After running \code{ARIMAforecast} or \code{ARIMA}:
#' \item{p}{Estimated parameters}
#' \item{yFor}{Forecasted values of output}
#' \item{yForV}{Variance of forecasted values of output}
#' \item{ySimul}{Bootstrap simulations for forecasting distribution evaluation}
#' 
#' After running \code{ARIMAvalidate}:
#' \item{table}{Estimation and validation table}
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{ARIMA}}, \code{\link{ARIMAvalidate}},
#'          
#' @examples
#' \donttest{
#' y <- log(AirPassengers)
#' m1 <- ARIMAforecast(y)
#' m1 <- ARIMAforecast(y, lambda = NULL)
#' }
#' @rdname ARIMAforecast
#' @export
ARIMAforecast = function(y, u = NULL, model = NULL, cnst = NULL, s = frequency(y), 
                      criterion = "bic", h = 2 * s, verbose = FALSE, lambda = 1, 
                      maxOrders = c(3, 2, 3, 2, 1, 2), bootstrap = FALSE, nSimul = 5000,
                      fast = FALSE){
        m = ARIMAsetup(y, u, model, cnst, s, criterion, h, FALSE, lambda, maxOrders,
                       bootstrap, nSimul, fast)
        m = ARIMAestim(m)
        IC = m$IC
        if (is.null(IC) || !is.finite(IC)){
                IC = 1e10
        }
        if ((is.null(model) && !fast) || !is.finite(IC)){
                if (s == 1 && sum(abs(m$model[1 : 3] - c(0, 1, 1))) != 0){
                        # Yearly data
                        model = c(0, 1, 1, 0, 0, 0)
                        m1 = ARIMAsetup(y, u, model, cnst, s, criterion, h, FALSE, lambda, maxOrders,
                                        bootstrap, nSimul, fast)
                        m1 = ARIMAestim(m1)
                        if (is.finite(m1$IC) && m1$IC < IC) {
                                IC = m1$IC
                                m = m1
                        }
                        if (m$model[2] > 0){
                                model = m$model
                                model[2] = model[2] - 1
                                model[3] = min(model[3] + 1, maxOrders[3])
                                if (!all(model == c(0, 1, 1, 0, 0, 0))){
                                        m1 = ARIMAsetup(y, u, model, cnst, s, criterion, h, FALSE, lambda, maxOrders,
                                                        bootstrap, nSimul, fast)
                                        m1 = ARIMAestim(m1)
                                        if (is.finite(m1$IC) && m1$IC < IC){
                                                IC = m1$IC
                                                m = m1
                                        }
                                }
                        }
               } else if (s > 1){
                        # Non-yearly data
                        if (sum(abs(m$model - c(0, 1, 1, 0, 1, 1))) != 0){
                                model = c(0, 1, 1, 0, 1, 1)
                                m1 = ARIMAsetup(y, u, model, cnst, s, criterion, h, FALSE, lambda, maxOrders,
                                                bootstrap, nSimul, fast)
                                m1 = ARIMAestim(m1)
                                if (is.finite(m1$IC) && m1$IC < IC){
                                        IC = m1$IC
                                        m = m1
                                }
                        }
                        if (m$model[2] > 0 && m$model[5] > 0){
                                model = m$model
                                model[2] = model[2] - 1
                                model[1] = min(model[1] + 1, maxOrders[1])
                                if (!all(model == c(0, 1, 1, 0, 1, 1))){
                                        m1 = ARIMAsetup(y, u, model, cnst, s, criterion, h, FALSE, lambda, maxOrders,
                                                        bootstrap, nSimul, fast)
                                        m1 = ARIMAestim(m1)
                                        if (is.finite(m1$IC) && m1$IC < IC){
                                                IC = m1$IC
                                                m = m1
                                        }
                                }
                        }
                }
        }
        if (verbose)
                cat(m$table)
        return(m)
}
#' @title ARIMA
#' @description Runs all relevant functions for ARIMA modelling
#'
#' @details See help of \code{ARIMAforecast}.
#'
#' @param y a time series to forecast (it may be either a numerical vector or
#' a time series object). This is the only input required. If a vector, the additional
#' input \code{s} should be supplied compulsorily (see below).
#' @param u a matrix of input time series. If 
#' the output wanted to be forecast, matrix \code{u} should contain future values for inputs.
#' @param model the model to estimate. A vector c(p,d,q,P,D,Q) containing the model orders
#'               of an ARIMA(p,d,q)x(P,D,Q)_s model. A constant may be estimated with the
#'               cnst input.
#'               Use a NULL to automatically identify the ARIMA model.
#' @param cnst flag to include a constant in the model (TRUE/FALSE/NULL). Use NULL to estimate
#' @param s seasonal period of time series (1 for annual, 4 for quarterly, ...)
#' @param h forecast horizon. If the model includes inputs h is not used, the lenght of u is used instead.
#' @param criterion information criterion for identification stage ("aic", "bic", "aicc")
#' @param verbose intermediate estimation output (TRUE / FALSE)
#' @param lambda Box-Cox lambda parameter (NULL: estimate)
#' @param maxOrders a vector c(p,d,q,P,D,Q) containing the maximum orders of model orders 
#'                  to search for in the automatic identification
#' @param bootstrap use bootstrap simulation for predictive distributions
#' @param nSimul number of simulation runs for bootstrap simulation of predictive distributions
#' @param fast fast identification (avoids post-identification checks)
#' 
#' @author Diego J. Pedregal
#' 
#' @return An object of class \code{ARIMA}. It is a list with fields including all the inputs and
#'         the fields listed below as outputs. All the functions in this package fill in
#'         part of the fields of any \code{ARIMA} object as specified in what follows (function 
#'         \code{ARIMA} fills in all of them at once):
#' 
#' After running \code{ARIMAforecast} or \code{ARIMA}:
#' \item{p}{Estimated parameters}
#' \item{yFor}{Forecasted values of output}
#' \item{yForV}{Variance of forecasted values of output}
#' \item{ySimul}{Bootstrap simulations for forecasting distribution evaluation}
#' 
#' After running \code{ARIMAvalidate}:
#' \item{table}{Estimation and validation table}
#' 
#' @seealso \code{\link{ARIMAforecast}}, \code{\link{ARIMAvalidate}},
#'          
#' @examples
#' \donttest{
#' y <- log(AirPassengers)
#' m1 <- ARIMA(y)
#' m1 <- ARIMA(y, lambda = NULL)
#' }
#' @rdname ARIMA
#' @export
ARIMA = function(y, u = NULL, model = NULL, cnst = NULL, s = frequency(y), 
                 criterion = "bic", h = 2 * s, verbose = FALSE, lambda = 1, 
                 maxOrders = c(3, 2, 3, 2, 1, 2), bootstrap = FALSE, nSimul = 5000,
                 fast = FALSE){
        m = ARIMAsetup(y, u, model, cnst, s, criterion, h, FALSE, lambda, maxOrders,
                       bootstrap, nSimul, fast)
        m = ARIMAvalidate(m)
        IC = m$IC
        if (is.null(model) && !fast || !is.finite(IC)){
                if (s == 1 && sum(abs(m$model[1 : 3] - c(0, 1, 1))) != 0){
                        # Yearly data
                        model = c(0, 1, 1, 0, 0, 0)
                        m1 = ARIMAsetup(y, u, model, cnst, s, criterion, h, FALSE, lambda, maxOrders,
                                        bootstrap, nSimul, fast)
                        m1 = ARIMAvalidate(m1)
                        if (is.finite(m1$IC) && m1$IC < IC){
                                IC = m1$IC
                                m = m1
                        }
                        if (m$model[2] > 0){
                                model = m$model
                                model[2] = model[2] - 1
                                model[3] = min(model[3] + 1, maxOrders[3])
                                if (!all(model == c(0, 1, 1, 0, 0, 0))){
                                        m1 = ARIMAsetup(y, u, model, cnst, s, criterion, h, FALSE, lambda, maxOrders,
                                                        bootstrap, nSimul, fast)
                                        m1 = ARIMAvalidate(m1)
                                        if (is.finite(m1$IC) && m1$IC < IC){
                                                IC = m1$IC
                                                m = m1
                                        }
                                }
                        }
                } else if (s > 1){
                        # Non-yearly data
                        if (sum(abs(m$model - c(0, 1, 1, 0, 1, 1))) != 0){
                                model = c(0, 1, 1, 0, 1, 1)
                                m1 = ARIMAsetup(y, u, model, cnst, s, criterion, h, FALSE, lambda, maxOrders,
                                                bootstrap, nSimul, fast)
                                m1 = ARIMAvalidate(m1)
                                if (is.finite(m1$IC) && m1$IC < IC){
                                        IC = m1$IC
                                        m = m1
                                }
                        }
                        if (m$model[2] > 0 && m$model[5] > 0){
                                model = m$model
                                model[2] = model[2] - 1
                                model[1] = min(model[1] + 1, maxOrders[1])
                                if (!all(model == c(0, 1, 1, 0, 1, 1))){
                                        m1 = ARIMAsetup(y, u, model, cnst, s, criterion, h, FALSE, lambda, maxOrders,
                                                        bootstrap, nSimul, fast)
                                        m1 = ARIMAvalidate(m1)
                                        if (is.finite(m1$IC) && m1$IC < IC){
                                                IC = m1$IC
                                                m = m1
                                        }
                                }
                        }
                }
        }
        if (verbose)
                cat(m$table)
        return(m)
}

#' @title UCsetup
#' @description Sets up UC general univariate models
#'
#' @details See help of \code{UC}.
#'
#' @param y a time series to forecast (it may be either a numerical vector or
#' a time series object). This is the only input required. If a vector, the additional
#' input \code{periods} should be supplied compulsorily (see below).
#' @param u a matrix of external regressors included only in the observation equation. 
#' (it may be either a numerical vector or a time series object). If the output wanted 
#' to be forecast, matrix \code{u} should contain future values for inputs.
#' @param model the model to estimate. It is a single string indicating the type of 
#' model for each component. It allows two formats "trend/seasonal/irregular" or 
#' "trend/cycle/seasonal/irregular". The possibilities available for each component are:
#' \itemize{
#' \item Trend: ? / none / rw / irw / llt / dt / td; 
#' 
#' \item Seasonal: ? / none / equal / different;
#' 
#' \item Irregular: ? / none / arma(0, 0) / arma(p, q) - with p and q integer positive orders;
#'     
#' \item Cycles: ? / none / combination of positive or negative numbers. Positive numbers fix
#' the period of the cycle while negative values estimate the period taking as initial
#' condition the absolute value of the period supplied. Several cycles with positive or negative values are possible
#' and if a question mark is included, the model test for the existence of the cycles
#' specified. The following are valid examples with different meanings: 48, 48?, -48, -48?,
#' 48+60, -48+60, -48-60, 48-60, 48+60?, -48+60?, -48-60?, 48-60?.
#' }
#' @param outlier critical level of outlier tests. If NA it does not carry out any 
#' outlier detection (default). A positive value indicates the critical minimum
#' t test for outlier detection in any model during identification. Three types of outliers are
#' identified, namely Additive Outliers (AO), Level Shifts (LS) and Slope Change (SC).
#' @param stepwise stepwise identification procedure (TRUE / FALSE).
#' @param tTest augmented Dickey Fuller test for unit roots used in stepwise algorithm (TRUE / FALSE). 
#' The number of models to search for is reduced, depending on the result of this test.
#' @param p0 initial parameter vector for optimisation search.
#' @param h forecast horizon. If the model includes inputs h is not used, the lenght of u is used instead.
#' @param lambda Box-Cox transformation lambda, NULL for automatic estimation
#' @param criterion information criterion for identification ("aic", "bic" or "aicc").
#' @param periods vector of fundamental period and harmonics required.
#' @param verbose intermediate results shown about progress of estimation (TRUE / FALSE).
#' @param arma check for arma models for irregular components (TRUE / FALSE).
#' @param TVP vector of zeros and ones to indicate TVP parameters.
#' @param trendOptions trend models to select amongst (e.g., "rw/llt").
#' @param seasonalOptions seasonal models to select amongst (e.g., "none/differentt").
#' @param irregularOptions irregular models to select amongst (e.g., "none/arma(0,1)").
#' 
#' @author Diego J. Pedregal
#' 
#' @return An object of class \code{UComp}. It is a list with fields including all the inputs and
#'         the fields listed below as outputs. All the functions in this package fill in
#'         part of the fields of any \code{UComp} object as specified in what follows (function 
#'         \code{UC} fills in all of them at once):
#' 
#' After running \code{UCforecast}:
#' \itemize{
#' \item p:        Estimated parameters
#' \item v:        Estimated innovations (white noise in correctly specified models)
#' \item yFor:     Forecasted values of output
#' \item yForV:    Variance of forecasts
#' \item criteria: Value of criteria for estimated model
#' \item iter:     Number of iterations in estimation
#' \item grad:     Gradient at estimated parameters
#' \item covp:     Covariance matrix of parameters
#' }
#' 
#' After running \code{UCvalidate}:
#' \itemize{
#' \item table: Estimation and validation table
#' }
#' 
#' After running \code{UCcomponents}:
#' \itemize{
#' \item comp:      Estimated components in matrix form
#' \item compV:     Estimated components variance in matrix form
#' }
#' 
#' After running \code{UCfilter}, \code{UCsmooth} or  \code{UCdisturb}:
#' \itemize{
#' \item yFit:  Fitted values of output
#' \item yFitV: Estimated fitted values variance
#' \item a:     State estimates
#' \item P:     Variance of state estimates
#' \item aFor:  Forecasts of states
#' \item PFor:  Forecasts of states variances
#' }
#' 
#' After running \code{UCdisturb}:
#' \itemize{
#' \item eta: State perturbations estimates
#' \item eps: Observed perturbations estimates
#' }
#' 
#' Standard methods applicable to UComp objects are print, summary, plot,
#' fitted, residuals, logLik, AIC, BIC, coef, predict, tsdiag.
#' 
#' @seealso \code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, \code{\link{UCsmooth}}, 
#'          \code{\link{UCdisturb}}, \code{\link{UCcomponents}},
#'          \code{\link{UChp}}
#'          
#' @examples
#' \donttest{
#' y <- log(AirPassengers)
#' m1 <- UCsetup(y)
#' m1 <- UCsetup(y, outlier = 4)
#' m1 <- UCsetup(y, model = "llt/equal/arma(0,0)")
#' m1 <- UCsetup(y, model = "?/?/?/?")
#' m1 <- UCsetup(y, model = "llt/?/equal/?", outlier = 4)
#' }
#' @rdname UCsetup
#' @export
UCsetup = function(y, u = NULL, model = "?/none/?/?", h = 24, lambda = 1, outlier = 9999, tTest = FALSE, criterion = "aic",
                   periods = NA, verbose = FALSE, stepwise = FALSE, p0 = -9999.9, arma = FALSE,
                   TVP = NULL, trendOptions = "none/rw/llt/dt", seasonalOptions = "none/equal/different", irregularOptions = "none/arma(0,0)"){
    # Converting u vector to matrix
    if (length(size(u)) == 1 && size(u) > 0){
        u = matrix(u, 1, length(u))
    }
    if (length(size(u)) == 2 && size(u)[1] > size(u)[2])
        u = t(u)
    if (is.null(TVP) && !is.null(u))
        TVP = matrix(0, nrow(u))
    if (is.null(TVP))
        TVP = -9999.99
    if (is.null(u)){
        u = matrix(0, 1, 2)
    }
    if (is.null(lambda))
        lambda = 9999.9
    seas = frequency(y)
    # Checking periods
    if (is.ts(y) && any(is.na(periods)) && frequency(y) > 1){
        seas = frequency(y)
        periods = seas / (1 : floor(seas / 2))
    } else if (is.ts(y) && any(is.na(periods))){
        periods = 1
        seas = 1
    } else if (!is.ts(y) && any(is.na(periods))){
        stop("Input \"periods\" should be supplied!!")
    }
    rhos = rep(1, length(periods))
    if (length(y) < seas + 2 || length(y) < 8)
        stop("Error: Not enough data to estimate model!!")
    out = list(y = y,
               u = u,
               model = model,
               h = h,
               lambda = lambda,
               # Other less important
               arma =  arma,
               outlier = -abs(outlier),
               tTest = tTest,
               criterion = criterion,
               periods = periods,
               rhos = rhos,
               verbose = verbose,
               stepwise = stepwise,
               p0 = p0,
               criteria = NA,
               TVP = TVP,
               trendOptions = trendOptions,
               seasonalOptions = seasonalOptions,
               irregularOptions = irregularOptions,
               # Outputs
               seas = seas,
               comp = NA,
               compPlus = NA,
               compMinus = NA,
               coef = NA,
               p = NA,
               covp = NA,
               # grad = NA,
               v= NA,
               yFit = NA,
               yFor = NA,
               yFitV = NA,
               yForV = NA,
               a = NA,
               P = NA,
               eta = NA,
               eps = NA,
               estimOk = "Not estimated",
               table = NA)
               # iter = 0,
               # Other variables
               # hidden = list(d_t = 0,
               #               estimOk = "Not estimated",
               #               objFunValue = 0,
               #               innVariance = 1,
               #               nonStationaryTerms = NA,
               #               ns = NA,
               #               nPar = NA,
               #               harmonics = 0,
               #               constPar = NA,
               #               typePar = NA,
               #               cycleLimits = NA,
               #               typeOutliers = matrix(-1, 1, 2),
               #               truePar = NA,
               #               beta = NA,
               #               betaV = NA,
               #               seas = frequency(y),
               #               MSOE = FALSE,
               #               PTSnames = FALSE))
    return(structure(out, class = "UComp"))
}

#' @title UCforecast
#' @description Estimates and forecasts UC general univariate models
#'
#' @details \code{UCforecast} is a function for modelling and forecasting univariate
#' time series according to Unobserved Components models (UC). 
#' It sets up the model with a number of control variables that
#' govern the way the rest of functions in the package work. It also estimates 
#' the model parameters by Maximum Likelihood and forecasts the data.
#' Standard methods applicable to UComp objects are print, summary, plot,
#' fitted, residuals, logLik, AIC, BIC, coef, predict, tsdiag.
#'
#' @inheritParams UC
#' 
#' @return An object of class \code{UComp}. It is a list with fields including all the inputs and
#'         the fields listed below as outputs. All the functions in this package fill in
#'         part of the fields of any \code{UComp} object as specified in what follows (function 
#'         \code{UC} fills in all of them at once):
#' 
#' After running \code{UCforecast}:
#' \itemize{
#' \item p:        Estimated parameters
#' \item v:        Estimated innovations (white noise in correctly specified models)
#' \item yFor:     Forecasted values of output
#' \item yForV:    Forecasted values +- one standard error
#' \item criteria: Value of criteria for estimated model
#' \item iter:     Number of iterations in estimation
#' \item grad:     Gradient at estimated parameters
#' \item covp:     Covariance matrix of parameters
#' }
#' 
#' After running \code{UCvalidate}:
#' \itemize{
#' \item table: Estimation and validation table
#' }
#' 
#' After running \code{UCcomponents}:
#' \itemize{
#' \item comp:  Estimated components in matrix form
#' \item compV: Estimated components variance in matrix form
#' }
#' 
#' After running \code{UCfilter}, \code{UCsmooth} or  \code{UCdisturb}:
#' \itemize{
#' \item yFit:  Fitted values of output
#' \item yFitV: Variance of fitted values of output
#' \item a:     State estimates
#' \item P:     Variance of state estimates
#' \item aFor:  Forecasts of states
#' \item PFor:  Forecasts of states variances
#' }
#' 
#' After running \code{UCdisturb}:
#' \itemize{
#' \item eta: State perturbations estimates
#' \item eps: Observed perturbations estimates
#' }
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{UC}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, \code{\link{UCsmooth}}, 
#'          \code{\link{UCdisturb}}, \code{\link{UCcomponents}},
#'          \code{\link{UChp}}
#'          
#' @examples
#' \donttest{
#' y <- log(AirPassengers)
#' m1 <- UCforecast(y)
#' m1 <- UCforecast(y, model = "llt/equal/arma(0,0)")
#' }
#' @rdname UCforecast
#' @export
UCforecast = function(y, u = NULL, model = "?/none/?/?", h = 24, lambda = 1, outlier = 9999, tTest = FALSE, criterion = "aic",
                   periods = NA, verbose = FALSE, stepwise = FALSE, p0 = -9999.9, arma = FALSE,
                   TVP = NULL, trendOptions = "none/rw/llt/dt", seasonalOptions = "none/equal/different", irregularOptions = "none/arma(0,0)"){
    m = UCsetup(y, u, model, h, lambda, outlier, tTest, criterion, 
                periods, verbose, stepwise, p0, arma,
                TVP, trendOptions, seasonalOptions, irregularOptions)
    m = UCestim(m)
    if (is.null(u))
            m$u = u
    return(m)
}
#' @title UC
#' @description Runs all relevant functions for UC modelling
#'
#' @details \code{UC} is a function for modelling and forecasting univariate
#' time series according to Unobserved Components models (UC). 
#' It sets up the model with a number of control variables that
#' govern the way the rest of functions in the package work. It also estimates 
#' the model parameters by Maximum Likelihood, forecasts the data, performs smoothing,
#' estimates model disturbances, estimates components and shows statistical diagnostics.
#' Standard methods applicable to UComp objects are print, summary, plot,
#' fitted, residuals, logLik, AIC, BIC, coef, predict, tsdiag.
#'
#' @param y a time series to forecast (it may be either a numerical vector or
#' a time series object). This is the only input required. If a vector, the additional
#' input \code{periods} should be supplied compulsorily (see below).
#' @param u a matrix of external regressors included only in the observation equation. 
#' (it may be either a numerical vector or a time series object). If the output wanted 
#' to be forecast, matrix \code{u} should contain future values for inputs.
#' @param model the model to estimate. It is a single string indicating the type of 
#' model for each component. It allows two formats "trend/seasonal/irregular" or 
#' "trend/cycle/seasonal/irregular". The possibilities available for each component are:
#' \itemize{
#' \item Trend: ? / none / rw / irw / llt / dt / td; 
#' 
#' \item Seasonal: ? / none / equal / different;
#' 
#' \item Irregular: ? / none / arma(0, 0) / arma(p, q) - with p and q integer positive orders;
#'     
#' \item Cycles: ? / none / combination of positive or negative numbers. Positive numbers fix
#' the period of the cycle while negative values estimate the period taking as initial
#' condition the absolute value of the period supplied. Several cycles with positive or negative values are possible
#' and if a question mark is included, the model test for the existence of the cycles
#' specified. The following are valid examples with different meanings: 48, 48?, -48, -48?,
#' 48+60, -48+60, -48-60, 48-60, 48+60?, -48+60?, -48-60?, 48-60?.
#' }
#' @param outlier critical level of outlier tests. If NA it does not carry out any 
#' outlier detection (default). A positive value indicates the critical minimum
#' t test for outlier detection in any model during identification. Three types of outliers are
#' identified, namely Additive Outliers (AO), Level Shifts (LS) and Slope Change (SC).
#' @param stepwise stepwise identification procedure (TRUE / FALSE).
#' @param tTest augmented Dickey Fuller test for unit roots used in stepwise algorithm (TRUE / FALSE). 
#' The number of models to search for is reduced, depending on the result of this test.
#' @param p0 initial parameter vector for optimisation search.
#' @param h forecast horizon. If the model includes inputs h is not used, the lenght of u is used instead.
#' @param lambda Box-Cox transformation lambda, NULL for automatic estimation
#' @param criterion information criterion for identification ("aic", "bic" or "aicc").
#' @param periods vector of fundamental period and harmonics required.
#' @param verbose intermediate results shown about progress of estimation (TRUE / FALSE).
#' @param arma check for arma models for irregular components (TRUE / FALSE).
#' @param TVP vector of zeros and ones to indicate TVP parameters.
#' @param trendOptions trend models to select amongst (e.g., "rw/llt").
#' @param seasonalOptions seasonal models to select amongst (e.g., "none/differentt").
#' @param irregularOptions irregular models to select amongst (e.g., "none/arma(0,1)").
#' 
#' @author Diego J. Pedregal
#' 
#' @return An object of class \code{UComp}. It is a list with fields including all the inputs and
#'         the fields listed below as outputs. All the functions in this package fill in
#'         part of the fields of any \code{UComp} object as specified in what follows (function 
#'         \code{UC} fills in all of them at once):
#' 
#' After running \code{UCforecast} or \code{UCestim}:
#' \itemize{
#' \item p:        Estimated parameters
#' \item v:        Estimated innovations (white noise in correctly specified models)
#' \item yFor:     Forecasted values of output
#' \item yForV:    Forecasted values +- one standard error
#' \item criteria: Value of criteria for estimated model
#' \item iter:     Number of iterations in estimation
#' \item grad:     Gradient at estimated parameters
#' \item covp:     Covariance matrix of parameters
#' }
#' 
#' After running \code{UCvalidate}:
#' \itemize{
#' \item table: Estimation and validation table
#' }
#' 
#' After running \code{UCcomponents}:
#' \itemize{
#' \item comp:  Estimated components in matrix form
#' \item compV: Estimated components variance in matrix form
#' }
#' 
#' After running \code{UCfilter}, \code{UCsmooth} or  \code{UCdisturb}:
#' \itemize{
#' \item yFit:  Fitted values of output
#' \item yFitV: Variance of fitted values of output
#' \item a:     State estimates
#' \item P:     Variance of state estimates
#' \item aFor:  Forecasts of states
#' \item PFor:  Forecasts of states variances
#' }
#' 
#' After running \code{UCdisturb}:
#' \itemize{
#' \item eta: State perturbations estimates
#' \item eps: Observed perturbations estimates
#' }
#' 
#' @seealso \code{\link{UC}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, \code{\link{UCsmooth}}, 
#'          \code{\link{UCdisturb}}, \code{\link{UCcomponents}},
#'          \code{\link{UChp}}
#'          
#' @examples
#' \donttest{
#' y <- log(AirPassengers)
#' m1 <- UC(y)
#' m1 <- UC(y, model = "llt/different/arma(0,0)")
#' }
#' @rdname UC
#' @export
UC = function(y, u = NULL, model = "?/none/?/?", h = 24, lambda = 1, outlier = 9999, tTest = FALSE, criterion = "aic",
              periods = NA, verbose = FALSE, stepwise = FALSE, p0 = -9999.9, arma = FALSE,
              TVP = NULL, trendOptions = "none/rw/llt/dt", seasonalOptions = "none/equal/different", irregularOptions = "none/arma(0,0)"){
    m = UCsetup(y, u, model, h, lambda, outlier, tTest, criterion, 
                periods, verbose, stepwise, p0, arma,
                TVP, trendOptions, seasonalOptions, irregularOptions)
    m = UCcommand("all", m)
    if (is.null(u))
            m$u = u
    if (verbose)
        cat(m$table)
    # m = UCestim(m)
    # if (m$model == "error")
    # #     return(m)
    # if (verbose)
    #     m = UCvalidate(m, verbose)
    # else
    #     m = UCcomponents(m)
    return(m)
}

#' @title TETSsetup
#' @description Sets up TOBIT TETS general univariate models
#'
#' @details See help of \code{TETSforecast}.
#'
#' @param y a time series to forecast (it may be either a numerical vector or
#' a time series object). This is the only input required. If a vector, the additional
#' input \code{s} should be supplied compulsorily (see below).
#' @param u a matrix of input time series. If 
#' the output wanted to be forecast, matrix \code{u} should contain future values for inputs.
#' @param model the model to estimate. It is a single string indicating the type of 
#' model for each component with one or two letters:
#' \itemize{
#' \item Error: ? / A
#' 
#' \item Trend: ? / N / A / Ad
#' 
#' \item Seasonal: ? / N / A
#' 
#' }
#' @param s seasonal period of time series (1 for annual, 4 for quarterly, ...)
#' @param h forecast horizon. If the model includes inputs h is not used, the lenght of u is used instead.
#' @param criterion information criterion for identification ("aic", "bic" or "aicc").
#' @param forIntervals estimate forecasting intervals (TRUE / FALSE)
#' @param bootstrap use bootstrap simulation for predictive distributions
#' @param nSimul number of simulation runs for bootstrap simulation of predictive distributions
#' @param verbose intermediate estimation output (TRUE / FALSE)
#' @param alphaL constraints limits for alpha parameter
#' @param betaL constraints limits for beta parameter
#' @param gammaL constraints limits for gamma parameter
#' @param phiL constraints limits for phi parameter
#' @param p0 initial values for parameter search (alpha, beta, phi, gamma, sigma2) with consraints:
#' @param Ymin scalar or vector of time varying censoring values from below
#' @param Ymax scalar or vector of time varying censoring values from above
#' 
#' \itemize{ 
#' \item 0 < alpha < 1
#' 
#' \item 0 < beta < alpha
#' 
#' \item 0 < phi < 1
#' 
#' \item 0 < gamma < 1 - alpha
#' 
#' \item sigma2 > 0
#' }
#' 
#' @return An object of class \code{TETS}. It is a list with fields including all the inputs and
#'         the fields listed below as outputs. All the functions in this package fill in
#'         part of the fields of any \code{TETS} object as specified in what follows (function 
#'         \code{TETS} fills in all of them at once):
#' 
#' After running \code{TETSforecast}:
#' \item{p}{Estimated parameters}
#' \item{criteria}{Values for estimation criteria (LogLik, AIC, BIC, AICc)}
#' \item{yFor}{Forecasted values of output}
#' \item{yForV}{Variance of forecasted values of output}
#' \item{ySimul}{Bootstrap simulations for forecasting distribution evaluation}
#' 
#' After running \code{TETSvalidate}:
#' \item{table}{Estimation and validation table}
#' \item{comp}{Estimated components in matrix form}
#' 
#' After running \code{TETScomponents}:
#' \item{comp}{Estimated components in matrix form}
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{TETS}}, \code{\link{TETSforecast}}, \code{\link{TETSvalidate}},
#'          \code{\link{TETScomponents}}
#'          
#' @examples
#' \donttest{
#' y <- log(AirPassengers)
#' m1 <- TETSsetup(y)
#' m1 <- TETSsetup(y,  model = "???")
#' m1 <- TETSsetup(y,  model = "?AA")
#' }
#' @rdname TETSsetup
#' @export
TETSsetup = function(y, u = NULL, model = "???", s = frequency(y), h = 2 * s, criterion = "aicc", 
                    forIntervals = FALSE, bootstrap = FALSE, nSimul = 5000, verbose = FALSE,
                    alphaL = c(0, 1), betaL = alphaL, gammaL = alphaL, 
                    phiL = c(0.8, 0.98), p0 = -99999, Ymin = -Inf, Ymax = Inf){
    out = ETSsetup(y, u, model, s, h, criterion, 1, FALSE, FALSE, forIntervals,
                   bootstrap, nSimul, verbose, alphaL, betaL, gammaL, phiL, p0)
    out$Ymin = Ymin
    out$Ymax = Ymax
    if (min(Ymax - y, na.rm = TRUE) != 0 && min(y - Ymin, na.rm = TRUE) != 0){
        return(structure(out, class = "ETS"))
    } else {
        return(structure(out, class = "TETS"))
    }
    return(out)
}
#' @title TETSforecast
#' @description Estimates and forecasts TOBIT TETS general univariate models
#'
#' @details \code{TETSforecast} is a function for modelling and forecasting univariate
#' time series with TOBIT ExponenTial Smoothing (TETS) time series models. 
#' It sets up the model with a number of control variables that
#' govern the way the rest of functions in the package will work. It also estimates 
#' the model parameters by Maximum Likelihood and forecasts the data.
#'
#' @inheritParams TETS
#' 
#' @return An object of class \code{TETS}. It is a list with fields including all the inputs and
#'         the fields listed below as outputs. All the functions in this package fill in
#'         part of the fields of any \code{TETS} object as specified in what follows (function 
#'         \code{TETS} fills in all of them at once):
#' 
#' After running \code{TETSforecast}:
#' \item{p}{Estimated parameters}
#' \item{criteria}{Values for estimation criteria (LogLik, AIC, BIC, AICc)}
#' \item{yFor}{Forecasted values of output}
#' \item{yForV}{Variance of forecasted values of output}
#' \item{ySimul}{Bootstrap simulations for forecasting distribution evaluation}
#' 
#' After running \code{TETSvalidate}:
#' \item{table}{Estimation and validation table}
#' \item{comp}{Estimated components in matrix form}
#' 
#' After running \code{TETScomponents}:
#' \item{comp}{Estimated components in matrix form}
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{TETS}}, \code{\link{TETSvalidate}},
#'          \code{\link{TETScomponents}}
#'          
#' @examples
#' \donttest{
#' y <- log(AirPassengers)
#' m1 <- TETSforecast(y)
#' m1 <- TETSforecast(y, model = "A?A")
#' }
#' @rdname TETSforecast
#' @export
TETSforecast = function(y, u = NULL, model = "???", s = frequency(y), h = max(2 * s, 6), criterion = "aicc", 
                    forIntervals = FALSE, bootstrap = FALSE, nSimul = 5000, verbose = FALSE,
                    alphaL = c(0, 1), betaL = alphaL, gammaL = alphaL, 
                    phiL = c(0.8, 0.98), p0 = -99999, Ymin = -Inf, Ymax = Inf){
    m1 = TETSsetup(y, u, model, s, h, criterion, forIntervals,
                  bootstrap, nSimul, verbose, alphaL, betaL, gammaL, phiL, p0, Ymin, Ymax)
    if (inherits(m1, "ETS"))
        m1 = ETSestim(m1)
    else
        m1 = TETSestim(m1)
    if (verbose)
            cat(m1$table)
    return(m1)
}
#' @title TETS
#' @description Runs all relevant functions for TETS modelling
#'
#' @details See help of \code{TETSforecast}.
#'
#' @param y a time series to forecast (it may be either a numerical vector or
#' a time series object). This is the only input required. If a vector, the additional
#' input \code{s} should be supplied compulsorily (see below).
#' @param u a matrix of input time series. If 
#' the output wanted to be forecast, matrix \code{u} should contain future values for inputs.
#' @param model the model to estimate. It is a single string indicating the type of 
#' model for each component with one or two letters:
#' \itemize{
#' \item Error: ? / A
#' 
#' \item Trend: ? / N / A / Ad
#' 
#' \item Seasonal: ? / N / A
#' 
#' }
#' @param s seasonal period of time series (1 for annual, 4 for quarterly, ...)
#' @param h forecast horizon. If the model includes inputs h is not used, the lenght of u is used instead.
#' @param criterion information criterion for identification ("aic", "bic" or "aicc").
#' @param forIntervals estimate forecasting intervals (TRUE / FALSE)
#' @param bootstrap use bootstrap simulation for predictive distributions
#' @param nSimul number of simulation runs for bootstrap simulation of predictive distributions
#' @param verbose intermediate estimation output (TRUE / FALSE)
#' @param alphaL constraints limits for alpha parameter
#' @param betaL constraints limits for beta parameter
#' @param gammaL constraints limits for gamma parameter
#' @param phiL constraints limits for phi parameter
#' @param p0 initial values for parameter search (alpha, beta, phi, gamma, sigma2) with consraints:
#' @param Ymin scalar or vector of time varying censoring values from below
#' @param Ymax scalar or vector of time varying censoring values from above
#' 
#' \itemize{ 
#' \item 0 < alpha < 1
#' 
#' \item 0 < beta < alpha
#' 
#' \item 0 < phi < 1
#' 
#' \item 0 < gamma < 1 - alpha
#' 
#' \item sigma2 > 0
#' }
#' 
#' @author Diego J. Pedregal
#' 
#' @return An object of class \code{TETS}. It is a list with fields including all the inputs and
#'         the fields listed below as outputs. All the functions in this package fill in
#'         part of the fields of any \code{TETS} object as specified in what follows (function 
#'         \code{TETS} fills in all of them at once):
#' 
#' After running \code{TETSforecast}:
#' \item{p}{Estimated parameters}
#' \item{criteria}{Values for estimation criteria (LogLik, AIC, BIC, AICc)}
#' \item{yFor}{Forecasted values of output}
#' \item{yForV}{Variance of forecasted values of output}
#' \item{ySimul}{Bootstrap simulations for forecasting distribution evaluation}
#' 
#' After running \code{TETSvalidate}:
#' \item{table}{Estimation and validation table}
#' \item{comp}{Estimated components in matrix form}
#' 
#' After running \code{TETScomponents}:
#' \item{comp}{Estimated components in matrix form}
#' 
#' @seealso \code{\link{TETSforecast}}, \code{\link{TETSvalidate}},
#'          \code{\link{TETScomponents}}
#'          
#' @examples
#' \donttest{
#' y <- log(AirPassengers)
#' m1 <- TETS(y)
#' m1 <- TETS(y, model = "MAM")
#' }
#' @rdname TETS
#' @export
TETS = function(y, u = NULL, model = "???", s = frequency(y), h = 2 * s, criterion = "aicc", 
               forIntervals = FALSE, bootstrap = FALSE, nSimul = 5000, verbose = FALSE,
               alphaL = c(0, 1), betaL = alphaL, gammaL = alphaL, 
               phiL = c(0.8, 0.98), p0 = -99999, Ymin = -Inf, Ymax = Inf){
    m1 = TETSsetup(y, u, model, s, h, criterion, forIntervals,
                  bootstrap, nSimul, verbose, alphaL, betaL, gammaL, phiL, p0, Ymin, Ymax)
    if (inherits(m1, "ETS")){
        m1 = ETSvalidate(m1)
    } else {
        m1 = TETSvalidate(m1)
    }
    m1$v = m1$comp[, 1]
    if (verbose)
            cat(m1$table)
    return(m1)
}

#' Sales index for large retailers in Spain
#'
#' Sales index for food of large retailers in Spain
#'
#' @format Time series objects.
#' \describe{
#'   Monthly data from January 1995 to December 2019
#'   
#'   https://portal.mineco.gob.es/es-es/economiayempresa/EconomiaInformesMacro/Paginas/bdsice.aspx
#' }
#'         
#' @return No return value, called for side effects
#' 
#' @examples
#' \donttest{
#'  sales
#' }
"sales"

#' US GDP
#'
#' Seasonally adjusted quarterly US real gross domestic product (USgdp).
#'
#' @format Time series objects.
#' \describe{
#'   Quarterly data from 1962 to 2019
#' }
#'
#' @source \href{https://www.nber.org/}{USgdp}
#'         
#' @return No return value, called for side effects
#' 
#' @examples
#' \donttest{
#'  USgdp
#' }
"USgdp"

#' OECD GDP
#'
#' Seasonally adjusted quarterly OECD real gross domestic product (OECDgdp).
#'
#' @format Time series objects.
#' \describe{
#'   Quarterly data from 1962 to 2019
#'   
#'   https://portal.mineco.gob.es/es-es/economiayempresa/EconomiaInformesMacro/Paginas/bdsice.aspx
#' }
#'
#' @return No return value, called for side effects
#' 
#' @examples
#' \donttest{
#'  OECDgdp
#' }
"OECDgdp"

#' Methane concentration at Cape Grim in Australia
#'
#' Methane concentration at Cape Grim in Australia (ch4).
#'
#' @format Time series objects.
#' \describe{
#'   Monthly data from January 1992 to December 2019
#' }
#'
#' @source \href{https://www.csiro.au/en/research/natural-environment/atmosphere/Latest-greenhouse-gas-data}{CH4 data}
#'         
#' @return No return value, called for side effects
#' 
#' @examples
#' \donttest{
#'  ch4
#' }
"ch4"

#' Airpassengers in Spain
#' 
#' Foreign arrivals by air in Spain in thousands of passengers (airpas).
#'
#' @format Time series objects.
#' \describe{
#'   Monthly data from 1969
#'   
#'   https://portal.mineco.gob.es/es-es/economiayempresa/EconomiaInformesMacro/Paginas/bdsice.aspx
#' }
#' 
#' @return No return value, called for side effects
#' 
#' @examples
#' \donttest{
#'  airpas
#' }
"airpas"

#' Spanish GDP
#'
#' Quarterly real Spanish Gross Domestic Product (gdp)
#'
#' @format Time series objects.
#' \describe{
#'   Quarterly since 1995
#'   
#'   https://portal.mineco.gob.es/es-es/economiayempresa/EconomiaInformesMacro/Paginas/bdsice.aspx
#' }
#'
#' @return No return value, called for side effects
#' 
#' @examples
#' \donttest{
#'  gdp
#' }
"gdp"

#' Spanish Industrial Production Index
#' 
#' Spanish Industrial Production Index (ipi).
#'
#'
#' @format Objeto time series.
#' \describe{
#'   Monthly since 1975
#'   
#'   https://portal.mineco.gob.es/es-es/economiayempresa/EconomiaInformesMacro/Paginas/bdsice.aspx
#' }
#'
#' @return No return value, called for side effects
#' 
#' @examples
#' \donttest{
#'  ipi
#' }
"ipi"

#' @title ARIMAvalidate
#' @description Shows a table of estimation and diagnostics results for ARIMA models
#'
#' @param m an object of type \code{ARIMA} created with \code{ARIMAforecast}
#' 
#' @return The same input object with the appropriate fields 
#' filled in, in particular:
#' \item{table}{Estimation and validation table}
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{ARIMA}}, \code{\link{ARIMAforecast}}, \code{\link{ARIMAvalidate}},
#'          
#' @examples
#' \donttest{
#' m1 <- ARIMAforecast(log(gdp))
#' m1 <- ARIMAvalidate(m1)
#' }
#' @rdname ARIMAvalidate
#' @export
ARIMAvalidate = function(m){
    if (is.null(m$u))
        u = m$u
    else {
        if (is.vector(m$u)){
            u = matrix(m$u, 1, length(m$u))
        } else {
            nu = dim(m$u)
            u = as.numeric(m$u);
            u = matrix(u, nu[1], nu[2])
        }
    }
    if (sum(m$model) == 0)
            m$model = NULL
    output = ARIMAc("validate", as.numeric(m$y), u, m$model, m$cnst, m$s, m$criterion, 
                    m$h, m$verbose, m$lambda, m$maxOrders, m$bootstrap, m$nSimul,
                    m$fast, m$identDiff, m$identMethod)
    if (length(output) == 1){   # ERROR!!
            stop()
    }
    m$table = output$table
    m$error = output$error
    m$v = output$error
    m$p = output$p
    m$ySimul = output$ySimul
    m$lambda = output$lambda
    m$model = output$orders
    m$cnst = output$cnst
    m$u = output$u
    m$BIC = output$BIC
    m$AIC = output$AIC
    m$AICc = output$AICc
    m$IC = output$IC
    # m$betaAug = output$betaAug
    # m$betaAugVar = output$betaAugVar
    if (!is.null(dim(m$u))){
            lu = dim(m$u)[2]
    } else {
            lu = length(m$u)
    }
    if (lu > 0)
            m$h = lu - length(m$y)
    if (is.ts(m$y) && m$h > 0){
            fake = ts(c(m$y, NA), start = start(m$y), frequency = frequency(m$y))
            m$yFor = ts(output$yFor, start = end(fake), frequency = frequency(m$y))
            m$yForV = ts(output$yForV, start = end(fake), frequency = frequency(m$y))
            if (m$bootstrap)
                    m$ySimul = ts(output$ySimul, start = end(fake), frequency = frequency(m$y))
    } else if (m$h > 0) {
            m$yFor = output$yFor
            m$yForV = output$yForV
            m$ySimul = output$ySimul
    }
    # Buscando test heterocedasticidad con valor p NaN
    # ind = which(grepl("nan", m$table))
    # if (any(ind)){
    #     for (i in 1 : length(ind)){
    #         line = m$table[ind[i]]
    #         df = as.numeric(substr(line, 9, 12))
    #         Fstat = as.numeric(substr(line, 15, 31))
    #         pval = round(pf(Fstat, df, df), 4)
    #         line = gsub("   nan", pval, line)
    #         m$table[ind[i]] = line
    #     }
    # }
    # if (m$verbose)
    #     cat(m$table)
    return(m)
}
    

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

UCompC <- function(commands, ys, us, models, hs, lambdas, outliers, tTests, criterions, periodss, rhoss, verboses, stepwises, p0s, armas, TVPs, seass, trendOptionss, seasonalOptionss, irregularOptionss) {
    .Call(`_UComp_UCompC`, commands, ys, us, models, hs, lambdas, outliers, tTests, criterions, periodss, rhoss, verboses, stepwises, p0s, armas, TVPs, seass, trendOptionss, seasonalOptionss, irregularOptionss)
}

ETSc <- function(commands, ys, us, models, ss, hs, criterions, armaIdents, identAlls, forIntervalss, bootstraps, nSimuls, verboses, lambdas, alphaLs, betaLs, gammaLs, phiLs, p0s) {
    .Call(`_UComp_ETSc`, commands, ys, us, models, ss, hs, criterions, armaIdents, identAlls, forIntervalss, bootstraps, nSimuls, verboses, lambdas, alphaLs, betaLs, gammaLs, phiLs, p0s)
}

ARIMAc <- function(commands, ys, us, orderss, cnsts, ss, criterions, hs, verboses, lambdas, maxOrderss, bootstraps, nSimuls, fasts, identDiffs, identMethods) {
    .Call(`_UComp_ARIMAc`, commands, ys, us, orderss, cnsts, ss, criterions, hs, verboses, lambdas, maxOrderss, bootstraps, nSimuls, fasts, identDiffs, identMethods)
}

TETSc <- function(commands, ys, us, models, ss, hs, criterions, armaIdents, identAlls, forIntervalss, bootstraps, nSimuls, verboses, lambdas, alphaLs, betaLs, gammaLs, phiLs, p0s, Ymins, Ymaxs) {
    .Call(`_UComp_TETSc`, commands, ys, us, models, ss, hs, criterions, armaIdents, identAlls, forIntervalss, bootstraps, nSimuls, verboses, lambdas, alphaLs, betaLs, gammaLs, phiLs, p0s, Ymins, Ymaxs)
}

#' @title UCcommand
#' @description Auxiliar function for UC modeling
#'
#' @param command Command to execute: "forecast", "validate", "filter", "smooth", "disturb", "components", "all"
#' 
#' @param sys A \code{UComp} object created with \code{UC}
#' 
#' @return The input \code{UComp} object with the appropriate fields filled in
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, 
#'          \code{\link{UCsmooth}}, \code{\link{UCcomponents}}, \code{\link{UCdisturb}}
#'          
#' @examples
#' \donttest{
#' cycle <- UChp(USgdp)
#' plot(cycle)
#' }
#' @rdname UCcommand
#' @export
UCcommand = function(command, sys) {
    isnullu = FALSE
    if (is.null(sys$u)) {
        isnullu = TRUE
        sys$u = matrix(0, 1, 2)
    }
    if (sys$model == "error") {
        stop("ERROR: Model not specified or estimation failed!!\n")
        return(sys)
    }
    if (is.ts(sys$y)){
        seas = frequency(sys$y)
        y = as.numeric(sys$y)
    } else {
        y = sys$y
    }
    if (is.ts(sys$u)){
        u = as.numeric(sys$u)
    } else {
        u = sys$u
    }
    nu = dim(u)[2]
    kInitial = dim(u)[1]
    if (nu == 2){
        nu = length(sys$y) + sys$h
        kInitial = 0
    }
    if (any(sys$rhos < 0)) {
            if (any(sys$rhos == 1)) {
                    sys$periods = 1
                    sys$rhos = 1
            } else {
                    sys$periods = seas / (1 : floor(seas / 2))
                    sys$rhos = rep(1, length(sys$periods))
            }
    }
    output = UCompC(command, y, u, sys$model, sys$h, sys$lambda, sys$outlier, sys$tTest,
                    sys$criterion, sys$periods, sys$rhos, sys$verbose, sys$stepwise, sys$p0, sys$arma,
                    sys$TVP, sys$seas, sys$trendOptions, sys$seasonalOptions, sys$irregularOptions)
    if (output$model == "error"){
        sys$model = "error"
        return(sys);
    }
    if (is.ts(sys$y)){
        fY = frequency(sys$y)
        sY = start(sys$y, frequency = fY)
        aux = ts(matrix(NA, length(sys$y) + 1, 1), sY, frequency = fY)
        if (length(output$yFor > 0)){
            sys$yFor = ts(output$yFor, end(aux), frequency = fY)
            sys$yForV = ts(output$yForV, end(aux), frequency = fY)
        }
    } else {
        if (length(output$yFor > 0)){
            sys$yFor = output$yFor
            sys$yForV = output$yForV
        }
    }
    # Convert to R list
    # if (grepl("?", sys$model, fixed = TRUE)){
        sys$model = output$model
    # }
    sys$h = output$h
    sys$hidden$estimOk = output$estimOk
    sys$criteria = output$criteria;
    sys$lambda = output$lambda
    sys$periods = output$periods
    sys$rhos = output$rhos
    sys$p = as.vector(output$p)
    nPar = length(sys$p)
    names(sys$p) = output$parNames[1 : nPar]
    # sys$p0 = output$p0
    # Adapting periods of estimated cycles, rhos and model
    # ind = which(sys$rhos < 0)
    # if (length(ind) > 0) {
    #     # Changing cycle model
    #     pCycle = sys$periods[ind]
    #     mCycle = NULL
    #     for (i in 1 : length(pCycle)) {
    #         mCycle = paste0(mCycle, "+", round(pCycle[i], 2))
    #     }
    #     sys$model = sub("(/)[^/]+(/)", paste0("\\1", mCycle, "\\2"), sys$model)
    # }
 
    # if (any(sys$periods < 0)) {
    #     sys$model = MODEL
    #     sys$periods = PERIODS
    # }
    # print("line 65 UCestim")
    # print(sys$model)
    # print(MODEL)
    # print(sys$periods)
    if (is.ts(sys$y)){
        nNan = min(which(!is.na(y))) - 1
        initial = start(sys$y, frequency = frequency(y))
        if (nNan > 0){
            initial = end(ts(1 : (nNan + 1), start=initial, frequency=frequency(sys$y)))
        }
    }
    if (command == "validate" || command == "all") {
        sys$table = output$table
        sys$v = output$v
        sys$covp = output$covp
        sys$coef = as.vector(output$coef)
        nPar = length(sys$coef)
        rownames(sys$covp) = output$parNames[1 : dim(sys$covp)[1]]
        colnames(sys$covp) = output$parNames[1 : dim(sys$covp)[1]]
        names(sys$coef) = output$parNames[1 : nPar]
    }
    if (command == "components" || command == "all") {
        sys$comp = output$comp
        sys$compV = output$compV
        m = output$m  # + nCycles
        if (dim(u)[1] == 1 && dim(u)[2] == 2){
            k = 0
        } else {
            k = dim(u)[1]
        }
        nCycles = m - k - 4
        # Re-building matrices to their original sizes
        n = length(sys$comp) / m
        if (is.ts(sys$y)){
            sys$comp = ts(t(matrix(sys$comp, m, n)), start= initial, frequency = frequency(sys$y))
            sys$compV = ts(t(matrix(sys$compV, m, n)), start = initial, frequency = frequency(sys$y))
        } else {
            sys$comp = t(matrix(sys$comp, m, n))
            sys$compV = t(matrix(sys$compV, m, n))
        }
        if (length(size(sys$comp)) == 1){
            if (is.ts(sys$y)){
                sys$comp = ts(matrix(sys$comp, n + sys$h, 1), start = initial, frequency = frequency(sys$y))
                sys$compV = ts(matrix(sys$compV, n + sys$h, 1), start = initial, frequency = frequency(sys$y))
            } else {
                sys$comp = matrix(sys$comp, n + sys$h, 1)
                sys$compV = matrix(sys$compV, n + sys$h, 1)
            }
        }
        names1 = strsplit(output$compNames, "/")
        colnames(sys$comp) = names1[[1]][1 : ncol(sys$comp)]
        colnames(sys$compV) = names1[[1]][1 : ncol(sys$comp)]
    }
    if (command == "filter" || command == "smooth" || command == "disturb" || command == "all") {
        # n = length(sys$y) + sys$h
        # m = output$ns
        aux = dim(output$a)
        n = aux[2]
        m = aux[1]
        if (is.ts(sys$y)){
            fY = frequency(sys$y)
            sY = initial
            if (command == "disturb"){
                n = length(sys$y)
                mEta = length(output$eta) / n
                sys$eta = ts(t(matrix(output$eta, mEta, n)), sY, frequency = fY)
                sys$eps = ts(t(matrix(output$eps, 1, n)), sY, frequency = fY)
            } else {
                sys$a = ts(t(matrix(output$a, m, n)), sY, frequency = fY)
                # sys$aFor = ts(t(matrix(sys$aFor, m, sys$h)), end(aux), frequency = fY)
                sys$P = ts(t(matrix(output$P, m, n)), sY, frequency = fY)
                # sys$PFor = ts(t(matrix(sys$PFor, m, sys$h)), end(aux), frequency = fY)
                sys$yFit = ts(output$yFit, sY, frequency = fY)
                sys$yFitV = ts(output$yFitV, sY, frequency = fY)
                aux = ts(matrix(NA, length(sys$y) - length(output$v) + 1, 1), sY, frequency = fY)
                sys$v = ts(output$v, end(aux), frequency = fY)
            }
        } else {
            if (command == "disturb"){
                n = length(sys$y)
                sys$eta = t(matrix(output$eta, m, n))
                sys$eps = t(matrix(output$eps, 1, n))
            } else {
                sys$a = t(matrix(output$a, m, n))
                # sys$aFor = t(matrix(sys$aFor, m, sys$h))
                sys$P = t(matrix(output$P, m, n))
                # sys$PFor = t(matrix(sys$PFor, m, sys$h))
                sys$yFit = output$yFit
                sys$yFitV = output$yFitV
                sys$v = output$v
            }
        }
        if (command != "disturb"){
            names1 = strsplit(output$stateNames, "/")[[1]]
            nNames = length(names1)
            if (is.vector(sys$a)){
                sys$a = matrix(sys$a, length(sys$a), 1)
                sys$P = matrix(sys$P, length(sys$P), 1)
            }
            if (ncol(sys$a) >= nNames){
                sys$a = sys$a[, 1 : nNames]
                sys$P = sys$P[, 1 : nNames]
                if (length(size(sys$a)) == 1){
                    sys$a = matrix(sys$a, length(sys$a), 1)
                    sys$P = matrix(sys$P, length(sys$P), 1)
                }
                colnames(sys$a) = strsplit(output$stateNames, "/")[[1]]
                colnames(sys$P) = strsplit(output$stateNames, "/")[[1]]
            }
        }
    }
    if (isnullu)
         sys$u = NULL
    return(sys)
}
 

#' @title UCestim
#' @description Estimates and forecasts UC models
#'
#' @details \code{UCestim} estimates and forecasts a time series using an
#' UC model.
#' The optimization method is a BFGS quasi-Newton algorithm with a 
#' backtracking line search using Armijo conditions.
#' Parameter names in output table are the following:
#' \itemize{
#' \item Damping:   Damping factor for DT trend.
#' \item Level:     Variance of level disturbance.
#' \item Slope:     Variance of slope disturbance.
#' \item Rho(#):    Damping factor of cycle #.
#' \item Period(#): Estimated period of cycle #.
#' \item Var(#):    Variance of cycle #.
#' \item Seas(#):   Seasonal harmonic with period #.
#' \item Irregular: Variance of irregular component.
#' \item AR(#):     AR parameter of lag #.
#' \item MA(#):     MA parameter of lag #.
#' \item AO#:       Additive outlier in observation #.
#' \item LS#:       Level shift outlier in observation #.
#' \item SC#:       Slope change outlier in observation #.
#' \item Beta(#):   Beta parameter of input #.
#' \item Cnst:      Constant.
#' }
#' 
#' Standard methods applicable to UComp objects are print, summary, plot,
#' fitted, residuals, logLik, AIC, BIC, coef, predict, tsdiag.
#'
#' @param sys an object of type \code{UComp} created with \code{UC}
#' 
#' @return The same input object with the appropriate fields 
#' filled in, in particular:
#' \itemize{
#' \item p:        Estimated transformed parameters
#' \item v:        Estimated innovations (white noise in correctly specified models)
#' \item yFor:     Forecast values of output
#' \item yForV:    Forecasted values variance
#' \item criteria: Value of criteria for estimated model
#' \item covp:     Covariance matrix of estimated transformed parameters
#' \item grad:     Gradient of log-likelihood at the optimum
#' \item iter:     Estimation iterations
#' }
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, 
#'          \code{\link{UCsmooth}}, \code{\link{UCdisturb}}, \code{\link{UCcomponents}},
#'          \code{\link{UChp}}
#'          
#' @examples
#' \donttest{
#' m1 <- UCsetup(log(AirPassengers))
#' m1 <- UCestim(m1)
#' }
#' @rdname UCestim
#' @export
UCestim = function(sys){
    sys = UCcommand("forecast", sys)
    return(sys)
}

#' @title UCvalidate
#' @description Shows a table of estimation and diagnostics results for UC models.
#' Equivalent to print or summary.
#' The table shows information in four sections:
#' Firstly, information about the model estimated, the relevant 
#' periods of the seasonal component included, and further information about
#' convergence.
#' Secondly, parameters with their names are provided, the asymptotic standard errors, 
#' the ratio of the two, and the gradient at the optimum. One asterisk indicates 
#' concentrated-out parameters and two asterisks signals parameters constrained during estimation.
#' Thirdly, information criteria and the value of the log-likelihood.
#' Finally, diagnostic statistics about innovations, namely, the Ljung-Box Q test of absense
#' of autocorrelation statistic for several lags, the Jarque-Bera gaussianity test, and a
#' standard ratio of variances test.
#'
#' @param sys an object of type \code{UComp} created with \code{UC}
#' @param printScreen print to screen or just return output table
#' 
#' @return The same input object with the appropriate fields 
#' filled in, in particular:
#' \itemize{
#' \item table: Estimation and validation table
#' }
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCfilter}}, 
#'          \code{\link{UCsmooth}}, \code{\link{UCdisturb}}, \code{\link{UCcomponents}},
#'          \code{\link{UChp}}
#'          
#' @examples
#' \donttest{
#' m1 <- UC(log(gdp))
#' m1 <- UCvalidate(m1)
#' }
#' @rdname UCvalidate
#' @export
UCvalidate = function(sys, printScreen = TRUE){
    VERBOSE = sys$verbose
    sys$verbose = FALSE
    sys = UCcommand("validate", sys)
    sys$verbose = VERBOSE
    if (printScreen){
        cat(sys$table)
    }
    return(sys)
}

#' @title UChp
#' @description Hodrick-Prescott filter estimation
#'
#' @param y A time series object
#' 
#' @param lambda Smoothing constant (default: 1600)
#' 
#' @return The cycle estimation
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, 
#'          \code{\link{UCsmooth}}, \code{\link{UCcomponents}}, \code{\link{UCdisturb}}
#'          
#' @examples
#' \donttest{
#' cycle <- UChp(USgdp)
#' plot(cycle)
#' }
#' @rdname UChp
#' @export
UChp = function(y, lambda = 1600){
    m = UCsetup(y, model = "irw/none/arma(0,0)")
    m$hidden$truePar = c(log(1 / lambda) / 2, 0)
    m = UCcomponents(m)
    return(y - m$comp[, 1])
}

#' @title UCcomponents
#' @description Estimates unobserved components of UC models
#' Standard methods applicable to UComp objects are print, summary, plot,
#' fitted, residuals, logLik, AIC, BIC, coef, predict, tsdiag.
#'
#' @param sys an object of type \code{UComp} created with \code{UC} or \code{UCforecast}
#' 
#' @return The same input object with the appropriate fields 
#' filled in, in particular:
#' \itemize{
#' \item comp:  Estimated components in matrix form
#' \item compV: Estimated components variance in matrix form
#' }
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, 
#'          \code{\link{UCsmooth}}, \code{\link{UCdisturb}},
#'          \code{\link{UChp}}
#'          
#' @examples
#' \donttest{
#' m1 <- UC(log(AirPassengers))
#' m1 <- UCcomponents(m1)
#' }
#' @rdname UCcomponents
#' @export
UCcomponents= function(sys){
    VERBOSE = sys$verbose
    sys$verbose = FALSE
    sys = UCcommand("components", sys)
    sys$verbose = VERBOSE
    return(sys)
}

#' @title UCfilter
#' @description Runs the Kalman Filter for UC models
#' Standard methods applicable to \code{UComp} objects are print, summary, plot,
#' fitted, residuals, logLik, AIC, BIC, coef, predict, tsdiag.
#'
#' @param sys an object of type \code{UComp} created with \code{UC}
#' 
#' @return The same input object with the appropriate fields 
#' filled in, in particular:
#' \itemize{
#' \item yFit:  Fitted values of output
#' \item yFitV: Variance of fitted values of output
#' \item a:     State estimates
#' \item P:     Variance of state estimates (diagonal of covariance matrices)
#' }
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, 
#'          \code{\link{UCsmooth}}, \code{\link{UCdisturb}}, \code{\link{UCcomponents}},
#'          \code{\link{UChp}}
#'          
#' @examples
#' \donttest{
#' m1 <- UC(log(AirPassengers))
#' m1 <- UCfilter(m1)
#' }
#' @rdname UCfilter
#' @export
UCfilter = function(sys){
    VERBOSE = sys$verbose
    sys$verbose = FALSE
    sys = UCcommand("filter", sys)
    sys$verbose = VERBOSE
    return(sys)
}

#' @title UCsmooth
#' @description Runs the Fixed Interval Smoother for UC models
#' Standard methods applicable to \code{UComp} objects are print, summary, plot,
#' fitted, residuals, logLik, AIC, BIC, coef, predict, tsdiag.
#'
#' @param sys an object of type \code{UComp} created with \code{UC}
#' 
#' @return The same input object with the appropriate fields 
#' filled in, in particular:
#' \itemize{
#' \item yFit:  Fitted values of output
#' \item yFitV: Variance of fitted values of output
#' \item a:     State estimates
#' \item P:     Variance of state estimates (diagonal of covariance matrices)
#' }
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, 
#'          \code{\link{UCdisturb}}, \code{\link{UCcomponents}},
#'          \code{\link{UChp}}
#'          
#' @examples
#' \donttest{
#' m1 <- UC(log(AirPassengers))
#' m1 <- UCsmooth(m1)
#' }
#' @rdname UCsmooth
#' @export
UCsmooth = function(sys){
    VERBOSE = sys$verbose
    sys$verbose = FALSE
    sys = UCcommand("smooth", sys)
    sys$verbose = VERBOSE
    return(sys)
}

#' @title UCdisturb
#' @description Runs the Disturbance Smoother for UC models
#' Standard methods applicable to \code{UComp} objects are print, summary, plot,
#' fitted, residuals, logLik, AIC, BIC, coef, predict, tsdiag.
#'
#' @param sys an object of type \code{UComp} created with \code{UC}
#' 
#' @return The same input object with the appropriate fields 
#' filled in, in particular:
#' \itemize{
#' \item yFit:  Fitted values of output
#' \item yFitV: Variance of fitted values of output
#' \item a:     State estimates
#' \item P:     Variance of state estimates (diagonal of covariance matrices)
#' \item eta:   State perturbations estimates
#' \item eps:   Observed perturbations estimates
#' }
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, 
#'          \code{\link{UCsmooth}}, \code{\link{UCcomponents}},
#'          \code{\link{UChp}}
#'          
#' @examples
#' \donttest{
#' m1 <- UC(log(AirPassengers))
#' m1 <- UCdisturb(m1)
#' }
#' @rdname UCdisturb
#' @export
UCdisturb = function(sys){
    VERBOSE = sys$verbose
    sys$verbose = FALSE
    sys = UCcommand("disturb", sys)
    sys$verbose = VERBOSE
    return(sys)
}

#' @title TETSvalidate
#' @description Shows a table of estimation and diagnostics results for TOBIT TETS models
#'
#' @param m an object of type \code{TETS} created with \code{TETSforecast}
#' 
#' @return The same input object with the appropriate fields 
#' filled in, in particular:
#' \item{table}{Estimation and validation table}
#' 
#' @author Diego J. Pedregal
#' 
#' @seealso \code{\link{TETS}}, \code{\link{TETSforecast}}, \code{\link{TETSvalidate}},
#'          \code{\link{TETScomponents}}
#'          
#' @examples
#' \donttest{
#' m1 <- TETSforecast(log(gdp))
#' m1 <- TETSvalidate(m1)
#' }
#' @rdname TETSvalidate
#' @export
TETSvalidate = function(m){
    # suppressWarnings()
    if (is.null(m$u))
        u = m$u
    else {
        if (is.vector(m$u)){
            u = matrix(m$u, 1, length(m$u))
        } else {
            nu = dim(m$u)
            u = as.numeric(m$u);
            u = matrix(u, nu[1], nu[2])
        }
    }
    if (inherits(m, "ETS")){
            output = ETSc("validate", as.numeric(m$y), u, m$model, m$s, m$h,
                          m$criterion, m$armaIdent, m$identAll, m$forIntervals,
                          m$bootstrap, m$nSimul, m$verbose, m$lambda,
                          m$alphaL, m$betaL, m$gammaL, m$phiL, m$p0)
    } else {
            output = TETSc("validate", as.numeric(m$y), u, m$model, m$s, m$h,
                          m$criterion, m$armaIdent, m$identAll, m$forIntervals,
                          m$bootstrap, m$nSimul, m$verbose, m$lambda,
                          m$alphaL, m$betaL, m$gammaL, m$phiL, m$p0, m$Ymin, m$Ymax)
    }
    if (length(output) == 1){   # ERROR!!
        stop()
    } else {
        if (is.ts(m$y))
            m$comp = ts(output$comp, start = start(m$y), frequency = frequency(m$y))
        else
            m$comp = output$comp
        m$table = output$table
        # Buscando test heterocedasticidad con valor p NaN
        # ind = which(grepl("nan", m$table))
        # if (any(ind)){
        #     for (i in 1 : length(ind)){
        #         line = m$table[ind[i]]
        #         Fstat = as.numeric(substr(line, 15, 31))
        #         if (grepl("Bera", line)) {
        #             pval = round(pchisq(Fstat, 2), 4)
        #         } else if (grepl("H(", line)) {
        #             df = as.numeric(substr(line, 9, 12))
        #             pval = round(pf(Fstat, df, df), 4)
        #         }
        #         line = gsub("   nan", pval, line)
        #         m$table[ind[i]] = line
        #     }
        # }
        # if (m$verbose)
        #     cat(m$table)
        colnames(m$comp) = strsplit(output$compNames, split = "/")[[1]]
        m$model = output$model
        m$lambda = output$lambda
        m$p = output$p
        m$truep = output$truep
        lu = size(m$u)[1]
        if (lu > 0)
            m$h = lu - length(m$y)
        if (is.ts(m$y) && m$h > 0){
            fake = ts(c(m$y, NA), start = start(m$y), frequency = frequency(m$y))
            m$yFor = ts(output$yFor, start = end(fake), frequency = frequency(m$y))
            m$yForV = ts(output$yForV, start = end(fake), frequency = frequency(m$y))
            if (m$bootstrap)
                m$ySimul = ts(output$ySimul, start = end(fake), frequency = frequency(m$y))
        } else if (m$h > 0) {
            m$yFor = output$yFor
            m$yForV = output$yForV
            m$ySimul = output$ySimul
        }
        m$criteria = output$criteria
        return(m)
    }
}
    

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{extract}
\alias{extract}
\title{extract}
\usage{
extract(x, col, accordingTo = 1)
}
\arguments{
\item{x}{a data frame}

\item{col}{column to be ordered}

\item{accordingTo}{column to take as the pattern}
}
\value{
Data frame reordered accoring to a given column data
}
\description{
Reorder data frame returning column col reordered
according to the values in column accordingTo
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{size}
\alias{size}
\title{size}
\usage{
size(y)
}
\arguments{
\item{y}{a vector, matrix or array}
}
\value{
A vector with all the dimensions
}
\description{
Size of vector, matrix or array
}
\examples{
s <- size(matrix(4, 3, 2))
s <- size(rep(4, 3))
s <- size(array(4, c(3, 2, 2)))
}
\seealso{
\code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
         \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
         \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
         \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
         \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
         \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UCS3functions.R
\name{AIC.UComp}
\alias{AIC.UComp}
\title{AIC.UComp}
\usage{
\method{AIC}{UComp}(object, ..., k = 2)
}
\arguments{
\item{object}{Object of class \dQuote{UComp}.}

\item{...}{Additional inputs to function.}

\item{k}{The penalty per parameter to be used.}
}
\value{
AIC value of a UC model
}
\description{
Extract AIC value of UComp object
}
\details{
Selection criteria for models with different number of 
parameters, the smaller AIC the better. The formula used here is
\eqn{AIC=-2 (ln(L) - k) / n}, where \eqn{ln(L)} is the log-likelihood
at the optimum, \eqn{k} is the number of parameters plus
non-stationary states and \eqn{n} is the number of observations.
Mind that this formulation differs from the usual definition that
does not divide by \eqn{n}. This makes that AIC(m) and AIC(logLik(m))
give different results, being m an UComp object.
}
\examples{
\donttest{
y <- log(AirPassengers)
m1 <- UCforecast(y, model = "llt/equal/arma(0,0)")
AIC(m1)
}
}
\seealso{
\code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, \code{\link{UCsmooth}}, 
         \code{\link{UCdisturb}}, \code{\link{UCcomponents}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{varTest}
\alias{varTest}
\title{varTest}
\usage{
varTest(y, parts = 1/3)
}
\arguments{
\item{y}{a vector, ts or tsibble object}

\item{parts}{portion of sample to estimate variances}
}
\value{
Table with test results
}
\description{
Ratio of variances test
}
\examples{
varTest(AirPassengers)
}
\seealso{
\code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
         \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
         \code{\link{cusum}}, \code{\link{conv}},
         \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
         \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
         \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
         \code{\link{size}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{airpas}
\alias{airpas}
\title{Airpassengers in Spain}
\format{
Time series objects.
\describe{
  Monthly data from 1969
  
  https://portal.mineco.gob.es/es-es/economiayempresa/EconomiaInformesMacro/Paginas/bdsice.aspx
}
}
\usage{
airpas
}
\value{
No return value, called for side effects
}
\description{
Foreign arrivals by air in Spain in thousands of passengers (airpas).
}
\examples{
\donttest{
 airpas
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{OECDgdp}
\alias{OECDgdp}
\title{OECD GDP}
\format{
Time series objects.
\describe{
  Quarterly data from 1962 to 2019
  
  https://portal.mineco.gob.es/es-es/economiayempresa/EconomiaInformesMacro/Paginas/bdsice.aspx
}
}
\usage{
OECDgdp
}
\value{
No return value, called for side effects
}
\description{
Seasonally adjusted quarterly OECD real gross domestic product (OECDgdp).
}
\examples{
\donttest{
 OECDgdp
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{plotAcfPacf}
\alias{plotAcfPacf}
\title{plotAcfPacf}
\usage{
plotAcfPacf(ACF, PACF, s = 1, n = NA, runFromTest = FALSE)
}
\arguments{
\item{ACF}{variable to plot}

\item{PACF}{second variable to plot}

\item{s}{seasonal period}

\item{n}{number of coefficients}

\item{runFromTest}{internal check variable}
}
\value{
No return value, called for side effects
}
\description{
Plot of ACF and PACF
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{tsDisplay}
\alias{tsDisplay}
\title{tsDisplay}
\usage{
tsDisplay(y, nCoef = 25, nPar = 0, s = NA)
}
\arguments{
\item{y}{a vector, ts or tsibble object}

\item{nCoef}{number of autocorrelation coefficients to estimate}

\item{nPar}{number of parameters in a model if y is a residual}

\item{s}{seasonal period, number of observations per year}
}
\value{
No return value, called for side effects
}
\description{
Displays time series plot with autocorrelation functions
}
\examples{
tsDisplay(AirPassengers)
}
\seealso{
\code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
         \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
         \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
         \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
         \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
         \code{\link{plotSlide}}, \code{\link{Accuracy}},
         \code{\link{size}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{box.cox}
\alias{box.cox}
\title{box.cox}
\usage{
box.cox(x, lambda)
}
\arguments{
\item{x}{Time series object.}

\item{lambda}{Lambda parameter for Box-Cox transform.}
}
\value{
Box-Cox transformed time series
}
\description{
Runs Box-Cox transform of a time series
}
\examples{
\donttest{
y <- box.cox(AirPassengers, 0.5)
plot(y)
}
}
\seealso{
\code{\link{inv.box.cox}}, \code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, \code{\link{UCsmooth}}, 
         \code{\link{UCdisturb}}, \code{\link{UCcomponents}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ETSS3functions.R
\name{invBoxCox}
\alias{invBoxCox}
\title{invBoxCox}
\usage{
invBoxCox(y, yVar, lambda, const = 2)
}
\arguments{
\item{y}{matrix, array or vector}

\item{yVar}{matrix, array or vector of variances of y}

\item{lambda}{lambda parameter of Box-Cox transformation}

\item{const}{number of standard error for confidence band}
}
\value{
Inverse of Box-Cox heteroskedasticity transformation with confidence bands
}
\description{
Calculates inverse of Box-Cox transformation 
with confidence bands, calculated as const time the standard error
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{removeNaNs}
\alias{removeNaNs}
\title{removeNaNs}
\usage{
removeNaNs(x)
}
\arguments{
\item{x}{a vector or a ts object}
}
\value{
vector with nans removed (only those at beginning or end)
}
\description{
Remove nans at beginning or end of vector
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{roots}
\alias{roots}
\title{roots}
\usage{
roots(x)
}
\arguments{
\item{x}{coefficients of polynomial in descending order}
}
\value{
Roots of polynomial
}
\description{
Roots of polynomial
}
\examples{
roots(c(1, -2 ,1))
roots(conv(c(1, -1), c(1, 0.8)))
}
\seealso{
\code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
         \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
         \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
         \code{\link{armaFilter}}, \code{\link{dif}},
         \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
         \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
         \code{\link{size}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{zplane}
\alias{zplane}
\title{zplane}
\usage{
zplane(MApoly = 1, ARpoly = 1)
}
\arguments{
\item{MApoly}{coefficients of numerator polynomial in descending order}

\item{ARpoly}{coefficients of denominator polynomial in descending order}
}
\value{
No return value, called for side effects
}
\description{
Real-imaginary plane to show roots of digital filters (ARMA)
}
\details{
Shows the real-imaginary plane to show zeros (roots of numerator or
MA polynomial) and poles (roots of denominator of AR polynomial). Unit roots
and real vs imaginary roots can be seen by eye
}
\examples{
zplane(c(1, -2, 1), c(1, -0.8))
}
\seealso{
\code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
         \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
         \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
         \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
         \code{\link{acft}}, \code{\link{slide}},
         \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
         \code{\link{size}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{acft}
\alias{acft}
\title{acft}
\usage{
acft(MApoly = 1, ARpoly = 1, ncoef = 38, s = 1)
}
\arguments{
\item{MApoly}{coefficients of numerator polynomial in descending order}

\item{ARpoly}{coefficients of denominator polynomial in descending order}

\item{ncoef}{number of coefficients}

\item{s}{seasonal period, number of observations per year}
}
\value{
Theoretical autocorrelation functions
}
\description{
Theoretical autocorrelation functions of ARMA models
}
\examples{
acft(c(1, -0.8), c(1, 0.8))
}
\seealso{
\code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
         \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
         \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
         \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
         \code{\link{zplane}}, \code{\link{slide}},
         \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
         \code{\link{size}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UCS3functions.R
\name{getp0}
\alias{getp0}
\title{getp0}
\usage{
getp0(y, model = "llt/equal/arma(0,0)", periods = NA)
}
\arguments{
\item{y}{a time series to forecast.}

\item{model}{any valid \code{UComp} model without any ?.}

\item{periods}{vector of fundamental period and harmonics required.}
}
\value{
A set of parameters p0 of an object of class \code{UComp}
to use as input to \code{\link{UC}}, \code{\link{UCforecast}}.
}
\description{
Get initial conditions for parameters of \code{UComp} object
}
\details{
Provides initial parameters of a given model for the time series.
They may be changed arbitrarily by the user to include as an input \code{p0} to
\code{UC} or \code{UCforecast} functions (see example below).
There is no guarantee that the model will converge and selecting initial conditions
should be used with care.
}
\examples{
\dontrun{
p0 <- getp0(log(AirPassengers), model = "llt/equal/arma(0,0)")
p0[1] <- 0  # p0[1] <- NA
m <- UCforecast(log(AirPassengers), model = "llt/equal/arma(0,0)", p0 = p0)
}
}
\seealso{
\code{\link{UC}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, \code{\link{UCsmooth}}, 
         \code{\link{UCdisturb}}, \code{\link{UCcomponents}},
         \code{\link{UChp}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TETSmodel.R
\name{TETSsetup}
\alias{TETSsetup}
\title{TETSsetup}
\usage{
TETSsetup(
  y,
  u = NULL,
  model = "???",
  s = frequency(y),
  h = 2 * s,
  criterion = "aicc",
  forIntervals = FALSE,
  bootstrap = FALSE,
  nSimul = 5000,
  verbose = FALSE,
  alphaL = c(0, 1),
  betaL = alphaL,
  gammaL = alphaL,
  phiL = c(0.8, 0.98),
  p0 = -99999,
  Ymin = -Inf,
  Ymax = Inf
)
}
\arguments{
\item{y}{a time series to forecast (it may be either a numerical vector or
a time series object). This is the only input required. If a vector, the additional
input \code{s} should be supplied compulsorily (see below).}

\item{u}{a matrix of input time series. If 
the output wanted to be forecast, matrix \code{u} should contain future values for inputs.}

\item{model}{the model to estimate. It is a single string indicating the type of 
model for each component with one or two letters:
\itemize{
\item Error: ? / A

\item Trend: ? / N / A / Ad

\item Seasonal: ? / N / A

}}

\item{s}{seasonal period of time series (1 for annual, 4 for quarterly, ...)}

\item{h}{forecast horizon. If the model includes inputs h is not used, the lenght of u is used instead.}

\item{criterion}{information criterion for identification ("aic", "bic" or "aicc").}

\item{forIntervals}{estimate forecasting intervals (TRUE / FALSE)}

\item{bootstrap}{use bootstrap simulation for predictive distributions}

\item{nSimul}{number of simulation runs for bootstrap simulation of predictive distributions}

\item{verbose}{intermediate estimation output (TRUE / FALSE)}

\item{alphaL}{constraints limits for alpha parameter}

\item{betaL}{constraints limits for beta parameter}

\item{gammaL}{constraints limits for gamma parameter}

\item{phiL}{constraints limits for phi parameter}

\item{p0}{initial values for parameter search (alpha, beta, phi, gamma, sigma2) with consraints:}

\item{Ymin}{scalar or vector of time varying censoring values from below}

\item{Ymax}{scalar or vector of time varying censoring values from above

\itemize{ 
\item 0 < alpha < 1

\item 0 < beta < alpha

\item 0 < phi < 1

\item 0 < gamma < 1 - alpha

\item sigma2 > 0
}}
}
\value{
An object of class \code{TETS}. It is a list with fields including all the inputs and
        the fields listed below as outputs. All the functions in this package fill in
        part of the fields of any \code{TETS} object as specified in what follows (function 
        \code{TETS} fills in all of them at once):

After running \code{TETSforecast}:
\item{p}{Estimated parameters}
\item{criteria}{Values for estimation criteria (LogLik, AIC, BIC, AICc)}
\item{yFor}{Forecasted values of output}
\item{yForV}{Variance of forecasted values of output}
\item{ySimul}{Bootstrap simulations for forecasting distribution evaluation}

After running \code{TETSvalidate}:
\item{table}{Estimation and validation table}
\item{comp}{Estimated components in matrix form}

After running \code{TETScomponents}:
\item{comp}{Estimated components in matrix form}
}
\description{
Sets up TOBIT TETS general univariate models
}
\details{
See help of \code{TETSforecast}.
}
\examples{
\donttest{
y <- log(AirPassengers)
m1 <- TETSsetup(y)
m1 <- TETSsetup(y,  model = "???")
m1 <- TETSsetup(y,  model = "?AA")
}
}
\seealso{
\code{\link{TETS}}, \code{\link{TETSforecast}}, \code{\link{TETSvalidate}},
         \code{\link{TETScomponents}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UComp-package.R
\name{UComp}
\alias{UComp}
\alias{UComp-package}
\title{UComp}
\description{
Package for time series modelling and forecasting of 
times series models inspired on different sources:
}
\details{
\itemize{
\item Unobserved Components models due to A.C. Harvey (Basic Structural Model: BSM), enhanced with automatic identification 
tools by Diego J. Pedregal. 
\item ExponenTial Smoothing by R.J. Hyndman and colaborators.
\item ARIMA models by V. Gómez and A. Maravall
\item Tobit ETS models by Pedregal, Trapero and Holgado
}

The package is designed for automatic identification 
among a wide range of possible models. The models may include exogenous variables. ARMA irregular components 
and automatic detection of outliers in some instances.
}
\section{References}{

Harvey AC (1989). Forecasting, Structural Time Series Models and the Kalman Filter. Cam- bridge University Press.

de Jong, P & Penzer, J (1998). Diagnosing Shocks in Time Series, Journal of the American Statistical Association, 93, 442, 796-806.

Pedregal, DJ, & Young, PC (2002). Statistical approaches to modelling and forecasting time series. In M. Clements,
& D. Hendry (Eds.), Companion to economic forecasting (pp. 69–104). Oxford: Blackwell Publishers.

Durbin J, Koopman SJ (2012). Time Series Analysis by State Space Methods. 38. Oxford University Press.

Proietti T and Luati A (2013). Maximum likelihood estimation of time series models: the Kalman filter and beyond, in Handbook of research methods and applications in empirical macroeconomics, ed. Nigar Hashimzade and Michael Thornton, E. Elgar, UK.

Hyndman RJ, Koehler AB, Ord JK and Snyder RD (2008), Forecasting with exponential smoothing, The State Sapce approach,
Berlin, Springer-Verlag.

Gómez V and Maravall, A (2000), Automatic methods for univariate series. In Peña, D., Tiao, G.C. and Tsay R.S., A course in time series analyis. Wiley.

Trapero JR, Holgado E, Pedregal DJ (2024), Demand forecasting under lost sales stock policies, International Journal of Forecasting, 40, 1055-1068.
}

\section{Maintainer}{
 Diego J. Pedregal
}

\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ipi}
\alias{ipi}
\title{Spanish Industrial Production Index}
\format{
Objeto time series.
\describe{
  Monthly since 1975
  
  https://portal.mineco.gob.es/es-es/economiayempresa/EconomiaInformesMacro/Paginas/bdsice.aspx
}
}
\usage{
ipi
}
\value{
No return value, called for side effects
}
\description{
Spanish Industrial Production Index (ipi).
}
\examples{
\donttest{
 ipi
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UCmodel.R
\name{UCsetup}
\alias{UCsetup}
\title{UCsetup}
\usage{
UCsetup(
  y,
  u = NULL,
  model = "?/none/?/?",
  h = 24,
  lambda = 1,
  outlier = 9999,
  tTest = FALSE,
  criterion = "aic",
  periods = NA,
  verbose = FALSE,
  stepwise = FALSE,
  p0 = -9999.9,
  arma = FALSE,
  TVP = NULL,
  trendOptions = "none/rw/llt/dt",
  seasonalOptions = "none/equal/different",
  irregularOptions = "none/arma(0,0)"
)
}
\arguments{
\item{y}{a time series to forecast (it may be either a numerical vector or
a time series object). This is the only input required. If a vector, the additional
input \code{periods} should be supplied compulsorily (see below).}

\item{u}{a matrix of external regressors included only in the observation equation. 
(it may be either a numerical vector or a time series object). If the output wanted 
to be forecast, matrix \code{u} should contain future values for inputs.}

\item{model}{the model to estimate. It is a single string indicating the type of 
model for each component. It allows two formats "trend/seasonal/irregular" or 
"trend/cycle/seasonal/irregular". The possibilities available for each component are:
\itemize{
\item Trend: ? / none / rw / irw / llt / dt / td; 

\item Seasonal: ? / none / equal / different;

\item Irregular: ? / none / arma(0, 0) / arma(p, q) - with p and q integer positive orders;
    
\item Cycles: ? / none / combination of positive or negative numbers. Positive numbers fix
the period of the cycle while negative values estimate the period taking as initial
condition the absolute value of the period supplied. Several cycles with positive or negative values are possible
and if a question mark is included, the model test for the existence of the cycles
specified. The following are valid examples with different meanings: 48, 48?, -48, -48?,
48+60, -48+60, -48-60, 48-60, 48+60?, -48+60?, -48-60?, 48-60?.
}}

\item{h}{forecast horizon. If the model includes inputs h is not used, the lenght of u is used instead.}

\item{lambda}{Box-Cox transformation lambda, NULL for automatic estimation}

\item{outlier}{critical level of outlier tests. If NA it does not carry out any 
outlier detection (default). A positive value indicates the critical minimum
t test for outlier detection in any model during identification. Three types of outliers are
identified, namely Additive Outliers (AO), Level Shifts (LS) and Slope Change (SC).}

\item{tTest}{augmented Dickey Fuller test for unit roots used in stepwise algorithm (TRUE / FALSE). 
The number of models to search for is reduced, depending on the result of this test.}

\item{criterion}{information criterion for identification ("aic", "bic" or "aicc").}

\item{periods}{vector of fundamental period and harmonics required.}

\item{verbose}{intermediate results shown about progress of estimation (TRUE / FALSE).}

\item{stepwise}{stepwise identification procedure (TRUE / FALSE).}

\item{p0}{initial parameter vector for optimisation search.}

\item{arma}{check for arma models for irregular components (TRUE / FALSE).}

\item{TVP}{vector of zeros and ones to indicate TVP parameters.}

\item{trendOptions}{trend models to select amongst (e.g., "rw/llt").}

\item{seasonalOptions}{seasonal models to select amongst (e.g., "none/differentt").}

\item{irregularOptions}{irregular models to select amongst (e.g., "none/arma(0,1)").}
}
\value{
An object of class \code{UComp}. It is a list with fields including all the inputs and
        the fields listed below as outputs. All the functions in this package fill in
        part of the fields of any \code{UComp} object as specified in what follows (function 
        \code{UC} fills in all of them at once):

After running \code{UCforecast}:
\itemize{
\item p:        Estimated parameters
\item v:        Estimated innovations (white noise in correctly specified models)
\item yFor:     Forecasted values of output
\item yForV:    Variance of forecasts
\item criteria: Value of criteria for estimated model
\item iter:     Number of iterations in estimation
\item grad:     Gradient at estimated parameters
\item covp:     Covariance matrix of parameters
}

After running \code{UCvalidate}:
\itemize{
\item table: Estimation and validation table
}

After running \code{UCcomponents}:
\itemize{
\item comp:      Estimated components in matrix form
\item compV:     Estimated components variance in matrix form
}

After running \code{UCfilter}, \code{UCsmooth} or  \code{UCdisturb}:
\itemize{
\item yFit:  Fitted values of output
\item yFitV: Estimated fitted values variance
\item a:     State estimates
\item P:     Variance of state estimates
\item aFor:  Forecasts of states
\item PFor:  Forecasts of states variances
}

After running \code{UCdisturb}:
\itemize{
\item eta: State perturbations estimates
\item eps: Observed perturbations estimates
}

Standard methods applicable to UComp objects are print, summary, plot,
fitted, residuals, logLik, AIC, BIC, coef, predict, tsdiag.
}
\description{
Sets up UC general univariate models
}
\details{
See help of \code{UC}.
}
\examples{
\donttest{
y <- log(AirPassengers)
m1 <- UCsetup(y)
m1 <- UCsetup(y, outlier = 4)
m1 <- UCsetup(y, model = "llt/equal/arma(0,0)")
m1 <- UCsetup(y, model = "?/?/?/?")
m1 <- UCsetup(y, model = "llt/?/equal/?", outlier = 4)
}
}
\seealso{
\code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, \code{\link{UCsmooth}}, 
         \code{\link{UCdisturb}}, \code{\link{UCcomponents}},
         \code{\link{UChp}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UCestim.R
\name{UCestim}
\alias{UCestim}
\title{UCestim}
\usage{
UCestim(sys)
}
\arguments{
\item{sys}{an object of type \code{UComp} created with \code{UC}}
}
\value{
The same input object with the appropriate fields 
filled in, in particular:
\itemize{
\item p:        Estimated transformed parameters
\item v:        Estimated innovations (white noise in correctly specified models)
\item yFor:     Forecast values of output
\item yForV:    Forecasted values variance
\item criteria: Value of criteria for estimated model
\item covp:     Covariance matrix of estimated transformed parameters
\item grad:     Gradient of log-likelihood at the optimum
\item iter:     Estimation iterations
}
}
\description{
Estimates and forecasts UC models
}
\details{
\code{UCestim} estimates and forecasts a time series using an
UC model.
The optimization method is a BFGS quasi-Newton algorithm with a 
backtracking line search using Armijo conditions.
Parameter names in output table are the following:
\itemize{
\item Damping:   Damping factor for DT trend.
\item Level:     Variance of level disturbance.
\item Slope:     Variance of slope disturbance.
\item Rho(#):    Damping factor of cycle #.
\item Period(#): Estimated period of cycle #.
\item Var(#):    Variance of cycle #.
\item Seas(#):   Seasonal harmonic with period #.
\item Irregular: Variance of irregular component.
\item AR(#):     AR parameter of lag #.
\item MA(#):     MA parameter of lag #.
\item AO#:       Additive outlier in observation #.
\item LS#:       Level shift outlier in observation #.
\item SC#:       Slope change outlier in observation #.
\item Beta(#):   Beta parameter of input #.
\item Cnst:      Constant.
}

Standard methods applicable to UComp objects are print, summary, plot,
fitted, residuals, logLik, AIC, BIC, coef, predict, tsdiag.
}
\examples{
\donttest{
m1 <- UCsetup(log(AirPassengers))
m1 <- UCestim(m1)
}
}
\seealso{
\code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, 
         \code{\link{UCsmooth}}, \code{\link{UCdisturb}}, \code{\link{UCcomponents}},
         \code{\link{UChp}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{plotSlide}
\alias{plotSlide}
\title{plotSlide}
\usage{
plotSlide(py1, y, orig, step = 1, errorFun, collectFun = mean)
}
\arguments{
\item{py1}{output from slide function}

\item{y}{a vector, matrix or list of time series (the same used in slide call)}

\item{orig}{starting forecasting origin (the same used in slide call)}

\item{step}{observations ahead to move the forecasting origin (the same used in slide call)}

\item{errorFun}{user function to calculate error measures}

\item{collectFun}{aggregation function (mean, median, etc.)}
}
\value{
An array of forecasting errors of dimensions (horizon x nOrigs x nModels x nSeries)
}
\description{
Plot summarised results from slide
}
\examples{
\dontrun{plotSlide(py1, AirPassengers, 100, 1, errorFun)}
}
\seealso{
\code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
         \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
         \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
         \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
         \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
         \code{\link{Accuracy}}, \code{\link{tsDisplay}},
         \code{\link{size}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{slideAux}
\alias{slideAux}
\title{slideAux}
\usage{
slideAux(
  y,
  orig,
  forecFun,
  h = 12,
  step = 1,
  output = TRUE,
  graph = TRUE,
  window = NA,
  parallel = FALSE,
  isList = FALSE,
  ...
)
}
\arguments{
\item{y}{a vector or matrix of time series}

\item{orig}{starting forecasting origin}

\item{forecFun}{user function that implements forecasting methods}

\item{h}{forecasting horizon}

\item{step}{observations ahead to move the forecasting origin}

\item{output}{output TRUE/FALSE}

\item{graph}{fraphical output TRUE/FALSE}

\item{window}{fixed window width in number of observations (NA for non fixed)}

\item{parallel}{run forecasts in parallel}

\item{isList}{whether the input data y is a list or a matrix}

\item{...}{rest of inputs to forecFun function}
}
\value{
Auxiliary output of slide function for just one time series
}
\description{
Auxiliary function run from slide
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ETScomponents.R
\name{ETScomponents}
\alias{ETScomponents}
\title{ETScomponents}
\usage{
ETScomponents(m)
}
\arguments{
\item{m}{an object of type \code{ETS} created with \code{ETSforecast}}
}
\value{
The same input object with the appropriate fields 
filled in, in particular:
\item{comp}{Estimated components in matrix form}
}
\description{
Estimates components of ETS models
}
\examples{
\donttest{
m1 <- ETS(log(gdp))
m1 <- ETScomponents(m1)
}
}
\seealso{
\code{\link{ETS}}, \code{\link{ETSforecast}}, \code{\link{ETSvalidate}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{arma2tsi}
\alias{arma2tsi}
\title{arma2tsi}
\usage{
arma2tsi(MApoly, ARpoly, n = 100)
}
\arguments{
\item{MApoly}{coefficients of numerator polynomial in descending order}

\item{ARpoly}{coefficients of denominator polynomial in descending order}

\item{n}{number of coefficients}
}
\value{
Tsi (MA form) coefficients of equivalent ARMA model
}
\description{
AR polynomial coefficients of ARMA model
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TETSestim.R
\name{TETSestim}
\alias{TETSestim}
\title{TETSestim}
\usage{
TETSestim(m)
}
\arguments{
\item{m}{an object of type \code{TETS} created with \code{TETSforecast}}
}
\value{
The same input object with the appropriate fields 
filled in, in particular:
\item{p}{Estimated parameters}
\item{yFor}{Forecasted values of output}
\item{yForV}{Variance of forecasted values of output}
\item{ySimul}{Bootstrap simulations for forecasting distribution evaluation}
}
\description{
Estimates and forecasts TOBIT TETS models
}
\details{
\code{TETSestim} estimates and forecasts a time series using an
a TOBIT TETS model
}
\examples{
\donttest{
m1 <- TETSsetup(log(gdp))
m1 <- TETSestim(m1)
}
}
\seealso{
\code{\link{TETS}}, \code{\link{TETSforecast}}, \code{\link{TETSvalidate}},
         \code{\link{TETScomponents}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{armaFilter}
\alias{armaFilter}
\title{armaFilter}
\usage{
armaFilter(MA = 1, AR = 1, y)
}
\arguments{
\item{MA}{numerator polynomial}

\item{AR}{denominator polynomial}

\item{y}{a vector, ts or tsibble object}
}
\value{
Filtered time series
}
\description{
Filter of time series
}
\examples{
y <- armaFilter(1, c(1 , -0.8), rnorm(200))
}
\seealso{
\code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
         \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
         \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
         \code{\link{dif}}, \code{\link{roots}},
         \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
         \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
         \code{\link{size}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UCS3functions.R
\name{BIC.UComp}
\alias{BIC.UComp}
\title{BIC.UComp}
\usage{
\method{BIC}{UComp}(object, ...)
}
\arguments{
\item{object}{Object of class \dQuote{UComp}.}

\item{...}{Additional inputs to function.}
}
\value{
BIC value of a UC model
}
\description{
Extract BIC (or SBC) value of UComp object
}
\details{
Selection criteria for models with different number of 
parameters, the smaller BIC the better. The formula used here is
\eqn{BIC=(-2 ln(L) + k ln(n)) / n}, where \eqn{ln(L)} is the log-likelihood
at the optimum, \eqn{k} is the number of parameters plus
non-stationary states and \eqn{n} is the number of observations.
Mind that this formulation differs from the usual definition that
does not divide by \eqn{n}. This makes that BIC(m) and BIC(logLik(m))
give different results, being m an UComp object.
}
\examples{
\donttest{
y <- log(AirPassengers)
m1 <- UCforecast(y, model = "llt/equal/arma(0,0)")
BIC(m1)
}
}
\seealso{
\code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, \code{\link{UCsmooth}}, 
         \code{\link{UCdisturb}}, \code{\link{UCcomponents}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ARIMAmodel.R
\name{ARIMAforecast}
\alias{ARIMAforecast}
\title{ARIMAforecast}
\usage{
ARIMAforecast(
  y,
  u = NULL,
  model = NULL,
  cnst = NULL,
  s = frequency(y),
  criterion = "bic",
  h = 2 * s,
  verbose = FALSE,
  lambda = 1,
  maxOrders = c(3, 2, 3, 2, 1, 2),
  bootstrap = FALSE,
  nSimul = 5000,
  fast = FALSE
)
}
\arguments{
\item{y}{a time series to forecast (it may be either a numerical vector or
a time series object). This is the only input required. If a vector, the additional
input \code{s} should be supplied compulsorily (see below).}

\item{u}{a matrix of input time series. If 
the output wanted to be forecast, matrix \code{u} should contain future values for inputs.}

\item{model}{the model to estimate. A vector c(p,d,q,P,D,Q) containing the model orders
of an ARIMA(p,d,q)x(P,D,Q)_s model. A constant may be estimated with the
cnst input.
Use a NULL to automatically identify the ARIMA model.}

\item{cnst}{flag to include a constant in the model (TRUE/FALSE/NULL). Use NULL to estimate}

\item{s}{seasonal period of time series (1 for annual, 4 for quarterly, ...)}

\item{criterion}{information criterion for identification stage ("aic", "bic", "aicc")}

\item{h}{forecast horizon. If the model includes inputs h is not used, the lenght of u is used instead.}

\item{verbose}{intermediate estimation output (TRUE / FALSE)}

\item{lambda}{Box-Cox lambda parameter (NULL: estimate)}

\item{maxOrders}{a vector c(p,d,q,P,D,Q) containing the maximum orders of model orders 
to search for in the automatic identification}

\item{bootstrap}{use bootstrap simulation for predictive distributions}

\item{nSimul}{number of simulation runs for bootstrap simulation of predictive distributions}

\item{fast}{fast identification (avoids post-identification checks)}
}
\value{
An object of class \code{ARIMA}. It is a list with fields including all the inputs and
        the fields listed below as outputs. All the functions in this package fill in
        part of the fields of any \code{ARIMA} object as specified in what follows (function 
        \code{ARIMA} fills in all of them at once):

After running \code{ARIMAforecast} or \code{ARIMA}:
\item{p}{Estimated parameters}
\item{yFor}{Forecasted values of output}
\item{yForV}{Variance of forecasted values of output}
\item{ySimul}{Bootstrap simulations for forecasting distribution evaluation}

After running \code{ARIMAvalidate}:
\item{table}{Estimation and validation table}
}
\description{
Estimates and forecasts ARIMA general univariate models
}
\details{
\code{ARIMAforecast} is a function for modelling and forecasting univariate
time series with Autoregressive Integrated Moving Average (ARIMA) time series models. 
It sets up the model with a number of control variables that
govern the way the rest of functions in the package will work. It also estimates 
the model parameters by Maximum Likelihood and forecasts the data.
}
\examples{
\donttest{
y <- log(AirPassengers)
m1 <- ARIMAforecast(y)
m1 <- ARIMAforecast(y, lambda = NULL)
}
}
\seealso{
\code{\link{ARIMA}}, \code{\link{ARIMAvalidate}},
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{rowMedians}
\alias{rowMedians}
\title{rowMedians}
\usage{
rowMedians(x, na.rm = TRUE, ...)
}
\arguments{
\item{x}{a matrix}

\item{na.rm}{boolean indicating whether to remove nans}

\item{...}{rest of inputs}
}
\value{
A vector with all the medians in rows
}
\description{
Medians of matrix by rows
}
\examples{
s <- rowMedians(matrix(4, 3, 2))
}
\seealso{
\code{\link{colMedians}}, \code{\link{tests}},
         \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
         \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
         \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
         \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
         \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
         \code{\link{size}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{USgdp}
\alias{USgdp}
\title{US GDP}
\format{
Time series objects.
\describe{
  Quarterly data from 1962 to 2019
}
}
\source{
\href{https://www.nber.org/}{USgdp}
}
\usage{
USgdp
}
\value{
No return value, called for side effects
}
\description{
Seasonally adjusted quarterly US real gross domestic product (USgdp).
}
\examples{
\donttest{
 USgdp
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{cusum}
\alias{cusum}
\title{cusum}
\usage{
cusum(y, runFromTest = FALSE)
}
\arguments{
\item{y}{a vector, ts or tsibble object}

\item{runFromTest}{internal check variable}
}
\value{
No return value, called for side effects
}
\description{
Cusum and cusumsq tests
}
\examples{
cusum(AirPassengers)
}
\seealso{
\code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
         \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
         \code{\link{varTest}}, \code{\link{conv}},
         \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
         \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
         \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
         \code{\link{size}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ETSS3functions.R
\name{auxInvBoxCox}
\alias{auxInvBoxCox}
\title{auxInvBoxCox}
\usage{
auxInvBoxCox(y, lambda)
}
\arguments{
\item{y}{matrix, array or vector}

\item{lambda}{lambda parameter of Box-Cox transformation}
}
\value{
Inverse of Box-Cox heteroskedasticity transformation
}
\description{
Inverse of Box-Cox transformation
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{Accuracy}
\alias{Accuracy}
\title{Accuracy}
\usage{
Accuracy(py, y, s = frequency(y), collectFun = mean)
}
\arguments{
\item{py}{matrix of forecasts (h x nMethods x nForecasts)}

\item{y}{a matrix of actual values (n x nForecasts)}

\item{s}{seasonal period, number of observations per year}

\item{collectFun}{aggregation function (mean, median, etc.)}
}
\value{
Table of accuracy results
}
\description{
Accuracy for 1 time series y and several forecasting
 methods py and h steps ahead py is h x nMethods x nSeries
}
\examples{
\dontrun{Accuracy(py, y, 12)}
}
\seealso{
\code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
         \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
         \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
         \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
         \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
         \code{\link{plotSlide}}, \code{\link{tsDisplay}},
         \code{\link{size}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TETScomponents.R
\name{TETScomponents}
\alias{TETScomponents}
\title{TETScomponents}
\usage{
TETScomponents(m)
}
\arguments{
\item{m}{an object of type \code{TETS} created with \code{TETSforecast}}
}
\value{
The same input object with the appropriate fields 
filled in, in particular:
\item{comp}{Estimated components in matrix form}
}
\description{
Estimates components of TOBIT TETS models
}
\examples{
\donttest{
m1 <- TETS(log(gdp))
m1 <- TETScomponents(m1)
}
}
\seealso{
\code{\link{TETS}}, \code{\link{TETSforecast}}, \code{\link{TETSvalidate}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{tests}
\alias{tests}
\title{tests}
\usage{
tests(
  y,
  parts = 1/3,
  nCoef = min(25, length(x)/4),
  nPar = 0,
  s = frequency(y),
  avoid = 16
)
}
\arguments{
\item{y}{a vector, ts or tsibble object}

\item{parts}{proportion of sample to include in ratio of variances test}

\item{nCoef}{number of autocorrelation coefficients to estimate}

\item{nPar}{number of parameters in a model if y is a residual}

\item{s}{seasonal period, number of observations per year}

\item{avoid}{number of observations to avoid at beginning of sample to
eliminate initial effects}
}
\value{
Table with all test results
}
\description{
Tests on a time series
}
\details{
Multiple tests on a time series, including summary statistics,
autocorrelation, Gaussianity and heteroskedasticity,
}
\examples{
tests(AirPassengers)
}
\seealso{
\code{\link{colMedians}}, \code{\link{rowMedians}},
         \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
         \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
         \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
         \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
         \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
         \code{\link{size}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ARIMAestim.R
\name{ARIMAestim}
\alias{ARIMAestim}
\title{ARIMAestim}
\usage{
ARIMAestim(m)
}
\arguments{
\item{m}{an object of type \code{ARIMA} created with \code{ARIMAforecast}}
}
\value{
The same input object with the appropriate fields 
filled in, in particular:
\item{p}{Estimated parameters}
\item{yFor}{Forecasted values of output}
\item{yForV}{Variance of forecasted values of output}
\item{ySimul}{Bootstrap simulations for forecasting distribution evaluation}
}
\description{
Estimates and forecasts ARIMA models
}
\details{
\code{ARIMAestim} estimates and forecasts a time series using 
an ARIMA model
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{ident}
\alias{ident}
\title{ident}
\usage{
ident(y, nCoef = min(37, floor(length(y)/4)), nPar = 0, runFromTests = FALSE)
}
\arguments{
\item{y}{a vector, ts or tsibble object}

\item{nCoef}{number of autocorrelation coefficients to estimate}

\item{nPar}{number of parameters in a model if y is a residual}

\item{runFromTests}{internal check}
}
\value{
A vector with output table including ACF, etc.
}
\description{
Autocorrelation functions of a time series
}
\examples{
ident(AirPassengers)
}
\seealso{
\code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
         \code{\link{sumStats}}, \code{\link{gaussTest}},
         \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
         \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
         \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
         \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
         \code{\link{size}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UCestim.R
\name{UCcommand}
\alias{UCcommand}
\title{UCcommand}
\usage{
UCcommand(command, sys)
}
\arguments{
\item{command}{Command to execute: "forecast", "validate", "filter", "smooth", "disturb", "components", "all"}

\item{sys}{A \code{UComp} object created with \code{UC}}
}
\value{
The input \code{UComp} object with the appropriate fields filled in
}
\description{
Auxiliar function for UC modeling
}
\examples{
\donttest{
cycle <- UChp(USgdp)
plot(cycle)
}
}
\seealso{
\code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, 
         \code{\link{UCsmooth}}, \code{\link{UCcomponents}}, \code{\link{UCdisturb}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ETSestim.R
\name{ETSestim}
\alias{ETSestim}
\title{ETSestim}
\usage{
ETSestim(m)
}
\arguments{
\item{m}{an object of type \code{ETS} created with \code{ETSforecast}}
}
\value{
The same input object with the appropriate fields 
filled in, in particular:
\item{p}{Estimated parameters}
\item{yFor}{Forecasted values of output}
\item{yForV}{Variance of forecasted values of output}
\item{ySimul}{Bootstrap simulations for forecasting distribution evaluation}
}
\description{
Estimates and forecasts ETS models
}
\details{
\code{ETSestim} estimates and forecasts a time series using an
an ETS model
}
\examples{
\donttest{
m1 <- ETSsetup(log(gdp))
m1 <- ETSestim(m1)
}
}
\seealso{
\code{\link{ETS}}, \code{\link{ETSforecast}}, \code{\link{ETSvalidate}},
         \code{\link{ETScomponents}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ETSvalidate.R
\name{ETSvalidate}
\alias{ETSvalidate}
\title{ETSvalidate}
\usage{
ETSvalidate(m)
}
\arguments{
\item{m}{an object of type \code{ETS} created with \code{ETSforecast}}
}
\value{
The same input object with the appropriate fields 
filled in, in particular:
\item{table}{Estimation and validation table}
}
\description{
Shows a table of estimation and diagnostics results for ETS models
}
\examples{
\donttest{
m1 <- ETSforecast(log(gdp))
m1 <- ETSvalidate(m1)
}
}
\seealso{
\code{\link{ETS}}, \code{\link{ETSforecast}}, \code{\link{ETSvalidate}},
         \code{\link{ETScomponents}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UCestim.R
\name{UChp}
\alias{UChp}
\title{UChp}
\usage{
UChp(y, lambda = 1600)
}
\arguments{
\item{y}{A time series object}

\item{lambda}{Smoothing constant (default: 1600)}
}
\value{
The cycle estimation
}
\description{
Hodrick-Prescott filter estimation
}
\examples{
\donttest{
cycle <- UChp(USgdp)
plot(cycle)
}
}
\seealso{
\code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, 
         \code{\link{UCsmooth}}, \code{\link{UCcomponents}}, \code{\link{UCdisturb}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{plotBar}
\alias{plotBar}
\title{plotBar}
\usage{
plotBar(ACF, s = 1, n = NA, label = "ACF")
}
\arguments{
\item{ACF}{variable to plot}

\item{s}{seasonal period}

\item{n}{number of coefficients}

\item{label}{label for plot}
}
\value{
Handle of plot
}
\description{
Plot variable in bars
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ch4}
\alias{ch4}
\title{Methane concentration at Cape Grim in Australia}
\format{
Time series objects.
\describe{
  Monthly data from January 1992 to December 2019
}
}
\source{
\href{https://www.csiro.au/en/research/natural-environment/atmosphere/Latest-greenhouse-gas-data}{CH4 data}
}
\usage{
ch4
}
\value{
No return value, called for side effects
}
\description{
Methane concentration at Cape Grim in Australia (ch4).
}
\examples{
\donttest{
 ch4
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TETSvalidate.R
\name{TETSvalidate}
\alias{TETSvalidate}
\title{TETSvalidate}
\usage{
TETSvalidate(m)
}
\arguments{
\item{m}{an object of type \code{TETS} created with \code{TETSforecast}}
}
\value{
The same input object with the appropriate fields 
filled in, in particular:
\item{table}{Estimation and validation table}
}
\description{
Shows a table of estimation and diagnostics results for TOBIT TETS models
}
\examples{
\donttest{
m1 <- TETSforecast(log(gdp))
m1 <- TETSvalidate(m1)
}
}
\seealso{
\code{\link{TETS}}, \code{\link{TETSforecast}}, \code{\link{TETSvalidate}},
         \code{\link{TETScomponents}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UCS3functions.R
\name{predict.UComp}
\alias{predict.UComp}
\title{predict.UComp}
\usage{
\method{predict}{UComp}(object, newdata = NULL, n.ahead = NULL, level = 0.95, ...)
}
\arguments{
\item{object}{Object of class \dQuote{UComp}.}

\item{newdata}{New output data to apply \dQuote{UComp} object to.}

\item{n.ahead}{Number of steps ahead to forecast or new inputs variables 
including their predictions.}

\item{level}{Confidence level for prediction intervals.}

\item{...}{Ignored.}
}
\value{
Forecasts of a UC model

A matrix with the mean forecasts and lower and upper prediction intervals
}
\description{
Forecasting using structural Unobseved Components models with prediction intervals
}
\details{
See help of \code{UC}.
}
\examples{
\donttest{
y <- log(AirPassengers)
m1 <- UCforecast(y, model = "llt/eq/arma(0,0)")
f1 <- predict(m1)
}
}
\seealso{
\code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, \code{\link{UCsmooth}}, 
         \code{\link{UCdisturb}}, \code{\link{UCcomponents}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ARIMAvalidate.R
\name{ARIMAvalidate}
\alias{ARIMAvalidate}
\title{ARIMAvalidate}
\usage{
ARIMAvalidate(m)
}
\arguments{
\item{m}{an object of type \code{ARIMA} created with \code{ARIMAforecast}}
}
\value{
The same input object with the appropriate fields 
filled in, in particular:
\item{table}{Estimation and validation table}
}
\description{
Shows a table of estimation and diagnostics results for ARIMA models
}
\examples{
\donttest{
m1 <- ARIMAforecast(log(gdp))
m1 <- ARIMAvalidate(m1)
}
}
\seealso{
\code{\link{ARIMA}}, \code{\link{ARIMAforecast}}, \code{\link{ARIMAvalidate}},
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{sumStats}
\alias{sumStats}
\title{sumStats}
\usage{
sumStats(y, decimals = 5)
}
\arguments{
\item{y}{a vector, matrix of time series}

\item{decimals}{number of decimals for table}
}
\value{
Table of values in string matrix
}
\description{
Summary statistics of a matrix of variables
}
\details{
Position, dispersion, skewness, kurtosis, etc.
}
\examples{
s <- sumStats(AirPassengers)
}
\seealso{
\code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
         \code{\link{gaussTest}}, \code{\link{ident}},
         \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
         \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
         \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
         \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
         \code{\link{size}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{slide}
\alias{slide}
\title{slide}
\usage{
slide(
  y,
  orig,
  forecFun,
  ...,
  h = 12,
  step = 1,
  output = TRUE,
  window = NA,
  parallel = FALSE
)
}
\arguments{
\item{y}{a vector, a matrix or a list of time series}

\item{orig}{starting forecasting origin}

\item{forecFun}{user function that implements forecasting methods}

\item{...}{rest of inputs to forecFun function}

\item{h}{forecasting horizon}

\item{step}{observations ahead to move the forecasting origin}

\item{output}{output TRUE/FALSE}

\item{window}{fixed window width in number of observations (NA for non fixed)}

\item{parallel}{run forecasts in parallel}
}
\value{
An array of forecasts of dimensions (horizon x nOrigs x nModels x nSeries)
}
\description{
Rolling forecasting of a matrix of time series
}
\details{
Takes time series and run forecasting methods implemented in function
forecFun h steps ahead along the time series y, starting at forecasting
origin orig, and moving step observations ahead. Forecasts may be run in parallel
by setting parallel to TRUE. A fixed window width may be
specified with input window. The output is of dimensions (h, nOrigs, nModels, nSeries)
}
\examples{
\dontrun{slide(AirPassengers, 100, forecFun)}
}
\seealso{
\code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
         \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
         \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
         \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
         \code{\link{zplane}}, \code{\link{acft}},
         \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
         \code{\link{size}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UCestim.R
\name{UCvalidate}
\alias{UCvalidate}
\title{UCvalidate}
\usage{
UCvalidate(sys, printScreen = TRUE)
}
\arguments{
\item{sys}{an object of type \code{UComp} created with \code{UC}}

\item{printScreen}{print to screen or just return output table}
}
\value{
The same input object with the appropriate fields 
filled in, in particular:
\itemize{
\item table: Estimation and validation table
}
}
\description{
Shows a table of estimation and diagnostics results for UC models.
Equivalent to print or summary.
The table shows information in four sections:
Firstly, information about the model estimated, the relevant 
periods of the seasonal component included, and further information about
convergence.
Secondly, parameters with their names are provided, the asymptotic standard errors, 
the ratio of the two, and the gradient at the optimum. One asterisk indicates 
concentrated-out parameters and two asterisks signals parameters constrained during estimation.
Thirdly, information criteria and the value of the log-likelihood.
Finally, diagnostic statistics about innovations, namely, the Ljung-Box Q test of absense
of autocorrelation statistic for several lags, the Jarque-Bera gaussianity test, and a
standard ratio of variances test.
}
\examples{
\donttest{
m1 <- UC(log(gdp))
m1 <- UCvalidate(m1)
}
}
\seealso{
\code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCfilter}}, 
         \code{\link{UCsmooth}}, \code{\link{UCdisturb}}, \code{\link{UCcomponents}},
         \code{\link{UChp}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UCestim.R
\name{UCcomponents}
\alias{UCcomponents}
\title{UCcomponents}
\usage{
UCcomponents(sys)
}
\arguments{
\item{sys}{an object of type \code{UComp} created with \code{UC} or \code{UCforecast}}
}
\value{
The same input object with the appropriate fields 
filled in, in particular:
\itemize{
\item comp:  Estimated components in matrix form
\item compV: Estimated components variance in matrix form
}
}
\description{
Estimates unobserved components of UC models
Standard methods applicable to UComp objects are print, summary, plot,
fitted, residuals, logLik, AIC, BIC, coef, predict, tsdiag.
}
\examples{
\donttest{
m1 <- UC(log(AirPassengers))
m1 <- UCcomponents(m1)
}
}
\seealso{
\code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, \code{\link{UCfilter}}, 
         \code{\link{UCsmooth}}, \code{\link{UCdisturb}},
         \code{\link{UChp}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{plus_one}
\alias{plus_one}
\title{plus_one}
\usage{
plus_one(y)
}
\arguments{
\item{y}{a ts object}
}
\value{
Next time stamp
}
\description{
Returns date of next to end time series y
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UCestim.R
\name{UCfilter}
\alias{UCfilter}
\title{UCfilter}
\usage{
UCfilter(sys)
}
\arguments{
\item{sys}{an object of type \code{UComp} created with \code{UC}}
}
\value{
The same input object with the appropriate fields 
filled in, in particular:
\itemize{
\item yFit:  Fitted values of output
\item yFitV: Variance of fitted values of output
\item a:     State estimates
\item P:     Variance of state estimates (diagonal of covariance matrices)
}
}
\description{
Runs the Kalman Filter for UC models
Standard methods applicable to \code{UComp} objects are print, summary, plot,
fitted, residuals, logLik, AIC, BIC, coef, predict, tsdiag.
}
\examples{
\donttest{
m1 <- UC(log(AirPassengers))
m1 <- UCfilter(m1)
}
}
\seealso{
\code{\link{UC}}, \code{\link{UCforecast}}, \code{\link{UCvalidate}}, 
         \code{\link{UCsmooth}}, \code{\link{UCdisturb}}, \code{\link{UCcomponents}},
         \code{\link{UChp}}
}
\author{
Diego J. Pedregal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{conv}
\alias{conv}
\title{conv}
\usage{
conv(...)
}
\arguments{
\item{...}{list of vectors to convolute}
}
\value{
Convolution of all input vectors
}
\description{
1D convolution: filtering or polynomial multiplication
}
\examples{
conv(c(1, -1), c(1, -2, 1))
conv(c(1, -1), c(1, 0.8))
}
\seealso{
\code{\link{colMedians}}, \code{\link{rowMedians}}, \code{\link{tests}},
         \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
         \code{\link{cusum}}, \code{\link{varTest}},
         \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
         \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
         \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
         \code{\link{size}}
}
\author{
Diego J. Pedregal
}
